#!/bin/sh
#----------------------------------------------------------------------------
# eisfair-ng configuration generator script for WEBALIZER
# Copyright (c) 2007 - 2013 the eisfair team, team(at)eisfair(dot)org
#----------------------------------------------------------------------------

#exec 2> `pwd`/webalizer-trace$$.log
#set -x

# include config
. /etc/config.d/webalizer
. /etc/config.d/apache2

# set package name
packageName=webalizer


# ----------------------------------------------------------------------------
# create a config file
# ----------------------------------------------------------------------------
create_webalizer_configuration ()
{
echo "#------------------------------------------------------------------------------"
echo "# Webalizer configuration file generated by Eisfair setup"
echo "# DO NOT EDIT THIS FILE! Edit /etc/config.d/webalizer"
echo "#------------------------------------------------------------------------------"
echo ""

# LogFile defines the web server log file to use.  If not specified
# here or on on the command line, input will default to STDIN.  If
# the log filename ends in '.gz' (a gzip compressed file), or '.bz2'
# (bzip2 compressed file), it will be decompressed on the fly as it
# is being read.
#LogFile        /var/www/logs/access.log
echo "LogFile         $accesslog"

# LogType defines the log type being processed.  Normally, the Webalizer
# expects a CLF or Combined web server log as input.  Using this option,
# you can process ftp logs (xferlog as produced by wu-ftp and others),
# Squid native logs or W3C extended format web logs. Values can be 'clf',
# 'ftp', 'squid' or 'w3c'.  The default is 'clf'.
#LogType    clf
echo "LogType         $type"

# OutputDir is where you want to put the output files.  This should
# should be a full path name, however relative ones might work as well.
# If no output directory is specified, the current directory will be used.
#OutputDir      /var/lib/httpd/htdocs/usage
echo "OutputDir       $output_dir"

# HistoryName allows you to specify the name of the history file produced
# by the Webalizer.  The history file keeps the data for previous months,
# and is used for generating the main HTML page (index.html). The default
# is a file named "webalizer.hist", stored in the output directory being
# used.  The name can include a path, which will be relative to the output
# directory unless absolute (starts with a leading '/').
#HistoryName    webalizer.hist
echo "HistoryName     webalizer.hist"

# Incremental processing allows multiple partial log files to be used
# instead of one huge one.  Useful for large sites that have to rotate
# their log files more than once a month.  The Webalizer will save its
# internal state before exiting, and restore it the next time run, in
# order to continue processing where it left off.  This mode also causes
# The Webalizer to scan for and ignore duplicate records (records already
# processed by a previous run).  See the README file for additional
# information.  The value may be 'yes' or 'no', with a default of 'no'.
# The file 'webalizer.current' is used to store the current state data,
# and is located in the output directory of the program (unless changed
# with the IncrementalName option below).  Please read at least the section
# on Incremental processing in the README file before you enable this option.
#Incremental    no
echo "Incremental     yes"

# IncrementalName allows you to specify the filename for saving the
# incremental data in.  It is similar to the HistoryName option where the
# name is relative to the specified output directory, unless an absolute
# filename is specified.  The default is a file named "webalizer.current"
# kept in the normal output directory.  If you don't specify "Incremental"
# as 'yes' then this option has no meaning.
#IncrementalName    webalizer.current
echo "IncrementalName webalizer.current"

# ReportTitle is the text to display as the title.  The hostname
# (unless blank) is appended to the end of this string (seperated with
# a space) to generate the final full title string.
# Default is (for english) "Usage Statistics for".
#ReportTitle    Usage Statistics for
echo "ReportTitle     $title"

# HostName defines the hostname for the report.  This is used in
# the title, and is prepended to the URL table items.  This allows
# clicking on URLs in the report to go to the proper location in
# the event you are running the report on a 'virtual' web server,
# or for a server different than the one the report resides on.
# If not specified here, or on the command line, webalizer will
# try to get the hostname via a uname system call.  If that fails,
# it will default to "localhost".
#HostName   www.webalizer.org
echo "HostName        $host_name"

# HTMLExtension allows you to specify the filename extension to use
# for generated HTML pages.  Normally, this defaults to "html", but
# can be changed for sites who need it (like for PHP embeded pages).
#HTMLExtension  html
echo "HTMLExtension   html"

# PageType lets you tell the Webalizer what types of URLs you
# consider a 'page'.  Most people consider html and cgi documents
# as pages, while not images and audio files.  If no types are
# specified, defaults will be used ('htm*', 'cgi' and HTMLExtension
# if different for web logs, 'txt' for ftp logs).
#PageType    htm*
#PageType    cgi
#PageType   phtml
#PageType   php3
#PageType   pl
cat << EOF
PageType        htm*
PageType        cgi
PageType        asp
PageType        jsp
PageType        pdf
PageType        phtml
PageType        php
PageType        php3
PageType        php4
PageType        pl
PageType        py
PageType        shtml
PageType        txt

EOF

# PagePrefix allows all requests with a specified prefix to be
# considered as 'pages'. If you want everything under /documents
# to be treated as pages no matter what their extension is. Also
# useful if you have cgi-scripts with PATH_INFO.
#PagePrefix /documents
#PagePrefix /mycgi/parameters

# OmitPage lets you tell the Webalizer that certain URLs do not
# contain any pages.  No URL matching an OmitPage value will be
# counted as a page, even if it matches a PageType above or has
# no extension (e.g., a directory).  They will still be counted
# as a hit.
#OmitPage   /render

# UseHTTPS should be used if the analysis is being run on a
# secure server, and links to urls should use 'https://' instead
# of the default 'http://'.  If you need this, set it to 'yes'.
# Default is 'no'.  This only changes the behaviour of the 'Top
# URLs' table.
#UseHTTPS       no
echo "UseHTTPS        no"

# HTAccess allows the generation of a default .htaccess file in the
# output directory.  If enabled, a default .htaccess file will be
# created (with a single "DirectoryIndex" directive), unless one
# already exists.  Values may be 'yes' or 'no', with 'no'
# being the default (don't write .htaccess files).
#HTAccess   no

# StripCGI determines if URL CGI variables should be striped or not.
# Historically, the Webalizer stripped all CGI variables from the end
# of URLs to improve accuracy.  Some sites may prefer to keep the CGI
# variables in place, particularly those with highly dynamic pages.
# Values may be 'yes' or 'no', with the default being 'yes'.
#StripCGI   yes

# The TrimSquidURL option only has effect on squid type log files.
# When analyzing a squid log, it is usually desirable to have less
# granularity on the URLs.  TrimSquidURL = n where n is a number > 0
# causes all URLs to be truncated after the nth '/' after the http://
# portion.  Setting TrimSquidURL to one (1) will cause all URLs to be
# summarized by domain only.  The default is zero (0), which disables
# any such truncation and preserve the URLs as they are in the log.
# TrimSquidURL  0

# DNSCache specifies the DNS cache filename to use for reverse DNS lookups.
# This file must be specified if you wish to perform name lookups on any IP
# addresses found in the log file.  If an absolute path is not given as
# part of the filename (ie: starts with a leading '/'), then the name is
# relative to the default output directory.  See the DNS.README file for
# additional information.
#DNSCache   dns_cache.db
#echo "DNSCache        /var/lib/webalizer/dns_cache.db"

# DNSChildren allows you to specify how many "children" processes are
# run to perform DNS lookups to create or update the DNS cache file.
# If a number is specified, the DNS cache file will be created/updated
# each time the Webalizer is run, immediately prior to normal processing,
# by running the specified number of "children" processes to perform
# DNS lookups.  If used, the DNS cache filename MUST be specified as
# well.  The default value is zero (0), which disables DNS cache file
# creation/updates at run time.  The number of children processes to
# run may be anywhere from 1 to 100, however a large number may effect
# normal system operations.  Reasonable values should be between 5 and
# 20.  See the DNS.README file for additional information.
#DNSChildren    0
#echo "DNSChildren        0"

# CacheIPs allows unresolved IP addresses to be cached in the DNS
# database.  Normally, only resolved addresses are saved.  At some
# sites, particularly those with a large number of unresolvable IP
# addresses visiting, it may be useful to enable this feature so
# those addresses are not constantly looked up each time the program
# is run.  Values can be 'yes' or 'no', with 'no' being the default.
#CacheIPs   no

# CacheTTL specifies the time to live (TTL) value for cached DNS
# entries, in days.  This value may be anywhere between 1 and 100
# with the default being 7 days (1 week).
#CacheTTL   7

# The GeoDB option enables or disabled the use of the native
# Webalizer GeoDB geolocation services.  This is the preferred
# geolocation option.  Values may be 'yes' or 'no', with 'no'
# being the default.
#GeoDB      no
echo "GeoDB           no"

# GeoDBDatabase specifies an alternate database to use.  The
# default database is /usr/share/GeoDB/GeoDB.dat (however the
# path may be changed at compile time; use the -vV command
# line option to determine where).  If a different database is
# to be used, it may be specified here.  The name is relative
# to the output directory being used unless an absolute name
# (ie: starts with a leading '/') is specified.
#GeoDBDatabase  /usr/share/GeoDB/GeoDB.dat

# The GeoIP option enables or disables the use of geolocation
# services provided by the GeoIP library (http://www.maxmind.com),
# if available.  Values may be 'yes' or 'no, with 'no' being the
# default.  Note: if GeoDB is enabled, then this option will have
# no effect (GeoDB will be used regardless of this setting).
echo "GeoIP           yes"

# GeoIPDatabase specifies an alternate database filename to use by the
# GeoIP library.  If an absolute path is not given as part of the name
# (ie: starts with a leading '/'), then the name is relative to the
# default output directory. This option should not normally be needed.
echo "GeoIPDatabase   /usr/share/GeoIP/GeoIP.dat"

# HTMLPre defines HTML code to insert at the very beginning of the
# file.  Default is the DOCTYPE line shown below.  Max line length
# is 80 characters, so use multiple HTMLPre lines if you need more.
#HTMLPre <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
echo 'HTMLPre         <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">'

# HTMLHead defines HTML code to insert within the <HEAD></HEAD>
# block, immediately after the <TITLE> line.  Maximum line length
# is 80 characters, so use multiple lines if needed.
#HTMLHead <META NAME="author" CONTENT="The Webalizer">
#HTMLHead <META NAME="ROBOTS" CONTENT="NOINDEX, NOFOLLOW">
echo 'HTMLHead        <META NAME="author" CONTENT="The Webalizer">'
echo 'HTMLHead        <META NAME="ROBOTS" CONTENT="NOINDEX, NOFOLLOW">'

# HTMLBody defined the HTML code to be inserted, starting with the
# <BODY> tag.  If not specified, the default is shown below.  If
# used, you MUST include your own <BODY> tag as the first line.
# Maximum line length is 80 char, use multiple lines if needed.
#HTMLBody <BODY BGCOLOR="#E8E8E8" TEXT="#000000" LINK="#0000FF" VLINK="#FF0000">
echo 'HTMLBody        <BODY BGCOLOR="'$bgcolor'" TEXT="#000000" LINK="#000000" VLINK="#000000">'

# HTMLPost defines the HTML code to insert immediately before the
# first <HR> on the document, which is just after the title and
# "summary period"-"Generated on:" lines.  If anything, this should
# be used to clean up in case an image was inserted with HTMLBody.
# As with HTMLHead, you can define as many of these as you want and
# they will be inserted in the output stream in order of apperance.
# Max string size is 80 characters.  Use multiple lines if you need to.
#HTMLPost   <BR CLEAR="all">
echo 'HTMLPost        <BR CLEAR="all">'

# HTMLTail defines the HTML code to insert at the bottom of each
# HTML document, usually to include a link back to your home
# page or insert a small graphic.  It is inserted as a table
# data element (ie: <TD> your code here </TD>) and is right
# alligned with the page.  Max string size is 80 characters.
#HTMLTail <IMG SRC="msfree.png" ALT="100% Micro$oft free!">
echo 'HTMLTail        Generated on <a href="http://www.eisfair.org"><b>eisfair</b></a>'

# HTMLEnd defines the HTML code to add at the very end of the
# generated files.  It defaults to what is shown below.  If
# used, you MUST specify the </BODY> and </HTML> closing tags
# as the last lines.  Max string length is 80 characters.
#HTMLEnd </BODY></HTML>
echo "HTMLEnd         </BODY></HTML>"

# The LinkReferrer option determines if entries in the referrer table
# should be plain text or a HTML link to the referrer.  Values can be
# either 'yes' or 'no', with 'no' being the default.
#LinkReferrer   no

# The Quiet option suppresses output messages... Useful when run
# as a cron job to prevent bogus e-mails.  Values can be either
# "yes" or "no".  Default is "no".  Note: this does not suppress
# warnings and errors (which are printed to stderr).
#Quiet      no
echo "Quiet           yes"

# ReallyQuiet will supress all messages including errors and
# warnings.  Values can be 'yes' or 'no' with 'no' being the
# default.  If 'yes' is used here, it cannot be overriden from
# the command line, so use with caution.  A value of 'no' has
# no effect.
#ReallyQuiet    no
echo "ReallyQuiet     no"

# TimeMe allows you to force the display of timing information
# at the end of processing.  A value of 'yes' will force the
# timing information to be displayed.  A value of 'no' has no
# effect.
#TimeMe     no
echo "TimeMe          yes"

# GMTTime allows reports to show GMT (UTC) time instead of local
# time.  Default is to display the time the report was generated
# in the timezone of the local machine, such as EDT or PST.  This
# keyword allows you to have times displayed in UTC instead.  Use
# only if you really have a good reason, since it will probably
# screw up the reporting periods by however many hours your local
# time zone is off of GMT.
#GMTTime    no
echo "GMTTime         no"

# Debug prints additional information for error messages.  This
# will cause webalizer to dump bad records/fields instead of just
# telling you it found a bad one.   As usual, the value can be
# either "yes" or "no".  The default is "no".  It shouldn't be
# needed unless you start getting a lot of Warning or Error
# messages and want to see why.  (Note: warning and error messages
# are printed to stderr, not stdout like normal messages).
#Debug      no
echo "Debug           no"

# FoldSeqErr forces the Webalizer to ignore sequence errors.
# This is useful for Netscape and other web servers that cache
# the writing of log records and do not guarentee that they
# will be in chronological order.  The use of the FoldSeqErr
# option will cause out of sequence log records to be treated
# as if they had the same time stamp as the last valid record.
# Default is to ignore out of sequence log records.  The use
# of this feature is strongly discouraged and rarely needed.
# (the webalizer already compensates for up to 60 minutes of
# difference between records).
#FoldSeqErr no
echo "FoldSeqErr      no"

# VisitTimeout allows you to set the default timeout for a visit
# (sometimes called a 'session').  The default is 30 minutes,
# which should be fine for most sites.
# Visits are determined by looking at the time of the current
# request, and the time of the last request from the site.  If
# the time difference is greater than the VisitTimeout value, it
# is considered a new visit, and visit totals are incremented.
# Value is the number of seconds to timeout (default=1800=30min)
#VisitTimeout   1800
echo "VisitTimeout    1800"

# IgnoreHist shouldn't be used in a config file, but it is here
# just because it might be usefull in certain situations.  If the
# history file is ignored, the main "index.html" file will only
# report on the current log files contents.  Usefull only when you
# want to reproduce the reports from scratch.  USE WITH CAUTION!
# Valid values are "yes" or "no".  Default is "no".
#IgnoreHist no
echo "IgnoreHist      no"

# IgnoreState also shouldn't be used, but is here anyway.  It is
# similar to the IgnoreHist option, but for the incremental data
# file.  If this is set to 'yes', any existing incrememtal data
# will be ignored and a new data file will be written at the end
# of processing.  USE WITH CAUTION.  By ignoring an existing
# incremental data file, all previous processing for the current
# month will be lost, and those logs must be re-processed.
# Valid values are "yes" or "no".  Default is "no".
#IgnoreState    no

# CountryGraph allows the usage by country graph to be disabled.
# Values can be 'yes' or 'no', default is 'yes'.
#CountryGraph   yes
echo "CountryGraph    yes"

# CountryFlags allows flags to be displayed in the top country
# table in monthly reports.  Values can be 'yes' or 'no', with
# the default being 'no'.
#CountryFlags   no

# FlagDir specifies the location of flag graphics which will be
# used in the top country table.  If not specified, the default
# is to look in the 'flags' directory directly under the output
# directory being used for the reports.  If this option is used,
# the display of flag graphics will be enabled by default.
#FlagDir    flags

# DailyGraph and DailyStats allows the daily statistics graph
# and statistics table to be disabled (not displayed).  Values
# may be "yes" or "no". Default is "yes".
#DailyGraph yes
#DailyStats yes
echo "DailyGraph      yes"
echo "DailyStats      yes"

# HourlyGraph and HourlyStats allows the hourly statistics graph
# and statistics table to be disabled (not displayed).  Values
# may be "yes" or "no". Default is "yes".
#HourlyGraph    yes
#HourlyStats    yes
echo "HourlyGraph     yes"
echo "HourlyStats     yes"

# GraphLegend allows the color coded legends to be turned on or off
# in the graphs.  The default is for them to be displayed.  This only
# toggles the color coded legends, the other legends are not changed.
# If you think they are hideous and ugly, say 'no' here :)
#GraphLegend    yes
echo "GraphLegend     yes"

# GraphLines allows you to have index lines drawn behind the graphs.
# I personally am not crazy about them, but a lot of people requested
# them and they weren't a big deal to add.  The number represents the
# number of lines you want displayed.  Default is 2, you can disable
# the lines by using a value of zero ('0').  [max is 20]
# Note, due to rounding errors, some values don't work quite right.
# The lower the better, with 1,2,3,4,6 and 10 producing nice results.
#GraphLines 2
echo "GraphLines      6"

# IndexMonths defines the number of months to display in the main index
# (yearly summary) table.  Value can be between 12 and 120, with the
# default being 12 months (1 year).
#IndexMonths    12
echo "IndexMonths     60"

# YearHeaders enables/disables the display of year headers in the main
# index (yearly summary) table.  If enabled, year headers will be shown
# when the table is displaying more than 16 months worth of data. Values
# can be 'yes' or 'no', with 'yes' being the default.
#YearHeaders    yes
echo "YearHeaders     yes"

# YearTotals enables/disables the display of yearly totals in the main
# index (yearly summary) table.  If enabled, year totals will be shown
# when the table is displaying more than 16 months worth of data.  Values
# can be 'yes' or 'no', with 'yes' being the default.
#YearTotals yes
echo "YearTotals      no"

# GraphMonths defines the number of months to display in the main index
# (yearly summary) graph.  Value can be between 12 and 72 months, with
# the default being 12 months.
#GraphMonths    12
echo "GraphMonths     60"

# The "Top" options below define the number of entries for each table.
# Defaults are Sites=30, URLs=30, Referrers=30 and Agents=15, and
# Countries=30. TopKSites and TopKURLs (by KByte tables) both default
# to 10, as do the top entry/exit tables (TopEntry/TopExit).  The top
# search strings and usernames default to 20.  Tables may be disabled
# by using zero (0) for the value.
#TopSites        30
#TopKSites       10
#TopURLs         30
#TopKURLs        10
#TopReferrers    30
#TopAgents       15
#TopCountries    30
#TopEntry        10
#TopExit         10
#TopSearch       20
#TopUsers        20
echo "TopSites        15"
echo "TopKSites       30"
echo "TopURLs         10"
echo "TopKURLs        30"
echo "TopReferrers    5"
echo "TopAgents       160"
echo "TopCountries    30"
echo "TopEntry        10"
echo "TopExit         10"
echo "TopSearch       20"
echo "TopUsers        20"

# The All* keywords allow the display of all URLs, Sites, Referrers
# User Agents, Search Strings and Usernames.  If enabled, a seperate
# HTML page will be created, and a link will be added to the bottom
# of the appropriate "Top" table.  There are a couple of conditions
# for this to occur..  First, there must be more items than will fit
# in the "Top" table (otherwise it would just be duplicating what is
# already displayed).  Second, the listing will only show those items
# that are normally visable, which means it will not show any hidden
# items.  Grouped entries will be listed first, followed by individual
# items.  The value for these keywords can be either 'yes' or 'no',
# with the default being 'no'.  Please be aware that these pages can
# be quite large in size, particularly the sites page,  and seperate
# pages are generated for each month, which can consume quite a lot
# of disk space depending on the traffic to your site.
#AllSites   no
#AllURLs    no
#AllReferrers   no
#AllAgents  no
#AllSearchStr   no
#AllUsers       no
echo "AllSites        yes"
echo "AllURLs         yes"
echo "AllReferrers    yes"
echo "AllAgents       yes"
echo "AllSearchStr    yes"
echo "AllUsers        yes"

# The Webalizer normally strips the string 'index.' off the end of
# URLs in order to consolidate URL totals.  For example, the URL
# /somedir/index.html is turned into /somedir/ which is really the
# same URL.  This option allows you to specify additional strings
# to treat in the same way.  You don't need to specify 'index.' as
# it is always scanned for by The Webalizer, this option is just to
# specify _additional_ strings if needed.  If you don't need any,
# don't specify any as each string will be scanned for in EVERY
# log record... A bunch of them will degrade performance.  Also,
# the string is scanned for anywhere in the URL, so a string of
# 'home' would turn the URL /somedir/homepages/brad/home.html into
# just /somedir/ which is probably not what was intended.
#IndexAlias     home.htm
#IndexAlias homepage.htm

# The DefaultIndex option is used to enable/disable the use of
# "index." as the default index name to be stripped off the end of
# a URL (as described above).  Most sites will not need to use this
# option, but some may, such as those whose default index file name
# is different, or those that use "index.php" or similar URLs in a
# dynamic environment.  Values can be 'yes' or 'no', with the default
# being 'yes'.  This option does not effect any names added using the
# IndexAlias option, and those names will still function as described
# regardless of this setting.
#DefaultIndex   yes

# The Hide*, Group* and Ignore* and Include* keywords allow you to
# change the way Sites, URLs, Referrers, User Agents and Usernames
# are manipulated.  The Ignore* keywords will cause The Webalizer to
# completely ignore records as if they didn't exist (and thus not
# counted in the main site totals).  The Hide* keywords will prevent
# things from being displayed in the 'Top' tables, but will still be
# counted in the main totals.  The Group* keywords allow grouping
# similar objects as if they were one.  Grouped records are displayed
# in the 'Top' tables and can optionally be displayed in BOLD and/or
# shaded. Groups cannot be hidden, and are not counted in the main
# totals. The Group* options do not, by default, hide all the items
# that it matches.  If you want to hide the records that match (so just
# the grouping record is displayed), follow with an identical Hide*
# keyword with the same value.  (see example below)  In addition,
# Group* keywords may have an optional label which will be displayed
# instead of the keywords value.  The label should be seperated from
# the value by at least one 'white-space' character, such as a space
# or tab.  If the match string contains whitespace (spaces or tabs),
# the string should be quoted with either single or double quotes.
#
# The value can have either a leading or trailing '*' wildcard
# character.  If no wildcard is found, a match can occur anywhere
# in the string. Given a string "www.yourmama.com", the values "your",
# "*mama.com" and "www.your*" will all match.
# Your own site should be hidden
#HideSite   *webalizer.org
#HideSite   localhost
echo "HideSite        localhost"

# Your own site gives most referrals
#HideReferrer   webalizer.org/
echo "HideReferrer    localhost"

# This one hides non-referrers ("-" Direct requests)
#HideReferrer   Direct Request

# Usually you want to hide these
echo "HideURL         *.gif"
echo "HideURL         *.GIF"
echo "HideURL         *.jpg"
echo "HideURL         *.JPG"
echo "HideURL         *.png"
echo "HideURL         *.PNG"
echo "HideURL         *.ra"

echo "HideURL         *.css"
echo "HideURL         *.ico"

# Hiding agents is kind of futile
#HideAgent        RealPlayer

# You can also hide based on authenticated username
#HideUser        root
#HideUser        admin

# Grouping options
#GroupURL        /cgi-bin/*        CGI Scripts
#GroupURL        /images/*         Images
#GroupURL        /icons/*          Icons

cat << EOF
GroupSite       192.168*    Internal Network
HideSite        192.168*
#--------------------------------------------------------------------------
GroupSite	202.131.4.		Datacom (no website) MN
HideSite	202.131.4.
GroupSite	202.131.5.		Datacom (no website) MN
HideSite	202.131.5.
GroupSite	202.131.6.		Datacom (no website) MN
HideSite	202.131.6.
GroupSite	64.242.*		LookSmart_Crawler
HideSite	64.242.*
GroupSite	65.54.*			MSNBot
HideSite	65.54.*
GroupSite	66.147.154.*		IBM_Almaden
HideSite	66.147.154.*
GroupSite	66.249.*		GoogleBot
HideSite	66.249.*
GroupSite	68.142.*		Yahoo_Slurp
HideSite	68.142.*
GroupSite	216.239.36.*		GoogleBot_Translation
HideSite	216.239.36.*
GroupSite	216.239.38.*		GoogleBot_Translation
HideSite	216.239.38.*
GroupSite	*.pool.007mundo.com	<a href=http://www.007mundo.com>ISP: ETB CO</a>
HideSite	*.pool.007mundo.com
GroupSite	*.012.net.il		<a href=http://www.012.net.il>ISP: 012.net</a>
HideSite	*.012.net.il
GroupSite	*.9tel.net		<a href=http://www.9telecom.fr>ISP: 9telecom FR</a>
HideSite	*.9tel.net
GroupSite	*.a2000.nl		<a href=http://www.a2000.nl>ISP: UPC NL</a>
HideSite	*.a2000.nl
GroupSite	*.abb.com		<a href=http://www.abb.com>ABB Group **</a>
HideSite	*.abb.com
GroupSite	*.able.es		<a href=http://www.able.es>ISP: AUNA ES</a>
HideSite	*.able.es
GroupSite	proxy.able.es*		<a href=http://www.able.es>ISP: AUNA ES</a>
HideSite	proxy.able.es*
GroupSite	*.abo.fi		<a href=http://www.abo.fi>EDU: Abo Akademi FI</a>
HideSite	*.abo.fi
GroupSite	*.absamail.co.za	<a href=http://www.absamail.co.za>ISP: Absa ZA</a>
HideSite	*.absamail.co.za
GroupSite	*.acelerate.com		<a href=http://www.acelerate.com>ISP: Acelerate BO</a>
HideSite	*.acelerate.com
GroupSite	*.acelerate.net		<a href=http://www.acelerate.com>ISP: Acelerate BO</a>
HideSite	*.acelerate.net
GroupSite	*.actcom.co.il		<a href=http://www.actcom.co.il>ISP: Actom IL</a>
HideSite	*.actcom.co.il
GroupSite	*.ac-bordeaux.fr	<a href=http://www.ac-bordeaux.fr>EDU: Acad. Bordeaux FR</a>
HideSite	*.ac-bordeaux.fr
GroupSite	*.ad.jp			<a href=http://www.ocn.ne.jp>Spider: OCN</a>
HideSite	*.ad.jp
GroupSite	*.adslplus.ch		ISP: adslplus.ch (no website) CH
HideSite	*.adslplus.ch
GroupSite	*.acnielsen.co.uk	<a href=http://www.acnielsen.co.uk>AC Nielsen</a>
HideSite	*.acnielsen.co.uk
GroupSite	*.adam.es		<a href=http://www.adam.es>ISP: Adam ES</a>
HideSite	*.adam.es
GroupSite	*.adelphia.net		<a href=http://www.adelphia.net>ISP: Adelphia US</a>
HideSite	*.adelphia.net
GroupSite	*.adinet.com.uy		<a href=http://www.adinet.com.uy>Adinet Webmail UY</a>
HideSite	*.adinet.com.uy
GroupSite	*.adobe.com		<a href=http://www.adobe.com>Adobe US</a>
HideSite	*.adobe.com
GroupSite	*.advance.com.ar	<a href=http://www.advance.com.ar>ISP: Advance AR</a>
HideSite	*.advance.com.ar
GroupSite	*.afp.com		<a href=http://www.afp.com>ISP: AFP FR</a>
HideSite	*.afp.com
GroupSite	*.agh.edu.pl		<a href=http://www.agh.edu.pl>EDU: Gorniczo-Hutnizca PL</a>
HideSite	*.agh.edu.pl
GroupSite	*.aimit.nu		<a href=http://www.aimit.nu>ISP: Aimit NU</a>
HideSite	*.aimit.nu
GroupSite	*.airtel.net		<a href=http://www.airtel.net>ISP: Airtel (Vodafone) ES</a>
HideSite	*.airtel.net
GroupSite	*.ajato.com.br		<a href=http://www.ajato.com.br>ISP: Ajato BR</a>
HideSite	*.ajato.com.br
GroupSite	*.albany.edu		<a href=http://www.albany.edu>EDU: U. Albany US</a>
HideSite	*.albany.edu
GroupSite	*.alestra.net.mx	<a href=http://www.alestra.net.mx>ISP: alestra AT&amp;T MX</a>
HideSite	*.alestra.net.mx
GroupSite	*.alianzaviva.net	<a href=http://www.psi.net.pa>ISP: Alianza Viva PA</a>
HideSite	*.alianzaviva.net
GroupSite	*.alicedsl.de		<a href=http://www.alice-dsl.de>ISP: Alice DE</a>
HideSite	*.alicedsl.de
GroupSite	*.alliedadvpub.com	<a href=http://www.alliedadvpub.com>Allied Advertising US</a>
HideSite	*.alliedadvpub.com
GroupSite	*.amigo.net.gt		<a href=http://www.amigo.net.gt>ISP: Amigo Net GT</a>
HideSite	*.amigo.net.gt
GroupSite	*.ameritech.net		<a href=http://www.ameritech.net>ISP: SBC</a>
HideSite	*.ameritech.net
GroupSite	*.anaya.es		<a href=http://www.anaya.es>EDU: Grupo Anaya ES</a>
HideSite	*.anaya.es
GroupSite	*.andorpac.ad		<a href=http://www.andorpac.ad>ISP: Andorpac AD</a>
HideSite	*.andorpac.ad
GroupSite	*.antel.net.uy		<a href=http://www.antel.com.uy>ISP: ANTEL UY</a>
HideSite	*.antel.net.uy
GroupSite	*.anteldata.net.uy	<a href=http://www.antel.com.uy>ISP: ANTEL UY</a>
HideSite	*.anteldata.net.uy
GroupSite	*.alerondial.net	<a href=http://www.alerondial.net>ISP: Alerondial US</a>
HideSite	*.alerondial.net
GroupSite	*.alestra.net.mx	<a href=http://www.alestra.net.mx>ISP: alestra AT&amp;T MX</a>
HideSite	*.alestra.net.mx
GroupSite	*.alexa.com		<a href=http://www.alexa.com>Spider: Alexa Amazon US</a>
HideSite	*.alexa.com
GroupSite	*.alltel.net		<a href=http://www.alltel.net>ISP: Alltel US</a>
HideSite	*.alltel.net
GroupSite	*.apple.com		<a href=http://www.apple.com>Apple US</a>
HideSite	*.apple.com
GroupSite	*.archive.org		<a href=http://www.archive.org>Spider: archive.org **</a>
HideSite	*.archive.org
GroupSite	*.aol.com		<a href=http://www.aol.com>ISP: AOL **</a>
HideSite	*.aol.com
GroupSite	*.aragob.es		<a href=http://www.aragob.es>GOV: Aragon ES</a>
HideSite	*.aragob.es
GroupSite	*.arcor-ip.de		<a href=http://www.arcor.de>ISP: Arcor DE</a>
HideSite	*.arcor-ip.de
GroupSite	*.arcor-ip.net		<a href=http://www.arcor.de>ISP: Arcor DE</a>
HideSite	*.arcor-ip.net
GroupSite	*.army.mil		Military army (no website) US)
HideSite	*.army.mil
GroupSite	*.arnet.net.ar		<a href=http://www.arnet.net.ar>ISP: Arnet AR</a>
HideSite	*.arnet.net.ar
GroupSite	*.arsystel.com		<a href=http://www.arsystel.com>ISP: arsys.es ES</a>
HideSite	*.arsystel.com
GroupSite	*.artkas.fr		<a href=http://www.artkas.fr>ISP: Artkas FR</a>
HideSite	*.artkas.fr
GroupSite	*.as15758.net		<a href=http://www.as15758.net>ISP: PIPEX UK</a>
HideSite	*.as15758.net
GroupSite	*.asn-graz.ac.at	<a href=http://asn-graz.ac.at>EDU: Styrian Edu. server AT</a>
HideSite	*.asn-graz.ac.at
GroupSite	*.assertive.ca		<a href=http://www.assertive.ca>ISP: Assertive CA</a>
HideSite	*.assertive.ca
GroupSite	*.ataris.co.za		<a href=http://www.ataris.co.za>ISP: ISP Data ZA</a>
HideSite	*.ataris.co.za
GroupSite	*.att.ne.jp		<a href=http://www.att.ne.jp>ISP: JENS SpinNet JP</a>
HideSite	*.att.ne.jp
GroupSite	*.att.net		<a href=http://www.att.com>ISP: AT&amp;T US</a>
HideSite	*.att.net
GroupSite	*.attens.net		<a href=http://www.att.com>ISP: AT&amp;T US</a>
HideSite	*.attens.net
GroupSite	*.attbi.com		<a href=http://www.att.com>ISP: AT&amp;T US</a>
HideSite	*.attbi.com
GroupSite	*.attla.cl		<a href=http://www.attla.cl>ISP: Telmex CL</a>
HideSite	*.attla.cl
GroupSite	*.libre.auna.net		<a href=http://www.auna.net>ISP: AUNA ES</a>
HideSite	*.libre.auna.net
GroupSite	*.auna.net		<a href=http://www.auna.net>ISP: AUNA ES</a>
HideSite	*.auna.net
GroupSite	*.avantgo.com		<a href=http://www.avantgo.com>AvantGo **</a>
HideSite	*.avantgo.com
GroupSite	*.av.com		<a href=http://www.av.com>Spider: Altavista **</a>
HideSite	*.av.com
GroupSite	*.avantel.net.mx	<a href=http://www.avantel.net.mx>ISP: Avantel MX</a>
HideSite	*.avantel.net.mx
GroupSite	*.axelero.hu		<a href=http://www.axelero.hu>ISP: Axelero HU</a>
HideSite	*.axelero.hu
GroupSite	*.axiom.gr		<a href=http://www.axiom.gr>ISP: Axiom Net GR</a>
HideSite	*.axiom.gr
GroupSite	*.axone.fr		ISP: axone.fr (no website) FR
HideSite	*.axone.fr
GroupSite	*.axtel.net		<a href=http://www.axtel.net>ISP: Axtel MX</a>
HideSite	*.axtel.net
GroupSite	*.bancsabadell.com	<a href=http://www.bancsabadell.com>Bank: Sabadell ES</a>
HideSite	*.bancsabadell.com
GroupSite	*.bankofamerica.com	<a href=http://www.bankofamerica.com>Bank: OfAmerica US</a>
HideSite	*.bankofamerica.com
GroupSite	*.barak.net.il		<a href=http://www.barak.net.il>ISP: Barak 013 IL</a>
HideSite	*.barak.net.il
GroupSite	*.bathspa.ac.uk		<a href=http://www.bathspa.ac.uk>EDU: U. Bath Spa UK</a>
HideSite	*.bathspa.ac.uk
GroupSite	*.bbc.co.uk		<a href=http://www.bbc.co.uk>BBC UK</a>
HideSite	*.bbc.co.uk
GroupSite	*.bbeyond.nl		<a href=http://www.bbeyond.nl>ISP: Bbeyond NL</a>
HideSite	*.bbeyond.nl
GroupSite	*.bbt.net.ar		<a href=http://www.bbt.net.ar>ISP: BBT AR</a>
HideSite	*.bbt.net.ar
GroupSite	*.bchosting.com		<a href=http://www.bchosting.com>ISP: BCHosting</a>
HideSite	*.bchosting.com
GroupSite	*.bcn.es		<a href=http://www.bcn.es>Portal: Web de Barcelona ES</a>
HideSite	*.bcn.es
GroupSite	*.belgacom.be		<a href=http://www.belgacom.be>ISP: Belgacom BE</a>
HideSite	*.belgacom.be
GroupSite	*.bellsouth.com.ec	<a href=http://www.bellsouth.com.ec>ISP: BellSouth EC</a>
HideSite	*.bellsouth.com.ec
GroupSite	*.bellsouth.net.ar	bellsouth.net.ar (no website) AR
HideSite	*.bellsouth.net.ar
GroupSite	*.bellsouth.net		<a href=http://www.bellsouth.net>ISP: BellSouth US</a>
HideSite	*.bellsouth.net
GroupSite	*.berlikomm.net		<a href=http://www.berlokomm.net>ISP: Berlikomm DE</a>
HideSite	*.berlikomm.net
GroupSite	*.bezeqint.net		<a href=http://www.bezeqint.net>ISP: Bezeq IL</a>
HideSite	*.bezeqint.net
GroupSite	*.bgsdistribution.sk	<a href=http://www.bgsdistribution.sk>BgsDistribution SK</a>
HideSite	*.bgsdistribution.sk
GroupSite	*.bham.ac.uk		<a href=http://www.bham.ac.uk>EDU: U. Birmingham UK</a>
HideSite	*.bham.ac.uk
GroupSite	*.bireme.br		<a href=http://www.bireme.br>Virtual Health Library BR</a>
HideSite	*.bireme.br
GroupSite	*.blacksun.net		<a href=http://www.blacksun.net>ISP: Black Sun **</a>
HideSite	*.blacksun.net
GroupSite	*.bloglines.com		<a href=http://www.bloglines.com>Spider: Bloglines.com</a>
HideSite	*.bloglines.com
GroupSite	*.bluewin.ch		<a href=http://www.bluewin.ch>Bluewin CH</a>
HideSite	*.bluewin.ch
GroupSite	*.blueyonder.co.uk	<a href=http://www.blueyonder.co.uk>ISP: BlueYonder UK</a>
HideSite	*.blueyonder.co.uk
GroupSite	*.bmstu.ru		<a href=http://www.bmstu.ru>EDU: Moscow U.Tech. BMTSU RU</a>
HideSite	*.bmstu.ru
GroupSite	*.boebank.co.za		<a href=http://www.boebank.co.za>Bank: BOE Bank ZA</a>
HideSite	*.boebank.co.za
GroupSite	*.bonet.se		<a href=http://www.bonet.se>ISP: Bostream Bredband SE</a>
HideSite	*.bonet.se
GroupSite	*.u-bourgogne.fr	<a href=http://www.u-bourgogne.fr>EDU: U. Bourgogne FR</a>
HideSite	*.u-bourgogne.fr
GroupSite	*.bouyguestelecom.fr	<a href=http://www.bouyguestelecom.fr>ISP: Bouygues FR</a>
HideSite	*.bouyguestelecom.fr
GroupSite	*.brabys.co.za		<a href=http://www.brabys.co.za>S.Africa Business Dir ZA</a>
HideSite	*.brabys.co.za
GroupSite	*.brasiltelecom.net.br	<a href=http://www.brasiltelecom.net.br>ISP: BT BR</a>
HideSite	*.brasiltelecom.net.br
GroupSite	*.brasnet.org		<a href=http://brasnet.org>BRASnet IRC channel BR</a>
HideSite	*.brasnet.org
GroupSite	*.bredbandsbolaget.se	<a href=http://www.bredbandsbolaget.se>ISP: BDb. SE</a>
HideSite	*.bredbandsbolaget.se
GroupSite	*.broadred.net		ISP: broadred.net (no website)
HideSite	*.broadred.net
GroupSite	*.brunel.ac.uk		<a href=http://www.brunel.ac.uk>EDU: U. Brunel UK</a>
HideSite	*.brunel.ac.uk
GroupSite	*.brutele.be		<a href=http://www.brutele.be>ISP: Brutele BE</a>
HideSite	*.brutele.be
GroupSite	*.bt.net		<a href=http://www.bt.net>ISP: BT (Europe)</a>
HideSite	*.bt.net
GroupSite	*.btcentralplus.com	ISP: btcentralplus.com (no website) UK
HideSite	*.btcentralplus.com
GroupSite	*.btinternet.com	<a href=http://www.btinternet.com>ISP: BTopenworld UK</a>
HideSite	*.btinternet.com
GroupSite	*.btopenworld.com	<a href=http://www.btopenworld.com>ISP: BT Open World UK</a>
HideSite	*.btopenworld.com
GroupSite	*.buap.mx		<a href=http://www.buap.mx>EDU: B. U. Auton. Puebla MX</a>
HideSite	*.buap.mx
GroupSite	*.bull.fr		<a href=http://www.bull.fr>ISP: Bull FR</a>
HideSite	*.bull.fr
GroupSite	*.bulldogdsl.com	<a href=http://www.bulldogdsl.com>ISP: bulldogdsl UK</a>
HideSite	*.bulldogdsl.com
GroupSite	*.bvconline.com.ar 	<a href=http://www.bvconline.com.ar>ISP: bvconline AR</a>
HideSite	*.bvconline.com.ar 
GroupSite	*.bvrp.com		<a href=http://www.bvrp.com>BVRP Software FR</a>
HideSite	*.bvrp.com
GroupSite	*.c-able.ne.jp		<a href=http://www.c-able.ne.jp>ISP: c-able JP</a>
HideSite	*.c-able.ne.jp
GroupSite	*.cable.net.co		<a href=http://cable.net.co>ISP: TV Cable CO</a>
HideSite	*.cable.net.co
GroupSite	*.cablecolor.hn		<a href=http://www.cablecolor.hn>ISP: CableColor HN</a>
HideSite	*.cablecolor.hn
GroupSite	*.cablenet.com.ni	<a href=http://www.cablenet.com.ni>ISP: Cablenet NI</a>
HideSite	*.cablenet.com.ni
GroupSite	*.cablenet-va.com	<a href=http://www.charter.com>ISP: Charter **</a>
HideSite	*.cablenet-va.com
GroupSite	*.cableonda.net		<a href=http://www.cableonda.net>ISP: Cableonda PA</a>
HideSite	*.cableonda.net
GroupSite	*.cableonline.com.mx	<a href=http://www.cablered.com.mx>ISP: Cablemas PMX</a>
HideSite	*.cableonline.com.mx
GroupSite	*.cablered.com.mx	<a href=http://www.cablered.com.mx>ISP: Cablemas MX</a>
HideSite	*.cablered.com.mx
GroupSite	*.cablevision.net.mx	ISP: CableVision http://www.cablevision.net.mx MX
HideSite	*.cablevision.net.mx
GroupSite	*.caltech.edu		<a href=http://www.caltech.edu>EDU: California I.T. US</a>
HideSite	*.caltech.edu
GroupSite	*.canberra.edu.au	<a href=http://www.canberra.edu.au>EDU: U. Camberra AU</a>
HideSite	*.canberra.edu.au
GroupSite	*.cantv.net		<a href=http://www.cantv.net>ISP: Cantv.net VE</a>
HideSite	*.cantv.net
GroupSite	*.cap.edu.mx		<a href=http://www.cap.edu.mx>EDU: C. Ameri. Puebla MX</a>
HideSite	*.cap.edu.mx
GroupSite	*.caq.edu.ec		<a href=http://caq.edu.ec>EDU: C. Aleman Quito EC</a>
HideSite	*.caq.edu.ec
GroupSite	*.carats.net		<a href=http://www.carats.net>ISP: CARATS Caribe</a>
HideSite	*.carats.net
GroupSite	*.casema.nl		<a href=http://www.casema.nl>ISP: Casema NL</a>
HideSite	*.casema.nl
GroupSite	*.cattel.com		<a href=http://www.cattel.com>ISP: Catalana Teleco. ES</a>
HideSite	*.cattel.com
GroupSite	*.ccl.ru		<a href=http://www.ccl.ru>ISP: ccl? RU</a>
HideSite	*.ccl.ru
GroupSite	*.ccu.edu.tw		<a href=http://www.ccu.edu.tw>EDU: U. N. Chung Cheng TW</a>
HideSite	*.ccu.edu.tw
GroupSite	*.centurytel.net	<a href=http://www.centurytel.net>ISP: Centurytel US</a>
HideSite	*.centurytel.net
GroupSite	*.cerfnet.com		<a href=http://www.cerf.net>ISP: cerfnet.com AT&amp;T US</a>
HideSite	*.cerfnet.com
GroupSite	*.cciglobal.net.pa	<a href=http://www.cciglobal.net.pa>ISP: GBnet PA</a>
HideSite	*.cciglobal.net.pa
GroupSite	*.ca.charter.com	<a href=http://www.charter.com>ISP: Charter CA</a>
HideSite	*.ca.charter.com
GroupSite	*.charter.com		<a href=http://www.charter.com>ISP: Charter **</a>
HideSite	*.charter.com
GroupSite	*.charter-stl.com	<a href=http://www.charter.com>ISP: Charter **</a>
HideSite	*.charter-stl.com
GroupSite	*.chartermi.net		<a href=http://charter.msn.com>ISP: MSN-Charter **</a>
HideSite	*.chartermi.net
GroupSite	*.charterpipeline.net	<a href=http://www.charter.com>ISP: Charter **</a>
HideSite	*.charterpipeline.net
GroupSite	*.chello.be		<a href=http://www.chello.nl>ISP: Chello BE</a>
HideSite	*.chello.be
GroupSite	*.chello.fr		<a href=http://www.chello.nl>ISP: Chello FR</a>
HideSite	*.chello.fr
GroupSite	*.chello.pl		<a href=http://www.chello.nl>ISP: Chello PL</a>
HideSite	*.chello.pl
GroupSite	*.chello.se		<a href=http://www.chello.nl>ISP: Chello SE</a>
HideSite	*.chello.se
GroupSite	*.chello.nl		<a href=http://www.chello.nl>ISP: Chello NL</a>
HideSite	*.chello.nl
#GroupSite	*.chello.		<a href=http://www.chello.nl>ISP: Chello NL</a>
#HideSite	*.chello.
GroupSite	*.chevrontexaco.com	<a href=http://www.chevrontexaco.com>ChevronTexaco</a>
HideSite	*.chevrontexaco.com
GroupSite	*.chilesat.net		<a href=http://www.chilesat.net>ISP: 171 Chilesat CL</a>
HideSite	*.chilesat.net
GroupSite	*.cica.es		Centro Informatico de Informacion de Andalucia www.cica.es ES
HideSite	*.cica.es
GroupSite	*.cidem.com		<a href=http://www.cidem.com>CIDEM Generalitat ES</a>
HideSite	*.cidem.com
GroupSite	*.cigb.edu.cu		CIGB Centro Ing. Genetica y Biotec. www.cigb.edu.cu CU
HideSite	*.cigb.edu.cu
GroupSite	*.cisco.com		<a href=http://www.cisco.com>Cisco Systems</a>
HideSite	*.cisco.com
GroupSite	*.citefa.gov.ar		<a href=http://www.citefa.gov.ar>GOV: CITEFA AR</a>
HideSite	*.citefa.gov.ar
GroupSite	*.citykom.de		<a href=http://www.citykom.de>ISP: Citykom DE</a>
HideSite	*.citykom.de
GroupSite	*.cgocable.net		<a href=http://www.cgocable.ca>ISP: COGECO cable CA</a>
HideSite	*.cgocable.net
GroupSite	*.client.comcast.net	<a href=http://www.comcast.net>ISP: Comcast UE</a>
HideSite	*.client.comcast.net
GroupSite	*.club-internet.fr	<a href=http://www.club-internet.fr>ISP: Club-Int. FR</a>
HideSite	*.club-internet.fr
GroupSite	*.cern.ch		<a href=http://www.cern.ch>EDU: CERN CH</a>
HideSite	*.cern.ch
GroupSite	*.cesca.es		<a href=http://www.cesca.es>Cesca Supercomputing ES</a>
HideSite	*.cesca.es
GroupSite	*.cm.vtr.net		<a href=http://www.vtr.net>ISP: VTR CL</a>
HideSite	*.cm.vtr.net
GroupSite	*.cmdg.com		<a href=http://www.cmdg.com>Reed Construction Info. US</a>
HideSite	*.cmdg.com
GroupSite	*.hci.cs.cmu.edu	<a href=http://www.hci.cs.cmu.edu>EDU: Carnegie M.(CS) US</a>
HideSite	*.cmu.edu
GroupSite	*.cmu.edu		<a href=http://www.cmu.edu>EDU: U. Carnegie Mellon US</a>
HideSite	*.cmu.edu
GroupSite	*.iit.cnr.it		Spider: Ubicrawler ** ubi.imc.pi.cnr.it/projects/ubicrawler/
HideSite	*.iit.cnr.it
GroupSite	*.cnw-inter.net		<a href=http://www.christians.co.za>christians web ZA</a>
HideSite	*.cnw-inter.net
GroupSite	*.codelco.cl		<a href=http://www.codelco.cl>CODELCO Corp. del cobre CL</a>
HideSite	*.codelco.cl
GroupSite	*.codetel.net.do	<a href=http://www.codetel.net.do>ISP: Codetel DO</a>
HideSite	*.codetel.net.do
GroupSite	*.codex.co.za		<a href=http://www.codex.co.za>ISP: CODEX ZA</a>
HideSite	*.codex.co.za
GroupSite	*.colba.net		<a href=http://www.colba.net>ISP: Colba CA</a>
HideSite	*.colba.net
GroupSite	*.colorado.edu		<a href=http://www.colorado.edu>EDU: U. Colorado US</a>
HideSite	*.colorado.edu
GroupSite	*.coltfrance.com	<a href=http://www.coltfrance.com>ISP: Colt FR</a>
HideSite	*.coltfrance.com
GroupSite	*.colt.net		<a href=http://www.colt.net>ISP: Colt (European)</a>
HideSite	*.colt.net
GroupSite	*.com.uy		<a href=http://www.com.uy>ISP: www.com.uy UY</a>
HideSite	*.com.uy
GroupSite	*.comcast.net		<a href=http://www.comcast.net>ISP: Comcast US</a>
HideSite	*.comcast.net
GroupSite	*.comindico.com.au	<a href=http://www.comindico.com.a>ISP: Comindico AU</a>
HideSite	*.comindico.com.au
GroupSite	*.compaq.com		<a href=http://www.compaq.com>Compaq</a>
HideSite	*.compaq.com
GroupSite	*.compuserve.com	<a href=http://www.compuserve.com>ISP: Compuserve US</a>
HideSite	*.compuserve.com
GroupSite	*.computronic.edu.pe	<a href=http://www.computronic.edu.pe>EDU: Computro. PE</a>
HideSite	*.computronic.edu.pe
GroupSite	*.comsat.com.co		<a href=http://www.comsat.com.co>ISP: Comsat CO</a>
HideSite	*.comsat.com.co
GroupSite	*.comsats.net.pk	<a href=http://www.comsats.net.pk>ISP: Comsats PK</a>
HideSite	*.comsats.net.pk
GroupSite	*.connections.net.nz	<a href=http://www.connections.net.nz>ISP: Connec. NZ</a>
HideSite	*.connections.net.nz
GroupSite	*.conversent.net	<a href=http://www.conversent.net>ISP: Conversent US</a>
HideSite	*.conversent.net
GroupSite	*.coqui.net		<a href=http://www.coqui.net>ISP: Coqui (Caribe)</a>
HideSite	*.coqui.net
GroupSite	*.cornell.edu		<a href=http://www.cornell.edu>EDU: U. Cornell US</a>
HideSite	*.cornell.edu
GroupSite	*.cotas.com.bo		<a href=http://www.cotas.com.bo>ISP: Coop.T. SantaCruz BO</a>
HideSite	*.cotas.com.bo
GroupSite	*.covad.net		<a href=http://www.covad.com/>ISP: Covad **</a>
HideSite	*.covad.net
GroupSite	*.coxfiber.net		<a href=http://www.cox.net>ISP: Cox Cable US</a>
HideSite	*.coxfiber.net
GroupSite	*.cox-internet.com	<a href=http://www.cox-internet.com>ISP: Cox US</a>
HideSite	*.cox-internet.com
GroupSite	*.cox.net		<a href=http://www.cox.net>ISP: Cox Cable US</a>
HideSite	*.cox.net
GroupSite	*.covad.net		<a href=http://www.covad.net>ISP: Covad US</a>
HideSite	*.covad.net
GroupSite	*.criba.edu.ar		<a href=http://www.criba.edu.ar>EDU: CRIBABB AR</a>
HideSite	*.criba.edu.ar
GroupSite	*.csic.es		<a href=http://www.csic.es>CSIC ES</a>
HideSite	*.csic.es
GroupSite	*.csir.co.za		<a href=http://www.csir.co.za>CSIR Research ZA</a>
HideSite	*.csir.co.za
GroupSite	*.ct.co.cr		<a href=http://www.ct.co.cr>ISP: CableTica CR</a>
HideSite	*.ct.co.cr
GroupSite	*.ctech.ac.za		<a href=http://www.ctech.ac.za>ISP: Cape Tech ZA</a>
HideSite	*.ctech.ac.za
GroupSite	*.cityu.edu.hk		<a href=http://www.cityu.edu.hk>EDU: City-Univ. HK</a>
HideSite	*.cityu.edu.hk
GroupSite	*.cu.edu.tr		<a href=http://www.cu.edu.tr>EDU: Cukurova U. TR</a>
HideSite	*.cu.edu.tr
GroupSite	*.cuauhtemoc.edu.mx	<a href=http://www.cuauhtemoc.edu.mx>EDU: U. Cuauh. MX</a>
HideSite	*.cuauhtemoc.edu.mx
GroupSite	*.cujae.edu.cu		<a href=http://www.cujae.edu.cu>EDU: Inst.S.Pol.J.A.E. CU</a>
HideSite	*.cujae.edu.cu
GroupSite	*.cuncablemas.com	cuncablemas.com (no website) ??
HideSite	*.cuncablemas.com
GroupSite	*.cwi.nl		EDU: Nat. Cent. Research Math&amp;Computer science www.cwi.nl NL
HideSite	*.cwi.nl
GroupSite	*.cwpanama.net		<a href=http://www.cwpanama.net>ISP: C&amp;W Panama PA</a>
HideSite	*.cwpanama.net
GroupSite	*.cybercable.tm.fr	<a href=http://www.cybercable.tm.fr>ISP: Cybercable FR</a>
HideSite	*.cybercable.tm.fr
GroupSite	*.d1-online.com		<a href=http://www.t-mobile.de>ISP: T-Mobile DE</a>
HideSite	*.d1-online.com
GroupSite	*.daimi.au.dk		<a href=http://www.daimi.au.dk>EDU: Comp.Scienc. Daimi DK</a>
HideSite	*.daimi.au.dk
GroupSite	*.dal.ca		<a href=http://www.dal.ca>EDU: Dalhousie U. CA</a>
HideSite	*.dal.ca
GroupSite	*.dasdial.net		dasdial.net (no website) DE?
HideSite	*.dasdial.net
GroupSite	*.datanet.hu		<a href=http://www.datanet.hu>Datanet HU</a>
HideSite	*.datanet.hu
GroupSite	*.datapro.co.za		<a href=http://www.datapro.co.za>ISP: DataPro ZA</a>
HideSite	*.datapro.co.za
GroupSite	*.kcl.ac.uk		<a href=http://www.kcl.ac.uk>EDU: Kings College UK</a>
HideSite	*.kcl.ac.uk
GroupSite	*.demon.co.uk		<a href=http://www.demon.co.uk>ISP: Demon UK</a>
HideSite	*.demon.co.uk
GroupSite	*.depaul.edu		<a href=http://www.depaul.edu>EDU: U. DePaul UE</a>
HideSite	*.depaul.edu
GroupSite	*.deusto.es		<a href=http://www.deusto.es>EDU: U. Deusto ES</a>
HideSite	*.deusto.es
GroupSite	*.dial.net.mx		<a href=http://www.avantel.net.mx>ISP Avantel MX</a>
HideSite	*.dial.net.mx
GroupSite	*.dial-up.net		<a href=http://www.is.co.za>Internet Solutions ZA</a>
HideSite	*.dial-up.net
GroupSite	*.dialsprint.net	<a href=http://www.sprint.com>ISP: Sprint</a>
HideSite	*.dialsprint.net
GroupSite	*.digitelone.com	<a href=http://www.digitelone.com>ISP: DigitelOne PH</a>
HideSite	*.digitelone.com
GroupSite	*.dinanet.net.co	<a href=http://www.dinanet.net.co>ISP: Dinanet CO</a>
HideSite	*.dinanet.net.co
GroupSite	*.dion.ne.jp		<a href=http://www.dion.ne.jp>ISP: Dion JP</a>
HideSite	*.dion.ne.jp
GroupSite	*.direcpc.com		<a href=http://www.direcpc.com>ISP: DirecPC(satellite)</a>
HideSite	*.direcpc.com
GroupSite	*.direct-adsl.nl	<a href=http://www.kpn.com>ISP: KPN NL</a>
HideSite	*.direct-adsl.nl
GroupSite	*.static.kpn.net	<a href=http://www.kpn.com>ISP: KPN NL</a>
HideSite	*.static.kpn.net
GroupSite	*.directhit.com		<a href=http://www.directhit.com>Search: Teoma US</a>
HideSite	*.directhit.com
GroupSite	*.dir.com		<a href=http://www.dir.com>Search: Dir.com **</a>
HideSite	*.dir.com
GroupSite	*.discovernet.net	<a href=http://www.discovernet.net>ISP: Discovernet UK</a>
HideSite	*.discovernet.net
GroupSite	*.dls.net		<a href=http://www.dls.net>ISP: DLS US</a>
HideSite	*.dls.net
GroupSite	*.dosanclas.com.ar 	<a href=http://www.dosanclas.com.ar>Dos Anclas Salt AR</a>
HideSite	*.dosanclas.com.ar 
GroupSite	*.drexel.edu		<a href=http://www.drexel.edu>EDU: U. Drexel US</a>
HideSite	*.drexel.edu
GroupSite	*.dxi.net		<a href=http://www.dxi.net>ISP: DXI UK</a>
HideSite	*.dxi.net
GroupSite	*.eac.edu		<a href=http://www.eac.edu>EDU: U. Eastern Arizona US</a>
HideSite	*.eac.edu
GroupSite	*.eaccess.ne.jp		<a href=http://www.eaccess.net>ISP: EAccess JP</a>
HideSite	*.eaccess.ne.jp
GroupSite	*.eads.net		www.eads.net European Aeronautic Defence and Space Company
HideSite	*.eads.net
GroupSite	*.eafit.edu.co		<a href=http://www.eafit.edu.co>EDU: U. EAFIT CO</a>
HideSite	*.eafit.edu.co
GroupSite	*.earlham.edu		<a href=http://www.earlham.edu>EDU: Earlhm Collegue US</a>
HideSite	*.earlham.edu
GroupSite	*.earthlink.net		<a href=http://www.earthlink.net>ISP: Earthlink</a>
HideSite	*.earthlink.net
GroupSite	*.eastlink.ca		<a href=http://www.eastlink.ca>ISP: Eastlink CA</a>
HideSite	*.eastlink.ca
GroupSite	*.easynet.be		<a href=http://www.easynet.be>ISP: Easynet BE</a>
HideSite	*.easynet.be
GroupSite	*.easynet.es		<a href=http://www.easynet.fr>ISP: Easynet ES</a>
HideSite	*.easynet.es
GroupSite	*.easynet.fr		<a href=http://www.easynet.fr>ISP: Easynet FR</a>
HideSite	*.easynet.fr
GroupSite	*.easynet.net		<a href=http://www.easynet.net>ISP: Easynet **</a>
HideSite	*.easynet.net
GroupSite	*.eca.ac.uk		<a href=http://www.eca.ac.uk>EDU: Edinburgh CollegeofArt UK</a>
HideSite	*.eca.ac.uk
GroupSite	*.echoechoplus.com	<a href=http://echoechoplus.com>Hosting: EchoEchoPlus **</a>
HideSite	*.echoechoplus.com
GroupSite	*.berkwin.ednet.ns.ca	<a href=http://www.berkwin.ednet.ns.ca>EDU: Glooscap CA</a>
HideSite	*.berkwin.ns.ca
GroupSite	*.ednet.ns.ca		<a href=http://ednet.ns.ca>EDU: Dept. Education N.V. CA</a>
HideSite	*.ednet.ns.ca
GroupSite	*.ego.net.mx		<a href=http://ego.net.mx>ISP: e-go MX</a>
HideSite	*.ego.net.mx
GroupSite	*.ehu.es		<a href=http://www.ehu.es>EDU: Euskal Herriko U. ES</a>
HideSite	*.ehu.es
GroupSite	*.eircom.net		<a href=http://www.eircom.net>ISP: Eircom IE</a>
HideSite	*.eircom.net
GroupSite	*.elp.rr.com		<a href=http://www.elp.rr.com>ISP: El Paso,TimeWarner US</a>
HideSite	*.elp.rr.com
GroupSite	*.rr.com		<a href=http://www.rr.com>ISP RoadRunner US</a>
HideSite	*.rr.com
GroupSite	*.elsendero.net		<a href=http://www.elsendero.net>ISP: El Sendero ES</a>
HideSite	*.elsendero.net
GroupSite	*.emory.edu		<a href=http://www.emory.edu>EDU: U. Emory US</a>
HideSite	*.emory.edu
GroupSite	*.enet.cu		<a href=http://www.etecsa.cu>ISP: ENET/ETECSA CU</a>
HideSite	*.enet.cu
GroupSite	*.enitel.net.ni		<a href=http://www.enitel.net.ni>ISP: Enitel NI</a>
HideSite	*.enitel.net.ni
GroupSite	*.entel.cl		<a href=http://www.entel.cl>ISP: Entel CL</a>
HideSite	*.entel.cl
GroupSite	*.entelchile.net	<a href=http://www.entelchile.net>ISP: Entelchile 123 CL</a>
HideSite	*.entelchile.net
GroupSite	*.epm.net.co		<a href=http://www.epm.net.co>ISP: EPM.Net CO</a>
HideSite	*.epm.net.co
GroupSite	*.esade.es		<a href=http://www.esade.es>EDU: ESADE ES</a>
HideSite	*.esade.es
GroupSite	*.esat.net		<a href=http://www.esat.net>ISP: BT Ignite US</a>
HideSite	*.esat.net
GroupSite	*.escape.com		<a href=http://www.escape.com>ISP: Escape US</a>
HideSite	*.escape.com
GroupSite	*.esi.es		<a href=http://www.esi.es>ISP: ESI ES</a>
HideSite	*.esi.es
GroupSite	*.eskom.co.za		<a href=http://www.eskom.co.za>ISP: Eskom ZA</a>
HideSite	*.eskom.co.za
GroupSite	*.espol.edu.ec		<a href=http://www.espol.edu.ec>EDU: E.S.Pol. Litoral EC</a>
HideSite	*.espol.edu.ec
GroupSite	*.etel.at		<a href=http://www.etel.at>ISP: eTel AT</a>
HideSite	*.etel.at
GroupSite	*.eurociber.es		<a href=http://www.eurociber.es>ISP: Eurociber ES</a>
HideSite	*.eurociber.es
GroupSite	*.euronet.nl		<a href=http://www.euronet.nl>ISP: Euronet NL</a>
HideSite	*.euronet.nl
GroupSite	*.euskaltel.es		<a href=http://www.euskatel.es>ISP: Euskatel ES</a>
HideSite	*.euskaltel.es
GroupSite	*.ev1.net		<a href=http://www.ev1.net>ISP: EV1 US</a>
HideSite	*.ev1.net
GroupSite	*.exabot.com		<a href=http://www.exabot.com>Search: Exabot</a>
HideSite	*.exabot.com
GroupSite	*.eyrolles.com		<a href=http://www.eyrolles.com>Eyrolles FR</a>
HideSite	*.eyrolles.com
GroupSite	*.lviv.farlep.net	<a href=http://www.lviv.farlep.net>ISP: ?????? UA</a>
HideSite	*.lviv.farlep.net
GroupSite	*.fastsearch.net	<a href=http://www.fastsearch.net>Spider: FAST</a>
HideSite	*.fastsearch.net
GroupSite	*.favaloro.edu.ar	<a href=http://www.favaloro.edu.ar>EDU: Favaloro AR</a>
HideSite	*.favaloro.edu.ar
GroupSite	*.fdu.edu		<a href=http://www.fdu.edu>EDU: U. Fairleigh D. US</a>
HideSite	*.fdu.edu
GroupSite	*.fibertel.com.ar	<a href=http://www.fibertel.com.ar>ISP: FiberTel AR</a>
HideSite	*.fibertel.com.ar
GroupSite	*.fibrewired.on.ca	<a href=http://www.fibrewired.com>ISP: FibreWired CA</a>
HideSite	*.fibrewired.on.ca
GroupSite	*.fing.edu.uy		<a href=http://www.fing.edu.uy>EDU: U. Republica UY</a>
HideSite	*.fing.edu.uy
GroupSite	*.firstream.com		<a href=http://www.firststream.com>Firstream US</a>
HideSite	*.firstream.com
GroupSite	*.fit.ba		<a href=http://www.fit.ba>EDU: F. Informacijskih T. BA</a>
HideSite	*.fit.ba
GroupSite	*.fmcf.fr		<a href=http://www.fmcf.fr>ISP: Neuf telecom FR</a>
HideSite	*.fmcf.fr
GroupSite	*.fmed.edu.uy		<a href=http://www.fmed.edu.uy>EDU: Fac. Medicina UY</a>
HideSite	*.fmed.edu.uy
GroupSite	*.ford.com		<a href=http://www.ford.com>Ford US</a>
HideSite	*.ford.com
GroupSite	*.fpablovi.org		<a href=http://www.fpablovi.org>EDU: Fund. Pablo I ES</a>
HideSite	*.fpablovi.org
GroupSite	*.francetelecom.com	<a href=http://www.francetelecom.com>ISP: FT FR</a>
HideSite	*.francetelecom.com
GroupSite	*.freestart.hu		<a href=http://www.freestart.hu>ISP: Freestart HU</a>
HideSite	*.freestart.hu
GroupSite	*.freesurf.ch		<a href=http://www.freesurf.ch>ISP: Sunrise CH</a>
HideSite	*.freesurf.ch
GroupSite	*.frogfoot.net		<a href=http://www.frogfoot.net>ISP: frogfoot ZA</a>
HideSite	*.frogfoot.net
GroupSite	*.fujixerox.co.jp	<a href=http://www.fujixerox.co.jp>Fuji Xerox JP</a>
HideSite	*.fujixerox.co.jp
GroupSite	*.fullerton.edu		<a href=http://www.fullerton.edu>EDU: U. Fullerton US</a>
HideSite	*.fullerton.edu
GroupSite	*.galileo.edu		<a href=http://www.galileo.edu>EDU: U. Galileo GT</a>
HideSite	*.galileo.edu
GroupSite	*.games-workshop.es	<a href=http://www.games-workshop.es>GamesWorkshop ES</a>
HideSite	*.games-workshop.es
GroupSite	*.gatech.edu		<a href=http://www.gatech.edu>EDU: Georgia Tech.I. US</a>
HideSite	*.gatech.edu
GroupSite	*.gc.ca			<a href=http://www.gc.ca>GOV: Canada CA</a>
HideSite	*.gc.ca
GroupSite	*.gc.cc			<a href=http://www.gc.cc>ISP: Enic Cocos Islands CC</a>
HideSite	*.gc.cc
GroupSite	*.gigared.com		<a href=http://www.gigared.com>ISP: Gigared AR</a>
HideSite	*.gigared.com
GroupSite	*.gobiernodecanarias.org	GOV: www.gobiernodecanarias.org ES
HideSite	*.gobiernodecanarias.org
GroupSite	*.googlebot.com		<a href=http://www.googlebot.com>Spider: Google **</a>
HideSite	*.googlebot.com
GroupSite	*.grci.com		<a href=http://www.grci.com>ISP: grci.com AT&amp;T US</a>
HideSite	*.grci.com
GroupSite	*.griffin.net.uk	<a href=http://www.griffin.net.uk>ISP: Griffin UK</a>
HideSite	*.griffin.net.uk
GroupSite	*.green.ch		<a href=http://www.green.ch>ISP Green CH</a>
HideSite	*.green.ch
GroupSite	*.gua.net		<a href=http://www.gua.net>ISP/Portal: GUA.NET GT</a>
HideSite	*.gua.net
GroupSite	*.guate.net.gt		<a href=http://www.guate.net.gt>ISP: Convergence C. GT</a>
HideSite	*.guate.net.gt
GroupSite	*.guernsey.net		<a href=http://www.guernsey.net>ISP: Guernsey UK?</a>
HideSite	*.guernsey.net
GroupSite	*.gva.es		<a href=http://www.gva.es>GOV: G. Valenciana ES</a>
HideSite	*.gva.es
GroupSite	*.gvt.net.br		<a href=http://www.gvt.net.br>ISP: GVT BR</a>
HideSite	*.gvt.net.br
GroupSite	*.haestad.com		<a href=http://www.haestad.com>Haestad(water treatment) **</a>
HideSite	*.haestad.com
GroupSite	*.hansenet.de		<a href=http://www.hansenet.de>ISP: HanseNet DE</a>
HideSite	*.hansenet.de
GroupSite	*.hzeeland.nl		<a href=http://www.hzeeland.nl>EDU: H. Zeeland NL</a>
HideSite	*.hzeeland.nl
GroupSite	*.helsinki.fi		<a href=http://www.helsinki.fi>EDU: U. Helsinki FI</a>
HideSite	*.helsinki.fi
GroupSite	*.hetnet.nl		<a href=http://www.hetnet.nl>ISP: Het Net NL</a>
HideSite	*.hetnet.nl
GroupSite	*.hhs.nl		<a href=http://www.hhs.nl>EDU: Haagse Hogeschool NL</a>
HideSite	*.hhs.nl
GroupSite	*.hinet.net		<a href=http://www.hinet.net>ISP: Hinet TW</a>
HideSite	*.hinet.net
GroupSite	*.hispeed.ch 		<a href=http://www.hispeed.ch>Highspeed CH</a>
HideSite	*.hispeed.ch
GroupSite	*.hk.net		<a href=http://www.hknet.com>ISP: HKnet HK</a>
HideSite	*.hk.net
GroupSite	*.hkcable.com.hk	ISP: hkcable.com.hk HK
HideSite	*.hkcable.com.hk
GroupSite	*.hku.hk		<a href=http://www.hku.hk>EDU: U. Hong Kong HK</a>
HideSite	*.hku.hk
GroupSite	*.hol.gr		<a href=http://www.hol.gr>ISP: Hellas On Line GR</a>
HideSite	*.hol.gr
GroupSite	*.home.com		ISP: home.com **
HideSite	*.home.com
GroupSite	*.fr.home.nl		<a href=http://www.home.nl>ISP: Home FR</a>
HideSite	*.fr.home.nl
GroupSite	*.home.nl		<a href=http://www.home.nl>ISP: Home NL</a>
HideSite	*.home.nl
GroupSite	*.hp.com		<a href=http://www.hp.com>HP US</a>
HideSite	*.hp.com
GroupSite	*.hsbc.com		<a href=http://www.hsbc.com>ISP: HSBC **</a>
HideSite	*.hsbc.com
GroupSite	*.iae.nl		<a href=http://www.iae.nl>ISP: IAE NL</a>
HideSite	*.iae.nl
GroupSite	*.iber-x.net		<a href=http://www.iber-x.net>ISP/Portal: Iber-X ES</a>
HideSite	*.iber-x.net
GroupSite	*.iberbanda.es		<a href=http://www.iberbanda.es>ISP: Iberbanda ES</a>
HideSite	*.iberbanda.es
GroupSite	*.ibercom.com		<a href=http://www.ibercom.com>ISP: Ibercom ES</a>
HideSite	*.ibercom.com
GroupSite	*.ibi.com		<a href=http://www.informationbuilders.com>Inf. Builders</a>
HideSite	*.ibi.com
GroupSite	*.ibys.com.br		ISP: ibys.com.br (no website) BR
HideSite	*.ibys.com.br
GroupSite	*.ibm.com		<a href=http://www.ibm.com>IBM US</a>
HideSite	*.ibm.com
GroupSite	*.icest.edu.mx		<a href=http://www.icest.edu.mx>EDU: I. Tamaulipas MX</a>
HideSite	*.icest.edu.mx
GroupSite	*.icpnet.pl		<a href=http://www.icpnet.pl>ISP: ICP PL</a>
HideSite	*.icpnet.pl
GroupSite	*.ictnet.es		<a href=http://www.ictnet.es>ICTnet ES</a>
HideSite	*.ictnet.es
GroupSite	*.idial.com.br		ISP: idial.com.br (no website) BR
HideSite	*.idial.com.br
GroupSite	*.ie.edu		<a href=http://www.ie.edu>EDU: Instituto de Empresa ES</a>
HideSite	*.ie.edu
GroupSite	*.ied.edu.hk		<a href=http://www.ied.edu.hk>EDU: Hong Kong Edu.Inst. HK</a>
HideSite	*.ied.edu.hk
GroupSite	*.iese.edu		<a href=http://www.iese.edu>EDU: IESE/U. Navarra ES</a>
HideSite	*.iese.edu
GroupSite	*.ifl.net		<a href=http://www.ifl.net>EDU: InternetForLearning UK</a>
HideSite	*.ifl.net
GroupSite	*.ifxnetworks.net.ve	<a href=http://www.ifxnetworks.com.ve>ISP: IFX VE</a>
HideSite	*.ifxnetworks.net.ve
GroupSite	*.ifxnw.cl		<a href=http://www.ifxnw.cl>ISP: IFX Networks CL</a>
HideSite	*.ifxnw.cl
GroupSite	*.ifxnw.com.ar		<a href=http://www.ifxnw.com.ar>ISP: IFX Networks AR</a>
HideSite	*.ifxnw.com.ar
GroupSite	*.ifx.net.co		<a href=http://www.ifx.net.co>ISP: IFX Networks CO</a>
HideSite	*.ifx.net.co
GroupSite	*.de.ignite.net		<a href=http://www.ignite.net>ISP: BT Global DE</a>
HideSite	*.de.ignite.net
GroupSite	*.ihug.co.nz		<a href=http://www.ihug.co.nz>ISP: ihug NZ</a>
HideSite	*.ihug.co.nz
GroupSite	*.iie.org.mx		EDU: Instituto Investigaciones Electricas www.iie.org.mx MX
HideSite	*.iie.org.mx
GroupSite	*.iinet.net.au		<a href=http://www.iinet.net.au>ISP: iiNet AU</a>
HideSite	*.iinet.net.au
GroupSite	*.iki.ne.jp		<a href=http://www.iki.ne.jp>ISP: IKI JP</a>
HideSite	*.iki.ne.jp
GroupSite	*.iliad.fr		<a href=http://www.iliad.fr>ISP: Iliad - Free FR</a>
HideSite	*.iliad.fr
GroupSite	*.image.search.mud.yahoo.net	Spider: Yahoo! Image crawler **</a>
HideSite	*.image.search.mud.yahoo.net
GroupSite	*.imaginet.fr		<a href=http://www.imaginet.fr>ISP: Imaginet (YIN) FR</a>
HideSite	*.imaginet.fr
GroupSite	*.impsat.net.ec		<a href=http://www.impsat.com>ISP: IMPSAT EC</a>
HideSite	*.impsat.net.ec
GroupSite	*.impol.net		<a href=http://www.impol.net>ISP: Imperial ZA</a>
HideSite	*.impol.net
GroupSite	*.impsat.com.co		<a href=http://www.impsat.com.co>ISP: IMPSAT CO</a>
HideSite	*.impsat.com.co
GroupSite   *.in-berlin.de      <a href=http://www.in-berlin.de>ISP: IN-BERLIN</a>
HideSite    *.in-berlin.de
GroupSite	*.indiana.edu		<a href=http://www.indiana.edu>EDU: U. Indiana US</a>
HideSite	*.indiana.edu
GroupSite	*.indos.cz		<a href=http://www.indos.cz>ISP: Indos CZ</a>
HideSite	*.indos.cz
GroupSite	*.infonet.com		<a href=http://www.infonet.com>ISP: Infornet US</a>
HideSite	*.infonet.com
GroupSite	*.infosat.net		<a href=http://www.infosat.co.za>Infosat ZA</a>
HideSite	*.infosat.net
GroupSite	*.infoserv.net		ISP: infoserv (no website) MX?</a> 
HideSite	*.infoserv.net
GroupSite	*.infovia.com.ar	<a href=http://www.telefonica.com.ar>ISP: Infovia AR</a>
HideSite	*.infovia.com.ar
GroupSite	*.infoweb.ne.jp		<a href=http://www.infoweb.ne.jp>ISP: Infoweb @nifty JP</a>
HideSite	*.infoweb.ne.jp
GroupSite	*.inicia.es		<a href=http://www.inicia.es>ISP: Inicia ES</a>
HideSite	*.inicia.es
GroupSite	*.inktomi.com		<a href=http://www.inktomi.com/>Spider: Inktomi US</a>
HideSite	*.inktomi.com
GroupSite	*.inktomisearch.com	<a href=http://www.inktomi.com/>Spider: Inktomi US</a>
HideSite	*.inktomisearch.com
GroupSite	*.inria.fr		Ins. N. Recherche Informatique Et Automatique www.inria.fr FR
HideSite	*.inria.fr
GroupSite	*.insa-lyon.fr		<a href=http://www.insa-lyon.fr>EDU: I.N. Sciencie A. FR</a>
HideSite	*.insa-lyon.fr
GroupSite	*.insightbb.com		<a href=http://www.insightbb.com>ISP: insightBB US</a>
HideSite	*.insightbb.com
GroupSite	*.institutoleloir.org.ar	<a href=http://www.institutoleloir.org.ar>EDU: LeLoir AR</a>
HideSite	*.institutoleloir.org.ar
GroupSite	*.intec.edu.do		<a href=http://www.intec.edu.do>EDU: I.Tec. Santo Domin. DO</a>
HideSite	*.intec.edu.do
GroupSite	*.intel.com		<a href=http://www.intel.com>Intel</a>
HideSite	*.intel.com
GroupSite	*.inta.es		Instituto Nacional de Tecnica Aerospacial inta.es ES
HideSite	*.inta.es
GroupSite	*.intelnet.net.gt	<a href=http://www.intelnet.net.gt>ISP: Intelnett GT</a>
HideSite	*.intelnet.net.gt
GroupSite	*.inter.net.il 		<a href=http://www.msn.net.il>ISP: MSN</a>
HideSite	*.inter.net.il
GroupSite	*.interbgc.com		<a href=http://www.interbgc.com>ISP: InterBGCom RU</a>
HideSite	*.interbgc.com
GroupSite	*.interbusiness.it	<a href=http://www.interbusiness.it>ISP: 191 IT</a>
HideSite	*.interbusiness.it
GroupSite	*.intermonde.net 	<a href=http://www.intermonde.net>ISP: Intermonde</a>
HideSite	*.intermonde.net
GroupSite	*.internet.com		<a href=http://www.internet.com/>ISP: Internet.com</a>
HideSite	*.internet.com
GroupSite	*.internode.on.net	<a href=http://www.internode.on.ne>ISP: Internode AU</a>
HideSite	*.internode.on.net
GroupSite	*.interpacket.net	<a href=http://www.interpacket.net>ISP: Verestar US</a>
HideSite	*.interpacket.net
GroupSite	*.iol.cz		<a href=http://www.iol.cz>ISP: Cesky Tel. Sluzbi - IOL CZ</a>
HideSite	*.iol.cz
GroupSite	*.iol.ie		<a href=http://www.iol.ie>ISP: Ireland On-Line IE</a>
HideSite	*.iol.ie
GroupSite	*.ip.pt			<a href=http://www.ip.pt>ISP: Novis PT</a>
HideSite	*.ip.pt
GroupSite	*.ip-pluggen.com	ISP: ip-pluggen.com (no website) ??
HideSite	*.ip-pluggen.com
GroupSite	*.ipcom.comunitel.net	<a href=http://www.comunitel.es>ISP: Comunitel ES</a>
HideSite	*.ipcom.comunitel.net
GroupSite	*.ipicyt.edu.mx		<a href=http://www.ipicyt.edu.mx>EDU: I. Potosi Invest. MX</a>
HideSite	*.ipicyt.edu.mx
GroupSite	*.iplannetworks.com	iplannetworks (no website)
HideSite	*.iplannetworks.com
GroupSite	*.iplannetworks.net	iplannetworks (no website)
HideSite	*.iplannetworks.net
GroupSite	*.iprimus.net.au	<a href=http://www.iprimus.net.au>ISP: iPrimus AU</a>
HideSite	*.iprimus.net.au
GroupSite	*.is.net.ar		ISP: Internet Solutions is.net.ar (no website) AR
HideSite	*.is.net.ar
GroupSite	*.is.co.za		<a href=http://www.is.co.za>Internet Solutions ZA</a>
HideSite	*.is.co.za
GroupSite	*.ispdata.co.za		<a href=http://www.ataris.co.za>ISP: ISP Data ZA</a>
HideSite	*.ispdata.co.za
GroupSite	*.irta.es		<a href=http://www.irta.es>IRTA Research Inst. ES</a>
HideSite	*.irta.es
GroupSite	*.itesm.mx		<a href=http://www.itesm.mx>EDU: Inst.T. Monterrey MX</a>
HideSite	*.itesm.mx
GroupSite	*.itmexicali.edu.mx	<a href=http://www.itmexicali.edu.mx>EDU: I.T. Mexic MX</a>
HideSite	*.itmexicali.edu.mx
GroupSite	*.jazztel.es		<a href=http://www.jazztel.es>ISP: Jazztel ES</a>
HideSite	*.jazztel.es
GroupSite	*.jcu.cz		<a href=http://www.jcu.cz>EDU: Jihoceska U. CZ</a>
HideSite	*.jcu.cz
GroupSite	*.ji-net.com		<a href=http://www.ji-net.com>ISP: Ji-NET TH</a>
HideSite	*.ji-net.com
GroupSite	*.juntadeandalucia.es	GOV: Junta de andalucia www.juntadeandalucia.es ES
HideSite	*.juntadeandalucia.es
GroupSite	*.kabel-badenwuerttemberg.de    <a href=http://www.kabel-badenwuerttemberg.de>ISP: Kabel-BW DE</a>
HideSite	*.kabel-badenwuerttemberg.de
GroupSite	*.kabelfoon.nl		<a href=http://www.kabelfoon.nl>ISP: Kabelfoon NL</a>
HideSite	*.kabelfoon.nl
GroupSite	*.kablonet.net.tr	<a href=http://www.kablonet.net.tr>ISP: KabloNet TR</a>
HideSite	*.kablonet.net.tr
GroupSite	*.kaist.ac.kr		EDU: Korean I. of Science &amp; Tech. www.kaist.ac.kr KR</a>
HideSite	*.kaist.ac.kr
GroupSite	*.kbtelecom.net.tw	<a href=http://www.kbtelecom.net>ISP: KBtelecom TW</a>
HideSite	*.kbtelecom.net.tw
GroupSite	*.kdrs.de		<a href=http://www.kdrs.de>ISP: KDRS/RZRS DE</a>
HideSite	*.kdrs.de
GroupSite	*.kineticmds.com	<a href=http://www.kineticmds.com>Kinetic Marketing ZA</a>
HideSite	*.kineticmds.com
GroupSite	*.kingsley.co.za	<a href=http://www.kingsley.co.za>ISP: Kingskey ZA</a>
HideSite	*.kingsley.co.za
GroupSite	*.klm.nl		<a href=http://www.klm.nl>KLM Royal Dutch Airlines NL</a>
HideSite	*.klm.nl
GroupSite	*.knett.com.ar		<a href=http://www.knett.com.ar>ISP: Knet AR</a>
HideSite	*.knett.com.ar
GroupSite	*.kochi-tech.ac.jp	<a href=http://www.kochi-tech.ac.jp>EDU: Kochi U.Tech. JP</a>
HideSite	*.kochi-tech.ac.jp
GroupSite	*.komatsu.co.jp		<a href=http://www.komatsu.co.jp>Komatsu JP</a>
HideSite	*.komatsu.co.jp
GroupSite	*.kostal.com		<a href=http://www.kostal.com>ISP: Kostal DE</a>
HideSite	*.kostal.com
GroupSite	*.kt.kg			<a href=http://www.kt.kg>ISP: KT KG</a>
HideSite	*.kt.kg
GroupSite	*.kuleuven.ac.be	<a href=http://www.kuleuven.ac.be>EDU: K.U. Leuven BE</a>
HideSite	*.kuleuven.ac.be
GroupSite	*.lacaixa.com		<a href=http://www.lacaixa.com>Bank: La Caixa ES</a>
HideSite	*.lacaixa.com
GroupSite	*.lanacion.com.ar	<a href=http://www.lanacion.com.ar>ISP: La Nacion AR</a>
HideSite	*.lanacion.com.ar
GroupSite	*.lancs.ac.uk		<a href=http://www.lancs.ac.uk>EDU: U. Lancaster US</a>
HideSite	*.lancs.ac.uk
GroupSite	*.lehman.cuny.edu	<a href=http://www.lehman.cuny.edu>EDU: Lehman College US</a>
HideSite	*.lehman.cuny.edu
GroupSite	*.leon.uia.mx		<a href=http://www.leon.uia.mx>EDU: U. Iberoamer. Leon MX</a>
HideSite	*.leon.uia.mx
GroupSite	*.level3.net		<a href=http://www.level3.net>ISP: Level3 US</a>
HideSite	*.level3.net
GroupSite	*.lf.edu.uy		<a href=http://www.lf.edu.uy>EDU: Liceo Frances UY</a>
HideSite	*.lf.edu.uy
GroupSite	*.libertysurf.fr	<a href=http://www.libertysurf.fr>ISP: Tiscali-Libertysurf FR</a>
HideSite	*.libertysurf.fr
GroupSite	*.libertysurf.net	<a href=http://www.libertysurf.fr>ISP: Tiscali-Libertysurf FR</a>
HideSite	*.libertysurf.net
GroupSite	*.livjm.ac.uk		<a href=http://www.livjm.ac.uk>EDU: U. Liverpool J.M. UK</a>
HideSite	*.livjm.ac.uk
GroupSite	*.lleida.net		<a href=http://www.lleida.net>ISP: Lleida.net ES</a>
HideSite	*.lleida.net
GroupSite	*.looksmart.com		<a href=http://www.looksmart.com>Spider: Looksmart **</a>
HideSite	*.looksmart.com
GroupSite	*.losrios.edu		<a href=http://www.losrios.edu>EDU: Los Rios College US</a>
HideSite	*.losrios.edu
GroupSite	*.lmu.ac.uk		<a href=http://www.lmu.ac.uk>EDU: Leeds Metro. UK</a>
HideSite	*.lmu.ac.uk
GroupSite	*.lrz-muenchen.de	<a href=http://www.lrz-muenchen.de>Leibniz-Rechenzentrum DE</a>
HideSite	*.lrz-muenchen.de
GroupSite	*.lucent.com		<a href=http://www.lucent.com>ISP: Lucent US</a>
HideSite	*.lucent.com
GroupSite	*.marketscore.com	<a href=http://www.marketscore.com>ISP: MarketScore **</a>
HideSite	*.marketscore.com
GroupSite	*.maxnet.co.nz		<a href=http://www.maxnet.co.nz>ISP: Maxnet NZ</a>
HideSite	*.maxnet.co.nz
GroupSite	*.maxnet.net.nz		<a href=http://www.maxnet.co.nz>ISP: Maxnet NZ</a>
HideSite	*.maxnet.net.nz
GroupSite	*.maxonline.com.sg	<a href=http://www.maxonline.com>ISP: Maxonline SG</a>
HideSite	*.maxonline.com.sg
GroupSite	*.mcmtelecom.com.mx	<a href=http://www.mcmtelecom.com.mx>ISP: MCM Tele. MX</a>
HideSite	*.mcmtelecom.com.mx
GroupSite	*.cnice.mecd.es		<a href=http://www.cnice.mecd.es>EDU: CNICE ES</a>
HideSite	*.cnice.mecd.es
GroupSite	*.mecd.es		<a href=http://www.mecd.es>EDU: Ministerio Edu. Cien. ES</a>
HideSite	*.mecd.es
GroupSite	*.medtelecom.net 	<a href=http://www.medtelecom.net>ISP: Auna ES</a>
HideSite	*.medtelecom.net 
GroupSite	*.megalink.com		<a href=http://www.megalink.com>ISP: Megalink BO</a>
HideSite	*.megalink.com
GroupSite	*.megared.net.mx	<a href=http://www.megared.net.mx>ISP: Megared MX</a>
HideSite	*.megared.net.mx
GroupSite	*.menta.net		<a href=http://www.menta.net>ISP: Menta ES</a>
HideSite	*.menta.net
GroupSite	*.mercury.net		<a href=http://www.mercury.net>ISP: Mercury US</a>
HideSite	*.mercury.net
GroupSite	*.metconnect.net	<a href=http://www.metconnect.net>ISP: MetTel ??</a>
HideSite	*.metconnect.net
GroupSite	*.metropolis-inter.com	<a href=http://www.metropolis.cl>ISP: Metropolis CL</a>
HideSite	*.metropolis-inter.com
GroupSite	*.metrorail.co.za	<a href=http://www.metrorail.co.za>Metrorail ZA</a>
HideSite	*.metrorail.co.za
GroupSite	*.metrored.net.mx	<a href=http://www.metrored.net.mx>ISP: MetroRED MX</a>
HideSite	*.metrored.net.mx
GroupSite	*.mexis.net.mx		<a href=http://www.mexis.net.mx>ISP: Mexis MX</a>
HideSite	*.mexis.net.mx
GroupSite	*.microsoft.com		<a href=http://www.microsoft.com>Microsoft US</a>
HideSite	*.microsoft.com
GroupSite	*.mindspring.com	<a href=http://www.mindspring.com>ISP: Mindspring US</a>
HideSite	*.mindspring.com
GroupSite	*.millicomperu.com.pe	<a href=http://millicomperu.com.pe>ISP: Millicom PE</a>
HideSite	*.millicomperu.com.pe
GroupSite	*.miragorobot.com	<a href=http://www.miragorobot.com>Search Engine: Mirago UK</a>
HideSite	*.miragorobot.com
GroupSite	*.mit.edu		<a href=http://www.mit.edu>EDU: MIT US</a>
HideSite	*.mit.edu
GroupSite	*.mmm.com		<a href=http://www.mmm.com>ISP: 3M Worldwide **</a>
HideSite	*.mmm.com
GroupSite	*.montana.edu		<a href=http://www.montana.edu>EDU: U. Montana State US</a>
HideSite	*.montana.edu
GroupSite	*.motorola.com		<a href=http://www.motorola.com>ISP: Motorola **</a>
HideSite	*.motorola.com
GroupSite	*.mrse.com.ar		mrse.com.ar (Telefonica) AR
HideSite	*.mrse.com.ar
GroupSite	msnbot.msn.com		Search: MSN msnbot.msn.com **
HideSite	msnbot.msn.com
GroupSite	*.mtu-net.ru		<a href=http://www.mtu-net.ru>ISP Tochka RU</a>
HideSite	*.mtu-net.ru
GroupSite	*.mundivia.es		<a href=http://www.mundivia.es>ISP: Mundivia ES</a>
HideSite	*.mundivia.es
GroupSite	*.search.msn.com	<a href=http://search.msn.com>Spider: MSN **</a>
HideSite	*search.msn.com
GroupSite	*.mts.net		<a href=http://www.mts.net>ISP: myWinnipeg US</a>
HideSite	*.mts.net
GroupSite	*.multikabel.nl		<a href=http://www.multikabel.nl>ISP: Multikabel NL</a>
HideSite	*.multikabel.nl
GroupSite	*.mutualseg.cl		<a href=http://www.mutualseg.cl>mutualseg CL</a>
HideSite	*.mutualseg.cl
GroupSite	*.mweb.co.za		<a href=http://www.mweb.co.za>ISP: Mweb ZA</a>
HideSite	*.mweb.co.za
GroupSite	*.mylinuxisp.com	<a href=http://www.mylinuxisp.com>ISP: mylinuxisp US</a>
HideSite	*.mylinuxisp.com
GroupSite	*.nais.com		<a href=http://www.nais>EDU: NAIS Independient Schools US</a>
HideSite	*.nais.com
GroupSite	*.nameintel.com		<a href=http://www.nameintel.com>Search: WHOIS.SC **</a>
HideSite	*.nameintel.com
GroupSite	*.nerim.net		<a href=http://www.nerim.net>ISP: Nerim/Mangoosta FR</a>
HideSite	*.nerim.net
GroupSite	*.netcabo.pt		<a href=http://www.netcabo.pt>ISP: TV Cabo PT</a>
HideSite	*.netcabo.pt
GroupSite	*.netcom.ca		<a href=http://www.netcom.ca>ISP: Allstream CA</a>
HideSite	*.netcom.ca
GroupSite	*.netcologne.de		<a href=http://www.netcologne.de>ISP: NetCologne DE</a>
HideSite	*.netcologne.de
GroupSite	*.netcraft.com		<a href=http://www.netcraft.com>Netcraft US</a>
HideSite	*.netcraft.com
GroupSite	*.netizen.com.ar	<a href=http://www.netizen.com.ar>ISP: Netizen AR</a>
HideSite	*.netizen.com.ar
GroupSite	*.netservice.com.mx	<a href=http://www.netservice.com.mx>ISP: NetService MX</a>
HideSite	*.netservice.com.mx
GroupSite	*.netvigator.com	<a href=http://www.netvigator.com>Netvigator</a>
HideSite	*.netvigator.com
GroupSite	*.netvision.net.il	<a href=http://www.netvision.net.il>ISP: Netvision</a>
HideSite	*.netvision.net.il
GroupSite	*.newcastle.edu.au	<a href=http://www.newcastle.edu.au>EDU: U. New. AU</a>
HideSite	*.newcastle.edu.au
GroupSite	*.newsisfree.com	<a href=http://www.newsisfree.com>Spider: Newsisfree.com</a>
HideSite	*.newsisfree.com
GroupSite	*.nikhef.nl		EDU: Nat. I. Nuclear Physics, High Energy www.nikhef.nl NL
HideSite	*.nikhef.nl
GroupSite	*.nildram.co.uk		<a href=http://www.nildram.co.uk>ISP: Nildram UK</a>
HideSite	*.nildram.co.uk
GroupSite	*.nippan.co.jp		<a href=http://www.nippan.co.jp>ISP: nippan JP</a>
HideSite	*.nippan.co.jp
GroupSite	*.noos.fr		<a href=http://www.noos.fr>ISP: Noos FR</a>
HideSite	*.noos.fr
GroupSite	*.noos.net		<a href=http://www.noos.fr>ISP: Noos FR</a>
HideSite	*.noos.net
GroupSite	*.nordnet.fr		<a href=http://www.nordnet.fr>ISP: NordNet FR</a>
HideSite	*.nordnet.fr
GroupSite	*.northwestern.edu	<a href=http://northwestern.edu>EDU: U. Northwestern US</a>
HideSite	*.northwestern.edu
GroupSite	*.nova.edu		<a href=http://www.nova.edu>EDU: U. Nova US</a>
HideSite	*.nova.edu
GroupSite	*.novared.cl		<a href=http://www.novared.cl>ISP: Novared CL</a>
HideSite	*.novared.cl
GroupSite	*.novell.com		<a href=http://www.novell.com>Novell</a>
HideSite	*.novell.com
GroupSite	*.nsc.ru		<a href=http://www.nsc.ru>EDU: Academic Internet RU</a>
HideSite	*.nsc.ru
GroupSite	*.ntcu.net		<a href=http://www.ntcu.edu.tw>EDU: ntcu TW</a>
HideSite	*.ntcu.net
GroupSite	*.ntl.com		<a href=http://www.ntl.com>ISP: NTL UK</a>
HideSite	*.ntl.com
GroupSite	*.ntli.net		<a href=http://www.ntl.com>ISP: NTL UK</a>
HideSite	*.ntli.net
GroupSite	*.ntt.co.jp		<a href=http://www.ntt.co.jp>ISP: NTT Group JP</a>
HideSite	*.ntt.co.jp
GroupSite	*.ntu.edu.tw		<a href=http://www.ntu.edu.tw>EDU: Nat. Taiwan U. TW</a>
HideSite	*.ntu.edu.tw
GroupSite	*.numericable.fr	<a href=http://www.numericable.fre>ISP: Numericable FR</a>
HideSite	*.numericable.fr
GroupSite	*.nuvox.net		<a href=http://www.nuvox.net>ISP: NuVox US</a>
HideSite	*.nuvox.net
GroupSite	*.nyiso.com		<a href=http://www.nyiso.com>ISP: NYISO UK</a>
HideSite	*.nyiso.com
GroupSite	*.nyit.edu		<a href=http://www.nyit.edu>EDU: NY Inst. of Tech. US</a>
HideSite	*.nyit.edu
GroupSite	*.nytimes.com		<a href=http://www.nytimes.com>New York Times</a>
HideSite	*.nytimes.com
GroupSite	*.nyu.edu		<a href=http://www.nyu.edu>EDU: U. New York US</a>
HideSite	*.nyu.edu
GroupSite	*.oberlin.edu		<a href=http://www.oberlin.edu>EDU: Berlin Online DE</a>
HideSite	*.oberlin.edu
GroupSite	*.ocn.ne.jp		<a href=http://www.ocn.ne.jp>ISP/Spider: OCN</a>
HideSite	*.ocn.ne.jp
GroupSite	*.oddpost.com		<a href=http://www.oddpost.com>Spider: Oddpost</a>
HideSite	*.oddpost.com
GroupSite	*.odn.ne.jp		<a href=http://www.odn.ne.jp>ISP: Odn JP</a>
HideSite	*.odn.ne.jp
GroupSite	*.ods.co.za		<a href=http://www.ods.co.za>ISP: ODS ZA</a>
HideSite	*.ods.co.za
GroupSite	*.oh.us			oh.us (no website)
HideSite	*.oh.us
GroupSite	*.oyster.co.uk		<a href=http://www.oyster.co.uk>Oyster EmailSecurity UK</a>
HideSite	*.oyster.co.uk
GroupSite	*.onebb.com		<a href=http://www.onebb.com>ISP: onebb HK?</a>
HideSite	*.onebb.com
GroupSite	*.online-age.net	ISP: online-age.net (no website) ??
HideSite	*.online-age.net
GroupSite	*.onolab.com		<a href=http://www.ono.es>ISP: ONO ES</a>
HideSite	*.onolab.com
GroupSite	*.ono.com		<a href=http://www.ono.es>ISP: ONO ES</a>
HideSite	*.ono.com
GroupSite	*.omnicast.net		<a href=http://www.omnicast.net>ISP: Omnicast</a>
HideSite	*.omnicast.net
GroupSite	*.optonline.net		<a href=http://www.optonline.net>ISP: Optimum Online</a>
HideSite	*.optonline.net
GroupSite	*.optusnet.com.au	<a href=http://www.optusnet.au>ISP: Optusnet AU</a>
HideSite	*.optusnet.com.au
GroupSite	*.overture.com		<a href=http://www.overture.com>Spider: Overture **</a>
HideSite	*.overture.com
GroupSite	*.oxy.com		<a href=http://www.oxy.com>OXY Petroleum/Chemicals US</a>
HideSite	*.oxy.com
GroupSite	*.pacbell.net		<a href=http://www.pacbell.net>ISP: PacBell US</a>
HideSite	*.pacbell.net
GroupSite	*.paradise.net.nz	<a href=http://www.paradise.net.nz>ISP: Paradise NZ</a>
HideSite	*.paradise.net.nz
GroupSite	*.peugeot.fr		<a href=http://www.peugeot.fr>Peugeot FR</a>
HideSite	*.peugeot.fr
GroupSite	*.pfeilheim.sth.ac.at	EDU: pfeilheim www.pfeilheim.sth.ac.at AT
HideSite	*.pfeilheim.sth.ac.at
GroupSite	*.picsearch.com		<a href=http://www.picsearch.com>Spider: PicSearch</a>
HideSite	*.picsearch.com
GroupSite	*.piedmont.edu		<a href=http://www.piedmont.edu>EDU: Piedmont US</a>
HideSite	*.piedmont.edu
GroupSite	*.pla.net.py		<a href=http://www.pla.net.py>ISP: Planet PY</a>
HideSite	*.pla.net.py
GroupSite	*.plala.or.jp		<a href=http://www.plala.or.jp>ISP: Plala JP</a>
HideSite	*.plala.or.jp
GroupSite	*.planet.nl		<a href=http://www.planet.nl>ISP: Planet NL</a>
HideSite	*.planet.nl
GroupSite	*.pol.co.uk 		<a href=http://www.pol.co.uk>ISP: Energis UK</a>
HideSite	*.pol.co.uk
GroupSite	*.polimi.it		<a href=http://www.polimi.it>EDU: Politec. Milano IT</a>
HideSite	*.polimi.it
GroupSite	*.porta.net		<a href=http://www.porta.net>ISP: Porta EC</a>
HideSite	*.porta.net
GroupSite	*.pppool.de		<a href=http://www.freenet.de>ISP: freenet.de DE</a>
HideSite	*.pppool.de
GroupSite	*.prairieinet.net	<a href=http://www.prairieinet.net>ISP: Prairieinet US</a>
HideSite	*.prairieinet.net
GroupSite	*.prima.net.ar		<a href=http://www.grupoclarin.com.ar/>PRIMA AR</a>
HideSite	*.prima.net.ar
GroupSite	*.primus.ca		<a href=http://www.primus.ca>ISP: Primus CA</a>
HideSite	*.primus.ca
GroupSite	*.prodigy.net.mx	<a href=http://www.prodigy.net.mx>ISP: MSN MX</a>
HideSite	*.prodigy.net.mx
#GroupSite	*.prodigy.		<a href=http://www.prodigy.net>ISP: Prodigy</a>
#HideSite	*.prodigy.
GroupSite	*.proxad.net		<a href=http://www.proxad.net>ISP: ProXad-Free FR</a>
HideSite	*.proxad.net
GroupSite	*.prod-empresarial.com.mx	prod-empresarial.com.mx (no website) MX
HideSite	*.prod-empresarial.com.mx
GroupSite	*.prod-infinitum.com.mx	prod-infinitum.com.mx (no website) MX
HideSite	*.prod-infinitum.com.mx
GroupSite	*.prserv.net		<a href=http://services.prserv.net>ISP: prserv AT&amp;T **</a>
HideSite	*.prserv.net
GroupSite	*.psi.net		<a href=http://www.psinet.com>ISP_ PSINet **</a>
HideSite	*.psi.net
GroupSite	*.psi.net.pa		<a href=http://www.psi.net.pa>ISP: Alianza Viva PA</a>
HideSite	*.psi.net.pa
GroupSite	*.psinetpa.net		<a href=http://www.psinet.com>ISP: PSINet **</a>
HideSite	*.psinetpa.net
GroupSite	*.arl.psu.edu		EDU: Applied Research lab Penn State U. www.arl.psu.edu
HideSite	*.arl.psu.edu
GroupSite	*.psu.edu		<a href=http://www.psu.edu>EDU: U. Penn State US</a>
HideSite	*.psu.edu
GroupSite	*.ptprime.net		ISP: ptprime.net (no website) ??
HideSite	*.ptprime.net
GroupSite	*.prtc.net		<a href=http://www.prtc.net>ISP: Coqui PR</a>
HideSite	*.prtc.net
GroupSite	*.public.lib.ga.us	Public Internet access in US libraries public.lib.ga.us US
HideSite	*.public.lib.ga.us
GroupSite	*.pue.uia.mx		<a href=http://www.pue.uia.mx>EDU: U. Iberoamer. MX</a>
HideSite	*.pue.uia.mx
GroupSite	*.punto.net.ec		<a href=http://www.punto.net.ec>ISP: PuntoNet EC</a>
HideSite	*.punto.net.ec
GroupSite	*.qsc.de		<a href=http://www.qsc.de>ISP: QSC DE</a>
HideSite	*.qsc.de
GroupSite	*.qx.net		<a href=http://www.qx.net>ISP: QX US</a>
HideSite	*.qx.net
GroupSite	*.qwest.net		<a href=http://www.qwest.net>ISP: Qwest US</a>
HideSite	*.qwest.net
GroupSite	*.racsa.co.cr		<a href=http://www.racsa.co.cr>ISP: Racsa CR</a>
HideSite	*.racsa.co.cr
GroupSite	*.rain.fr		<a href=http://www.rain.fr>ISP: Transpac FR</a>
HideSite	*.rain.fr
GroupSite	*.rcn.com		<a href=http://www.rcn.com>ISP: RCN</a>
HideSite	*.rcn.com
GroupSite	*.rasserver.net		<a href=http://icgcomm.com/products/isp/ras.asp>ISP: ICG</a>
HideSite	*.rasserver.net
GroupSite	*.rediris.es		<a href=http://www.rediris.es>EDU: RedIRIS ES</a>
HideSite	*.rediris.es
GroupSite	*.red.mundo-r.com	<a href=http://www.mundo-r.com>ISP: mundo-R Galicia ES</a>
HideSite	*.red.mundo-r.com
GroupSite	*.retevision.es		<a href=http://www.retevision.es>ISP: Retevision ES</a>
HideSite	*.retevision.es
GroupSite	*.reuna.cl		<a href=http://www.reuna.cl>EDU: Red Universi. Nacional CL</a>
HideSite	*.reuna.cl
GroupSite	*.rima-tde.net		<a href=http://www.telefonica.es>ISP: Telefonica ES</a>
HideSite	*.rima-tde.net
GroupSite	*.rnb.com		<a href=http://www.hsbc.com>ISP: HSBC</a>
HideSite	*.rnb.com
GroupSite	*.roedu.net		<a href=http://www.roedu.net>EDU: Romanian Edu. Net. RO</a>
HideSite	*.roedu.net
GroupSite	*.rogers.com		<a href=http://www.rogers.com>ISP: Rogers Broadband</a>
HideSite	*.rogers.com
GroupSite	*.rol.ru		<a href=http://www.rol.ru>ISP: rol RU</a>
HideSite	*.rol.ru
GroupSite	*.routit.net		ISP: *.routit.net **</a>
HideSite	*.routit.net
GroupSite	*.rr.com		<a href=http://www.rr.com>ISP: Road Runner</a>
HideSite	*.rr.com
GroupSite	*.rtve.es		<a href=http://www.rtve.es>RTVE Television Espanola ES</a>
HideSite	*.rtve.es
GroupSite	*.ru.ac.za		<a href=http://www.ru.ac.za>EDU: Rhodes U. ZA</a>
HideSite	*.ru.ac.za
GroupSite	*.rug.nl		<a href=http://www.rug.nl>EDU: Rijks U. Groningen NL</a>
HideSite	*.rug.nl
GroupSite	*.retecal.es		<a href=http://www.retecal.es>ISP: Retecal ES</a>
HideSite	*.retecal.es
GroupSite	*.salleurl.edu		<a href=http://www.salleurl.edu>EDU: La Salle ES</a>
HideSite	*.salleurl.edu
GroupSite	*.sandi.net		<a href=http://www.sandi.net>EDU: San Diego Schools US</a>
HideSite	*.sandi.net
GroupSite	*.santpau.es		<a href=http://www.santpau.es>Hospital Sant Pau ES</a>
HideSite	*.santpau.es
GroupSite	*.sarenet.es		<a href=http://www.sarenet.es>ISP: Sarenet ES</a>
HideSite	*.sarenet.es
GroupSite	*.sas.com		<a href=http://www.sas.com>SAS Software company **</a>
HideSite	*.sas.com
GroupSite	*.sasi.fr		<a href=http://www.sasi.fr>ISP: CEGI/SASI/SELOG FR</a>
HideSite	*.sasi.fr
GroupSite	*.satnet.net		<a href=http://www.satnet.net>ISP/Portal: Satnet EC</a>
HideSite	*.satnet.net
GroupSite	*.savia.es		<a href=http://www.savia.es>Savia AMADEUS ES</a>
HideSite	*.savia.es
GroupSite	*.scs.es		<a href=http://www.scs.es>GOV: Servei Catala Salut ES</a>
HideSite	*.scs.es
GroupSite	*.seanet.ro		<a href=http://www.seanet.ro>ISP: Seanet RO</a>
HideSite	*.seanet.ro
GroupSite	*.secyt.gov.ar		<a href=http://www.secyt.gov.ar>GOV: secyt Research AR</a>
HideSite	*.secyt.gov.ar
GroupSite	*.seeder.net		<a href=http://www.seeder.net>Portal: Seeder CH?TW?</a>
HideSite	*.seeder.net
GroupSite	*.sep.gob.mx		<a href=http://www.sep.gob.mx>EDU: Sec.Educ.Publica MX</a>
HideSite	*.sep.gob.mx
GroupSite	*.serendipity.org.za	<a href=http://www.serendipity.org.za>ISP: Seren. ZA</a>
HideSite	*.serendipity.org.za
GroupSite	*.shawcable.net		<a href=http://www.shawcable.com>ISP: Shawcable CA</a>
HideSite	*.shawcable.net
GroupSite	*.sinectis.com.ar	<a href=http://www.sinectis.com.ar>ISP: UOL AR</a>
HideSite	*.sinectis.com.ar
GroupSite	*.sint.pl		<a href=http://www.sint.pl>ISP: Swiat PL</a>
HideSite	*.sint.pl
GroupSite	*.skanova.com		<a href=http://www.skanova.com>ISP: Skanova NO</a>
HideSite	*.skanova.com
GroupSite	*.skoda.cz		<a href=http://www.skoda.cz>Skoda CZ</a>
HideSite	*.skoda.cz
GroupSite	*.sky.net.co		<a href=http://www.sky.net.co>ISP: Sky Internet CO</a>
HideSite	*.sky.net.co
GroupSite	*.skyinet.net		<a href=http://www.skyinet.net>ISP: Sky Internet **</a>
HideSite	*.skyinet.net
GroupSite	*.skynet.be		<a href=http://www.skyinet.net>ISP: Sky Internet BE</a>
HideSite	*.skynet.be
GroupSite	*.smals-mvm.be		<a href=http://www.smals-mvm.be>ISP: smals-mvm BE</a>
HideSite	*.smals-mvm.be
GroupSite	*.softwin.ro		<a href=http://www.softwin.ro>ISP: Softwin RO</a>
HideSite	*.softwin.ro
GroupSite	*.solmelia.es		<a href=http://www.solmelia.es>Hoteles Sol Melia ES</a>
HideSite	*.solmelia.es
GroupSite	*.sonera		<a href=http://www.telia.com>ISP: Telia</a>
HideSite	*.sonera
GroupSite	*.so-net.com.hk		<a href=http://www.so-net.com.hk>ISP: So-net Sony HK</a>
HideSite	*.so-net.com.hk
GroupSite	*.sonic.net		<a href=http://www.sonic.net>ISP: Sonic.net</a>
HideSite	*.sonic.net
GroupSite	*.sovintel.ru		<a href=http://www.sovintel.ru>ISP: Sovintel RU</a>
HideSite	*.sovintel.ru
GroupSite	*.sp.edu.sg		<a href=http://www.sp.edu.sg></a>
HideSite	*.sp.edu.sg
GroupSite	*.speakeasy.net		<a href=http://www.speakeasy.net>ISP: SpeakEasy NYC US</a>
HideSite	*.speakeasy.net
GroupSite	*.spectranet.com	<a href=http://www.spectranet.com>ISP: spectranet IN</a>
HideSite	*.spectranet.com
GroupSite	*.speedy.net.pe		ISP: Telefonica?? speedy.net.pe (no website) PE
HideSite	*.speedy.net.pe
GroupSite	*.speedy.com.ar		ISP: speedy.com.ar (no website) AR
HideSite	*.speedy.com.ar
GroupSite	*.sprint-canada.net	<a href=http://www.sprint.com>ISP: Sprint</a>
HideSite	*.sprint-canada.net
GroupSite	*.sprint-hsd.net	<a href=http://www.sprint.com>ISP: Sprint</a>
HideSite	*.sprint-hsd.net
GroupSite	*.sprint-v.com.ru	<a href=http://www.sprint-v.com.ru>ISP: Sprint-v RU</a>
HideSite	*.sprint-v.com.ru
GroupSite	*.star.net.uk		<a href=http://www.star.net.uk>ISP: Star UK</a>
HideSite	*.star.net.uk
GroupSite	*.starband.net		<a href=http://www.starband.net>ISP: StarBand US</a>
HideSite	*.starband.net
GroupSite	*.strath.ac.uk		<a href=http://www.strath.ac.uk>EDU: U. Strathclyde UK</a>
HideSite	*.strath.ac.uk
GroupSite	*.storm.co.za		<a href=http://www.storm.co.za>ISP: Storm ZA</a>
HideSite	*.storm.co.za
GroupSite	*.sun.ac.za		<a href=http://www.sun.ac.za>EDU: U. Stellenbosch ZA</a>
HideSite	*.sun.ac.za
GroupSite	*.surnet.cl		<a href=http://www.surnet.cl>ISP: Surnet CL</a>
HideSite	*.surnet.cl
GroupSite	*.sprint		<a href=http://www.sprint.com>ISP: Sprint</a>
HideSite	*.sprint
GroupSite	*.sp-online.de		<a href=http://www.sp-online.de>ISP: SP-online DE</a>
HideSite	*.sp-online.de
GroupSite	*.stv.ru		<a href=http://www.stv.ru>ISP: stavropol RU</a>
HideSite	*.stv.ru
GroupSite	*.sun.com		<a href=http://www.sun.com>Sun Microsystems</a>
HideSite	*.sun.com
GroupSite	*.sunderland.ac.uk	<a href=http://www.sunderland.ac.uk>EDU: Sunderl. UK</a>
HideSite	*.sunderland.ac.uk
GroupSite	*.sunysb.edu		<a href=http://www.sunysb.edu>EDU: U. Stony Brook US</a>
HideSite	*.sunysb.edu
GroupSite	*.supercabletv.net.co	ISP: www.supercabletv.net.co CO)
HideSite	*.supercabletv.net.co
GroupSite	*.supercable.es		<a href=http://www.supercable.es>ISP: AUNA ES</a>
HideSite	*.supercable.es
GroupSite	*.surfear.com		<a href=http://www.surfear.com>Search: surfear.com ES</a>
HideSite	*.surfear.com
GroupSite	*.svwh.net		<a href=http://www.svwh.net>ISP: Silicon Valley US</a>
HideSite	*.svwh.net
GroupSite	*.swbell.net		<a href=http://www.swbell.net>ISP: SouthWestern Bell</a>
HideSite	*.swbell.net
GroupSite	*.swing.be		<a href=http://www.swing.be>ISP Skynet BE</a>
HideSite	*.swing.be
GroupSite	*.swipnet.se		<a href=http://www.swipnet.se>ISP: swip.net SE</a>
HideSite	*.swipnet.se
GroupSite	*.swisscom.com		<a href=http://www.swiss.com>ISP: SwissCOM CH</a>
HideSite	*.swisscom.com
GroupSite	*.sympatico.ca		<a href=http://www.sympatico.ca>ISP: Sympatico CA</a>
HideSite	*.sympatico.ca   
GroupSite	*.sztaki.hu		<a href=http://www.sztaki.hu>Spider: sztaki.hu</a>
HideSite	*.sztaki.hu
GroupSite	*.sz-sb.de		<a href=http://www.sz-sb.de/>ISP: Saarland DE</a>
HideSite	*.sz-sb.de
GroupSite	*.t-dialin.net		<a href=http://www.t-online.net>ISP: T-Online DE</a>
HideSite	*.t-dialin.net
GroupSite	*.t-ipconnect.de	<a href=http://www.t-online.net>ISP: T-Online DE</a>
HideSite	*.t-ipconnect.de
GroupSite	*.t-m-s.com		<a href=http://www.t-m-s.com>The Marketing Shop ZA</a>
HideSite	*.t-m-s.com
GroupSite	*.t-net.net.ve		<a href=http://www.t-net.et.ve>MIPUNTO.COM VE</a>
HideSite	*.t-net.net.ve
GroupSite	*.t-online.de		<a href=http://www.t-online.de>T-Online DE</a>
HideSite	*.t-online.de
GroupSite	*.t-sysnet.com		ISP: t-sysnet.com (unknown)
HideSite	*.t-sysnet.com
GroupSite	*.tachyon.net		<a href=http://tachyon.net>ISP: Tachyon Networks US</a>
HideSite	*.tachyon.net
GroupSite	*.taisho.co.jp		<a href=http://www.taisho.co.jp>ISP: taisho JP</a>
HideSite	*.taisho.co.jp
GroupSite	*.tamu.edu		<a href=http://www.tamu.edu>EDU: U. Texas A&amp;M US</a>
HideSite	*.tamu.edu
GroupSite	*.technion.ac.il	<a href=http://www.technion.ac.il>EDU: Technion U. IL</a>
HideSite	*.technion.ac.il
GroupSite	*.techtelnet.net	<a href=http://www.techtelnet.net>ISP: Techtelnet AR</a>
HideSite	*.techtelnet.net
GroupSite	*.tektrans.com		<a href=http://www.tektrans.com>TEK Translations US</a>
HideSite	*.tektrans.com
GroupSite	*.tel.net.ba		<a href=http://www.tel.net.ba>ISP: HT::NET BA</a>
HideSite	*.tel.net.ba
GroupSite	*.telcel.net.ve		<a href=http://www.telcel.net.ve>ISP: Telcel VE</a>
HideSite	*.telcel.net.ve
GroupSite	*.telconet.net		<a href=http://www.telconet.net>ISP: Telconet EC</a>
HideSite	*.telconet.net
GroupSite	*.tele.dk		<a href=http://www.tele.dk>ISP: TDC DK</a>
HideSite	*.tele.dk
GroupSite	*.tele2.fr		<a href=http://www.tele2.fr>ISP: Tele2 FR</a>
HideSite	*.tele2.fr
GroupSite	*.tele2.nl		<a href=http://www.tele2.nl>ISP: Tele2 NL</a>
HideSite	*.tele2.nl
GroupSite	*.telecable.es		<a href=http://www.telecable.es>ISP: Telecable ES</a>
HideSite	*.telecable.es
GroupSite	*.telecom.net.ar	<a href=http://www.telecom.com.ar>ISP: Telecom AR</a>
HideSite	*.telecom.net.ar
GroupSite	*.telecomitalia.it	<a href=http://www.telecomitalia.it>ISP: Telecom IT</a>
HideSite	*.telecomitalia.it
GroupSite	*.telefonica-data.net	<a href=http://www.telefonica.es>ISP: Telefonica ES</a>
HideSite	*.telefonica-data.net
GroupSite	*.telemedia.net.mx	<a href=http://www.telemedia.net.mx>ISP: Telemedia MX</a>
HideSite	*.telemedia.net.mx
GroupSite	*.telepacific.net	<a href=http://www.telepacific.com>ISP: Telepacific US</a>
HideSite	*.telepacific.net
GroupSite	*.telefonica-ca.net	<a href=http://www.telefonica-ca.net>ISP: Telefonica SV</a>
HideSite	*.telefonica-ca.net
GroupSite	*.telekom.at		<a href=http://www.telekom.at>ISP: Telekom Austria AT</a>
HideSite	*.telekom.at
GroupSite	*.telemach.net		<a href=http://www.telemach.net>ISP: Telemach SI</a>
HideSite	*.telemach.net
GroupSite	*.telenet.be		<a href=http://www.telenet.be>ISP: Telenet BE</a>
HideSite	*.telenet.be
GroupSite	*.telepac.pt		<a href=http://www.telepac.pt>ISP: Telepac PT</a>
HideSite	*.telepac.pt
GroupSite	*.telesat.com.co	<a href=http://www.telesat.com.com>ISP: Telesat CO</a>
HideSite	*.telesat.com.co
GroupSite	*.telesp.net.br		<a href=http://www.telefonica.es>ISP: Telefonica BR</a>
HideSite	*.telesp.net.br
GroupSite	*.teleweb.at		<a href=http://www.teleweb.at>ISP: Chello AT</a>
HideSite	*.teleweb.at
GroupSite	*.telewest.co.uk	<a href=http://www.telewest.co.uk>ISP: Telewest UK</a>
HideSite	*.telewest.co.uk
GroupSite	*.telia.com		<a href=http://www.telia.com>ISP: Telia SE</a>
HideSite	*.telia.com
GroupSite	*.telkomadsl.com.za	telkomadsl.co.za ZA
HideSite	*.telkomadsl.com.za
GroupSite	*.telkom-ipnet.co.za	<a href=http://www.telkom-ipnet.co.za>ISP: telkom-ip ZA</a>
HideSite	*.telkom-ipnet.co.za
GroupSite	*.telmex.com.pe		<a href=http://www.telmex.com/pe>ISP: Telmex PE</a>
HideSite	*.telmex.com.pe
GroupSite	*.telnor.net 		<a href=http://www.telnor.com.mx>ISP: Telnor MX</a>
HideSite	*.telnor.net 
GroupSite	*.telstraclear.net  <a href=http://www.telstraclear.net>ISP: Telstra AU</a>
HideSite	*.telstraclear.net
GroupSite	*.telus.net		<a href=http://www.telus.net>ISP: Telus US</a>
HideSite	*.telus.net
GroupSite	*.temple.edu		<a href=http://www.temple.edu>EDU: Temple U. On-line **</a>
HideSite	*.temple.edu
GroupSite	*.teoma.com		<a href=http://www.teoma.com>Spider: Teoma US</a>
HideSite	*.teoma.com
GroupSite	*.terralink.de		<a href=http://www.sz-sb.de/>ISP: Terralink DE</a>
HideSite	*.terralink.de
GroupSite	*.terra.cl		<a href=http://www.terra.cl>ISP: Terra CL</a>
HideSite	*.terra.cl
GroupSite	*.terra.cl		<a href=http://www.terra.cl>ISP: Terra CL</a>
HideSite	*.terra.cl
GroupSite	*.terra.es		<a href=http://www.terra.es>ISP: Terra ES</a>
HideSite	*.terra.es
GroupSite	*.terra.net.mx		<a href=http://www.terra.com.mx>ISP: Terra MX</a>
HideSite	*.terra.net.mx
GroupSite	*.tf1.fr		<a href=http://www.tf1.r>ISP: TF1 FR</a>
HideSite	*.tf1.fr
GroupSite	*.the-beach.net		<a href=http://www.charter.com>ISP: Charter **</a>
HideSite	*.the-beach.net
GroupSite	*.theplanet.com		<a href=http://www.theplanet.com>ISP: The Planet US</a>
HideSite	*.theplanet.com
GroupSite	*.thawte.com		<a href=http://www.thawte.com>Thawte Certifying</a>
HideSite	*.thawte.com
GroupSite	*.tht.net		<a href=http://www.tht.net>ISP: THT.net CA</a>
HideSite	*.tht.net
GroupSite	*.thunderstone.com	<a href=http://www.thunderstone.com>ISP: Thunders. US</a>
HideSite	*.thunderstone.com
GroupSite	*.tie.cl		<a href=http://www.tie.cl>ISP: Telefonica CL</a>
HideSite	*.tie.cl
GroupSite	*.tinet.org		<a href=http://www.tinet.org>ISP: Tinet Tarragona ES</a>
HideSite	*.tinet.org
GroupSite	*.tinroof.co.za		<a href=http://www.tinroof.co.za>ISP: Fusion R. ZA</a>
HideSite	*.tinroof.co.za
GroupSite	*.uk.tiscali.com	<a href=http://www.uk.tiscali.com>ISP: Tiscali UK</a>
HideSite	*.uk.tiscali.com
GroupSite	*.tiscali.at		<a href=http://www.tiscali.at>ISP: Tiscali AT</a>
HideSite	*.tiscali.at
GroupSite	*.tiscali.be		<a href=http://www.tiscali.be>ISP: Tiscali BE</a>
HideSite	*.tiscali.be
GroupSite	*.tiscali.ch		<a href=http://www.tiscali.ch>ISP: Tiscali CH</a>
HideSite	*.tiscali.ch
GroupSite	*.tiscali.de		<a href=http://www.tiscali.de>ISP: Tiscali DE</a>
HideSite	*.tiscali.de
GroupSite	*.tiscali.es		<a href=http://www.tiscali.es>ISP: Tiscali ES</a>
HideSite	*.tiscali.es
GroupSite	*.tiscali.fr		<a href=http://www.tiscali.fr>ISP: Tiscali FR</a>
HideSite	*.tiscali.fr
GroupSite	*.tiscali.it		<a href=http://www.tiscali.it>ISP: Tiscali IT</a>
HideSite	*.tiscali.it
GroupSite	*.tiscali.nl		<a href=http://www.tiscali.nl>ISP: Tiscali NL</a>
HideSite	*.tiscali.nl
#GroupSite	*.tiscali		<a href=http://www.tiscali.com>ISP: Tiscali **</a>
#HideSite	*.tiscali
GroupSite	*.tktelekom.pl		<a href=http://www.tktelekom.pl>ISP: tktelekom PL</a>
HideSite	*.tktelekom.pl
GroupSite	*.tnl.net		<a href=http://www.tnl.net>ISP: www.tnl.net</a>
HideSite	*.tnl.net
GroupSite	*.tntech.edu		<a href=http://www.tntech.edu>EDU: U. Tennessee Tech US</a>
HideSite	*.tntech.edu
GroupSite	*.toshiba.co.jp		<a href=http://www.toshiba.co.jp>Toshiba</a>
HideSite	*.toshiba.co.jp
GroupSite	*.tp1rc.edu.tw		<a href=http://www.tp1rc.edu.tw>EDU: Taipei Campuses TW</a>
HideSite	*.tp1rc.edu.tw
GroupSite	*.tpgi.com.au		<a href=http://www.tpgi.com.au>ISP: TPG AU</a>
HideSite	*.tpgi.com.au
GroupSite	*.tpnet.pl		<a href=http://www.tpnet.pl>ISP: TP PL</a>
HideSite	*.tpnet.pl
GroupSite	*.tpv.ceca.es		<a href=http://www.ceca.es>Cajas de Ahorros Conf. ES</a>
HideSite	*.tpv.ceca.es
GroupSite	*.transpac.fr		<a href=http://www.transpac.fr>ISP: Transpac FR</a>
HideSite	*.transpac.fr
GroupSite	*.tricom.net		<a href=http://www.tricom.net>ISP: tricom ??</a>
HideSite	*.tricom.net
GroupSite	*.trip.net		<a href=http://www.trip.net>ISP: Trip Net US</a>
HideSite	*.trip.net
GroupSite	*.tsai.es		<a href=http://www.tsai.es>ISP: Telefonica ES</a>
HideSite	*.tsai.es
GroupSite	*.ttnet.net.tr		<a href=http://www.ttnet.net.tr>ISP: Turk Telekom. TR</a>
HideSite	*.ttnet.net.tr
GroupSite	*.turnitin.com		<a href=http://www.turnitin.com>Spider: Turnitin</a>
HideSite	*.turnitin.com
GroupSite	*.tu-darmstadt.de	<a href=http://www.tu-darmstadt.de>EDU: T.Uni. Darmstadt DE</a>
HideSite	*.tu-darmstadt.de
GroupSite	*.tu-muenchen.de	<a href=http://www.tu-muenchen.de>EDU: T.U.Munchen DE</a>
HideSite	*.tu-muenchen.de
GroupSite	*.tuwien.ac.at		<a href=http://www.tuwien.ac.at>EDU: Technische U. Wien AT</a>
HideSite	*.tuwien.ac.at
GroupSite	*.twtelecom.net		<a href=http://www.twtelecom.net>ISP: Time Warner US</a>
HideSite	*.twtelecom.net
GroupSite	*.u-3mrs.fr		<a href=http://www.u-3mrs.fr>EDU: U. Aix-Marseille FR</a>
HideSite	*.u-3mrs.fr
GroupSite	*.u-bordeaux.fr		<a href=http://www.u-bordeaux.fr>EDU: U. Bordeaux FR</a>
HideSite	*.u-bordeaux.fr
GroupSite	*.ua.es			<a href=http://www.ua.es>EDU: U. Alacant ES</a>
HideSite	*.ua.es
GroupSite	*.ua.pt			<a href=http://www.ua.pt>EDU: U. Aveiro PT</a>
HideSite	*.ua.pt
GroupSite	*.uab.es		<a href=http://www.uab.es>EDU: U. Autonoma Barcelona ES</a>
HideSite	*.uab.es
GroupSite	*.uabc.mx		<a href=http://www.uabc.mx>EDU: U. Aut. Baja California MX</a>
HideSite	*.uabc.mx
GroupSite	*.uach.mx		<a href=http://www.uach.mx>EDU: U. A. Chihuahua MX</a>
HideSite	*.uach.mx
GroupSite	*.uaem.mx		<a href=http://www.uaem.mx>EDU: U. A. Estado de Morelos MX</a>
HideSite	*.uaem.mx
GroupSite	*.uah.es		<a href=http://www.uah.es>EDU: U. Alcala ES</a>
HideSite	*.uah.es
GroupSite	*.ual.es		<a href=http://www.ual.es>EDU: U. Almeria ES</a>
HideSite	*.ual.es
GroupSite	*.uam.es		<a href=http://www.uam.es>EDU: U. Autonoma Madrid ES</a>
HideSite	*.uam.es
GroupSite	*.uam.mx		<a href=http://www.uam.mx>EDU: U.Auton. Metropolitana MX</a>
HideSite	*.uam.mx
GroupSite	*.uap.edu.pe		<a href=http://www.uap.edu.pe>EDU: U. Alas Peruanas PE</a>
HideSite	*.uap.edu.pe
GroupSite	*.ub.es			<a href=http://www.ub.es>EDU: U. Barcelona ES</a>
HideSite	*.ub.es
GroupSite	*.uba.ar		<a href=http://www.uba.ar>EDU: U. Buenos Aires AR</a>
HideSite	*.uba.ar
GroupSite	*.ubiobio.cl		<a href=http://www.ubiobio.cl>EDU: U. Bio-Bio CL</a>
HideSite	*.ubiobio.cl
GroupSite	*.ubu.es		<a href=http://www.ubu.es>EDU: U. Burgos ES</a>
HideSite	*.ubu.es
GroupSite	*.uca.es		<a href=http://www.uca.es>EDU: U. Cadiz ES</a>
HideSite	*.uca.es
GroupSite	*.ucd.ie		<a href=http://www.ucd.ie>EDU: U. College Dublin IE</a>
HideSite	*.ucd.ie
GroupSite	*.uchile.cl		<a href=http://www.uchile.cl>EDU: U. Chile CL</a>
HideSite	*.uchile.cl
GroupSite	*.ucla.edu		<a href=http://www.ucla.edu>EDU: U. Ucla US</a>
HideSite	*.ucla.edu
GroupSite	*.uclm.es		<a href=http://www.uclm.es>EDU: U. Castilla La-Mancha ES</a>
HideSite	*.uclm.es
GroupSite	*.ucm.es		<a href=http://www.ucm.es>EDU: U. Complutense Madrid ES</a>
HideSite	*.ucm.es
GroupSite	*.ucr.ac.cr		<a href=http://www.ucr.ac.cr>EDU: U. Costa Rica CR</a>
HideSite	*.ucr.ac.cr
GroupSite	*.uc3m.es		<a href=http://www.uc3m.es>EDU: U. Carlos III Madrid ES</a>
HideSite	*.uc3m.es
GroupSite	*.uco.es		<a href=http://www.uco.es>EDU: U. Cordoba ES</a>
HideSite	*.uco.es
GroupSite	*.uct.ac.za		<a href=http://www.uct.ac.za>EDU: U. Cape Town ZA</a>
HideSite	*.uct.ac.za
GroupSite	*.ucv.cl		<a href=http://www.ucv.cl>EDU: Pont. U. Cat. Valparaiso CL</a>
HideSite	*.ucv.cl
GroupSite	*.udallas.edu		<a href=http://www.udallas.edu>EDU: U. Dallas US</a>
HideSite	*.udallas.edu
GroupSite	*.udc.es		<a href=http://www.udc.es>EDU: U. Coru&ntilde;a ES</a>
HideSite	*.udc.es
GroupSite	*.udec.cl		<a href=http://www.udec.cl>EDU: U.Concepcion CL</a>
HideSite	*.udec.cl
GroupSite	*.udg.es		<a href=http://www.udg.es>EDU: U. Girona ES</a>
HideSite	*.udg.es
GroupSite	*.udg.mx		<a href=http://www.udg.mx>EDU: U. Guadalajara MX</a>
HideSite	*.udg.mx
GroupSite	*.udl.es		<a href=http://www.udl.es>EDU: U. Lleida ES</a>
HideSite	*.udl.es
GroupSite	*.udsm.ac.tz		<a href=http://www.udsm.ac.tz>EDU: U. Dar Es Salaam TZ</a>
HideSite	*.udsm.ac.tz
GroupSite	*.ufl.edu		<a href=http://www.ufl.edu>EDU: U. Florida US</a>
HideSite	*.ufl.edu
GroupSite	*.ufsc.br		<a href=http://www.ufsc.br>EDU: U.F. Santa Catarina BR</a>
HideSite	*.ufsc.br
GroupSite	*.ugr.es		<a href=http://www.ugr.es>EDU: U. Granada ES</a>
HideSite	*.ugr.es
GroupSite	*.uib.es		<a href=http://www.uib.es>EDU: U. Illes Balears ES</a>
HideSite	*.uib.es
GroupSite	*.uibk.ac.at		<a href=http://www.uibk.ac.at>EDU: Innsbruck U. AT</a>
HideSite	*.uibk.ac.at
GroupSite	*.uiuc.edu		<a href=http://www.uiuc.edu>EDU: U. Illinois US</a>
HideSite	*.uiuc.edu
GroupSite	*.ujaen.es		<a href=http://www.ujaen.es>EDU: U. de Jaen ES</a>
HideSite	*.ujaen.es
GroupSite	*.uji.es		<a href=http://www.uji.es>EDU: U. Jaume I ES</a>
HideSite	*.uji.es
GroupSite	*.ukma.kiev.ua		<a href=http://www.ukma.kiev.ua>EDU: U. Kyiv-Mohyla UA</a>
HideSite	*.ukma.kiev.ua
GroupSite	*.ula.ve		<a href=http://www.ula.ve>EDU: U. Los Andes Merida VE</a>
HideSite	*.ula.ve
GroupSite	*.ulb.ac.be		<a href=http://www.ulb.ac.be>EDU: U. L. Bruxelles BE</a>
HideSite	*.ulb.ac.be
GroupSite	*.um.es			<a href=http://www.um.es>EDU: U. Murcia ES</a>
HideSite	*.um.es
GroupSite	*.uma.es		<a href=http://www.uma.es>EDU: U. Malaga ES</a>
HideSite	*.uma.es
GroupSite	*.umanizales.edu.co	<a href=http://www.umanizales.edu.co>EDU: U. Manizales CO</a>
HideSite	*.umanizales.edu.co
GroupSite	*.umbc.edu		<a href=http://www.umbc.edu>EDU: Honors U. in Maryland US</a>
HideSite	*.umbc.edu
GroupSite	*.uml.edu		<a href=http://www.uml.edu>EDU: U. Massach. Lowell US</a>
HideSite	*.uml.edu
GroupSite	*.umontreal.ca		<a href=http://www.umontreal.ca>EDU: U. Montreal CA</a>
HideSite	*.umontreal.ca
GroupSite	*.umsl.edu		<a href=http://www.umsl.edu>EDU: U. Missouri St.L. US</a>
HideSite	*.umsl.edu
GroupSite	*.umu.se		<a href=http://www.umu.se>EDU: Umea U. SE</a>
HideSite	*.umu.se
GroupSite	*.unab.cl		<a href=http://www.unab.cl>EDU: U. N. Andres Bello CL</a>
HideSite	*.unab.cl
GroupSite	*.unal.edu.co		<a href=http://www.unal.edu.co>EDU: U .N. Colombia CO</a>
HideSite	*.unal.edu.co
GroupSite	*.unalmed.edu.co	<a href=http://www.unalmed.edu.co>EDU: U .N. Colombia CO</a>
HideSite	*.unalmed.edu.co
GroupSite	*.unam.mx		<a href=http://www.unam.mx>EDU: U. Nacional Mexico MX</a>
HideSite	*.unam.mx
GroupSite	*.unap.cl		<a href=http://www.unap.cl>EDU: U. Arturo Prat CL</a>
HideSite	*.unap.cl
GroupSite	*.unav.es		<a href=http://www.unav.es>EDU: U. Navarra-Pamplona ES</a>
HideSite	*.unav.es
GroupSite	*.unavarra.es		<a href=http://www.unavarra.es>EDU: U. Publi. Navarra ES</a>
HideSite	*.unavarra.es
GroupSite	*.uncu.edu.ar		<a href=http://www.uncu.edu.ar>EDU: U. Cuyo-Mendoza AR</a>
HideSite	*.uncu.edu.ar
GroupSite	*.und.ac.za		<a href=http://www.ukzn.ac.za>EDU: U. KwaZulu-Natal ZA</a>
HideSite	*.und.ac.za
GroupSite	*.uned.es		<a href=http://www.uned.es>EDU: UNED Remote U. ES</a>
HideSite	*.uned.es
GroupSite	*.unellez.edu.ve	<a href=http://www.unellez.edu.ve>EDU: UNELLEZ VE</a>
HideSite	*.unellez.edu.ve
GroupSite	*.unet.edu.ve		<a href=http://www.unet.edu.ve>EDU: U. Nac.E. Tachira VE</a>
HideSite	*.unet.edu.ve
GroupSite	*.unete.com.bo		<a href=http://www.unete.com.bo>ISP: Unete BO</a>
HideSite	*.unete.com.bo
GroupSite	*.ungs.edu.ar		<a href=http://www.ungs.edu.ar>EDU: U.N. G. Sarmiento AR</a>
HideSite	*.ungs.edu.ar
GroupSite	*.uni.edu.pe		<a href=http://www.uni.edu.pe>EDU: U. Nac. Ingenieria PE</a>
HideSite	*.uni.edu.pe
GroupSite	*.uni-bremen.de		<a href=http://www.uni-bremen.de>EDU: U. Bremen DE</a>
HideSite	*.uni-bremen.de
GroupSite	*.uni-freiburg.de	<a href=http://www.uni-freiburg.de>EDU: Freiburg Uni. DE</a>
HideSite	*.uni-freiburg.de
GroupSite	*.uni-kl.de		<a href=http://www.uni-kl.de>EDU: T.U. Klaiserlautern DE</a>
HideSite	*.uni-kl.de
GroupSite	*.uni-leipzig.de	<a href=http://www.uni-leipzig.de>EDU: U. Leipzig DE</a>
HideSite	*.uni-leipzig.de
GroupSite	*.uni-luebeck.de	<a href=http://www.uni-luebeck.de>EDU: U. Lubeck DE</a>
HideSite	*.uni-luebeck.de
GroupSite	*.uni-mainz.de		<a href=http://www.uni-mainz.de>EDU: J.G. U. Mainz DE</a>
HideSite	*.uni-mainz.de
GroupSite	*.uni-muenster.de	<a href=http://www.uni-muenster.de>EDU: U. Munster DE</a>
HideSite	*.uni-muenster.de
GroupSite	*.uni2.es		<a href=http://www.uni2.es>ISP: Uni2 ES</a>
HideSite	*.uni2.es
GroupSite	*.unican.es		<a href=http://www.unican.es>EDU: U. Cantabria ES</a>
HideSite	*.unican.es
GroupSite	*.unicc.org		<a href=http://www.unicc.org>ICC int. computing centre UE</a>
HideSite	*.unicc.org
GroupSite	*.unige.ch		<a href=http://www.unige.ch>EDU: U. Geneve CH</a>
HideSite	*.unige.ch
GroupSite	*.unileon.es		<a href=http://www.unileon.es>EDU: U. Leon ES</a>
HideSite	*.unileon.es
GroupSite	*.unimelb.edu.au	<a href=http://www.unimelb.edu.au>EDU: U.Melbourne AU</a>
HideSite	*.unimelb.edu.au
GroupSite	*.uniminuto.edu		<a href=http://www.uniminuto.edu>EDU: U. Minuto CO</a>
HideSite	*.uniminuto.edu
GroupSite	*.unimoron.edu.ar	<a href=http://www.unimoron.edu.ar>EDU: U. Moron AR</a>
HideSite	*.unimoron.edu.ar
GroupSite	*.uninet-ide.com.mx	<a href=http://www.uninet.edu>EDU: UniNet MX</a>
HideSite	*.uninet-ide.com.mx
GroupSite	*.uninet.net.mx		<a href=http://www.uninet.edu>EDU: UniNet MX</a>
HideSite	*.uninet.net.mx
GroupSite	*.uninorte.edu.co	<a href=http://www.uninorte.edu.co>EDU: U. Norte CO</a>
HideSite	*.uninorte.edu.co
GroupSite	*.unipiloto.edu.co	<a href=http://www.unipiloto.edu.co>EDU: U. Piloto CO</a>
HideSite	*.unipiloto.edu.co
GroupSite	*.unipr.it		<a href=http://www.unipr.it>EDU: U. Estudi Parma IT</a>
HideSite	*.unipr.it
GroupSite	*.unired.net.pe		<a href=http://www.unired.net.pe>ISP: Telefonica PE</a>
HideSite	*.unired.net.pe
GroupSite	*.uniroma1.it		<a href=http://www.uniroma1.it>EDU: La Sapienza IT</a>
HideSite	*.uniroma1.it
GroupSite	*.univalle.edu.co	<a href=http://www.univalle.edu.co>EDU: U. Valle CO</a>
HideSite	*.univalle.edu.co
GroupSite	*.uniovi.es		<a href=http://www.uniovi.es>EDU: U. Oviedo ES</a>
HideSite	*.uniovi.es
GroupSite	*.unisa.edu.au		<a href=http://www.unisa.edu.au>EDU: U. S. Australia AU</a>
HideSite	*.unisa.edu.au
GroupSite	*.unisys.co.jp		<a href=http://www.unisys.co.jp>ISP: Unisys JP</a>
HideSite	*.unisys.co.jp
GroupSite	*.univ-ag.fr		<a href=http://www.univ-ag.fr>EDU: U. Antilles-Guyane FR</a>
HideSite	*.univ-ag.fr
GroupSite	*.univ-lille1.fr	<a href=http://www.univ-lille1.fr>EDU: U. Lille FR</a>
HideSite	*.univ-lille1.fr
GroupSite	*.univ-lr.fr		<a href=http://www.univ-lr.fr>EDU: U. Rochelle FR</a>
HideSite	*.univ-lr.fr
GroupSite	*.univ-pau.fr		<a href=http://www.univ-pau.fr>EDU: U. de Pau FR</a>
HideSite	*.univ-pau.fr
GroupSite	*.univ-poitiers.fr	<a href=http://www.univ-poitiers.fr>EDU: U. Poitiers FR</a>
HideSite	*.univ-poitiers.fr
GroupSite	*.univ-rouen.fr		<a href=http://www.univ-rouen.fr>EDU: U. Rouen FR</a>
HideSite	*.univ-rouen.fr
GroupSite	*.univ-tlse2.fr		<a href=http://www.univ-tlse2.fr>EDU: U. Tolouse Mir. FR</a>
HideSite	*.univ-tlse2.fr
GroupSite	*.univie.ac.at		<a href=http://www.univie.ac.at>EDU: U. Wien AT</a>
HideSite	*.univie.ac.at
GroupSite	*.uniweb.net.co		<a href=http://www.uniweb.net.co>Webmail: Uniweb CO</a>
HideSite	*.uniweb.net.co
GroupSite	*.unizar.es		<a href=http://www.unizar.es>EDU: U. Zaragoza ES</a>
HideSite	*.unizar.es
GroupSite	*.unl.edu.ar		<a href=http://www.unl.edu.ar>EDU: U. Nac. del Litoral AR</a>
HideSite	*.unl.edu.ar
GroupSite	*.unlm.edu.ar		<a href=http://www.unlm.edu.ar>EDU: U. La Matanza AR</a>
HideSite	*.unlm.edu.ar
GroupSite	*.unlp.edu.ar		<a href=http://www.unlp.edu.ar>EDU: U. N. La Plata AR</a>
HideSite	*.unlp.edu.ar
GroupSite	*.unne.edu.ar		<a href=http://www.unne.edu.ar>EDU: U. Nac. Nordeste AR</a>
HideSite	*.unne.edu.ar
GroupSite	*.uns.edu.ar		<a href=http://www.uns.edu.ar>EDU: U. Nac. del Sur AR</a>
HideSite	*.uns.edu.ar
GroupSite	*.untd.com		<a href=http://www.untd.com>ISP: United Online US</a>
HideSite	*.untd.com
GroupSite	*.unr.edu.ar		<a href=http://www.unr.edu.ar>EDU: U. Nac. Rosario AR</a>
HideSite	*.unr.edu.ar
GroupSite	*.unsj-cuim.edu.ar	<a href=http://www.unsj-cuim.edu.ar>EDU: U .N. S. Juan AR</a>
HideSite	*.unsj-cuim.edu.ar
GroupSite	*.upr.clu.edu		<a href=http://www.upr.clu.edu>EDU: U. Puerto Rico PR</a>
HideSite	*.upr.clu.edu
GroupSite	*.utalca.cl		<a href=http://www.utalca.cl>EDU: U. Talca CL</a>
HideSite	*.utalca.cl
GroupSite	*.utem.cl		<a href=http://www.utem.cl>EDU: U. Tec. Metropolitana CL</a>
HideSite	*.utem.cl
GroupSite	*.utl.pt		<a href=http://www.utl.pt>EDU: U. T. Lisboa PT</a>
HideSite	*.utl.pt
GroupSite	*.uoc.es		<a href=http://www.uoc.es>EDU: U. Oberta Catalunya ES</a>
HideSite	*.uoc.es
GroupSite	*.uoregon.edu		<a href=http://www.uoregon.edu>EDU: U. Oregon US</a>
HideSite	*.uoregon.edu
GroupSite	*.up.ac.za		<a href=http://www.up.ac.za>EDU: U. Pretoria ZA</a>
HideSite	*.up.ac.za
GroupSite	*.upc.edu		<a href=http://www.upc.es>EDU: U. Polit. Catalunya ES</a>
HideSite	*.upc.edu
GroupSite	*.upc.es		<a href=http://www.upc.es>EDU: U. Polit. Catalunya ES</a>
HideSite	*.upc.es
GroupSite	*.upco.es		<a href=http://www.upco.es>EDU: U. Pontifica Comillas ES</a>
HideSite	*.upco.es
GroupSite	*.upf.edu		<a href=http://www.upf.es>EDU: U. Pompeu Fabra ES</a>
HideSite	*.upf.edu
GroupSite	*.upf.es		<a href=http://www.upf.es>EDU: U. Pompeu Fabra ES</a>
HideSite	*.upf.es
GroupSite	*.upit.ro		<a href=http://www.uit.ro>EDU: U. Pitesti RO</a>
HideSite	*.upit.ro
GroupSite	*.upm.es		<a href=http://www.upm.es>EDU: U. Polit. Madrid ES</a>
HideSite	*.upm.es
GroupSite	*.upenn.edu		<a href=http://www.upenn.edu>EDU: U. Pennsylvania US</a>
HideSite	*.upenn.edu
GroupSite	*.upmf-grenoble.fr	<a href=http://upmf-grenoble.fr>EDU: U. Pierre Mendes FR</a>
HideSite	*.upmf-grenoble.fr
GroupSite	*.upo.es		<a href=http://www.upo.es>EDU: U. Pablo de Olavide ES</a>
HideSite	*.upo.es
GroupSite	*.upoli.edu.ni		<a href=http://www.upoli.edu.ni>EDU: U. Pol. Nicaragua NI</a>
HideSite	*.upoli.edu.ni
GroupSite	*.ups.com		<a href=http://www.ups.com>UPS Global **</a>
HideSite	*.ups.com
GroupSite	*.upv.es		<a href=http://www.upv.es>EDU: U. Polit. Valencia ES</a>
HideSite	*.upv.es
GroupSite	*.urbanet.ch		<a href=http://www.urbanet.ch>ISP: Urbanet (Cablecom) CH</a>
HideSite	*.urbanet.ch
GroupSite	*.urjc.es		<a href=http://www.urjc.es>EDU: U. Rey Juan Carlos ES</a>
HideSite	*.urjc.es
GroupSite	*.url.edu		<a href=http://www.url.edu>EDU: U. Ramon Llull ES</a>
HideSite	*.url.edu
GroupSite	*.urv.es		<a href=http://www.urv.es>EDU: U. Rovira Virgili ES</a>
HideSite	*.urv.es
GroupSite	*.usal.es		<a href=http://www.usal.es>EDU: U. Salamanca ES</a>
HideSite	*.usal.es
GroupSite	*.uswest.net		<a href=http://www.uswest.net>ISP: Qwest (uswest.net) US</a>
HideSite	*.uswest.net
GroupSite	*.us.es			<a href=http://www.us.es>EDU: U. Sevilla ES</a>
HideSite	*.us.es
GroupSite	*.usc.es		<a href=http://www.usc.es>EDU: U. Santiago de C. ES</a>
HideSite	*.usc.es
GroupSite	*.utas.edu.au		<a href=http://www.utas.edu.au>EDU: U. Tasmania AU</a>
HideSite	*.utas.edu.au
GroupSite	*.utc.sk		<a href=http://www.utc.sk>EDU: Zilinska U. SK</a>
HideSite	*.utc.sk
GroupSite	*.uteq.edu.mx		<a href=http://www.uteq.edu.mx>EDU: U. Tech. Queretaro MX</a>
HideSite	*.uteq.edu.mx
GroupSite	*.utep.edu		<a href=http://www.utep.edu>EDU: U. Texas At El Paso US</a>
HideSite	*.utep.edu
GroupSite	*.utfsm.cl		<a href=http://www.utfsm.cl>EDU: U.T. Federico S.Maria CL</a>
HideSite	*.utfsm.cl
GroupSite	*.utk.edu		<a href=http://www.utk.edu>EDU: U. Tennessee US</a>
HideSite	*.utk.edu
GroupSite	*.utm.mx		<a href=http://www.utm.mx>EDU: U.T. Mixteca MX</a>
HideSite	*.utm.mx
GroupSite	*.utoronto.ca		<a href=http://www.utoronto.ca>EDU: U. Toronto CA</a>
HideSite	*.utoronto.ca
GroupSite	*.utsam.edu.ec		<a href=http://www.utsam.edu.ec>EDU: U. Tec. A. M. EC</a>
HideSite	*.utsam.edu.ec
GroupSite	*.uunet.be		<a href=http://www.uunet.be>ISP: Worlcom BE</a>
HideSite	*.uunet.be
GroupSite	*.uunet.co.za		<a href=http://www.uunet.co.za>ISP: Uunet ZA</a>
HideSite	*.uunet.co.za
GroupSite	*.uunet.fr		<a href=http://www.uunet.fr>ISP: MCI FR</a>
HideSite	*.uunet.fr
GroupSite	*.uu.net		<a href=http://www.uu.net>ISP: UU-Net US</a>
HideSite	*.uu.net
GroupSite	*.uv.es			<a href=http://www.uv.es>EDU: U. Valencia ES</a>
HideSite	*.uv.es
GroupSite	*.uva.es		<a href=http://www.uva.es>EDU: U. Valladolid ES</a>
HideSite	*.uva.es
GroupSite	*.uva.nl		<a href=http://www.uva.nl>EDU: U. Amsterdam NL</a>
HideSite	*.uva.nl
GroupSite	*.uva.ve		<a href=http://www.uva.ve>EDU: U. Andes VE</a>
HideSite	*.uva.ve
GroupSite	*.uvic.es		<a href=http://www.uvic.es>EDU: U. Vic ES</a>
HideSite	*.uvic.es
GroupSite	*.uvigo.es		<a href=http://www.uvigo.es>EDU: U. of Vigo</a>
HideSite	*.uvigo.es
GroupSite	*.uvt.nl		<a href=http://www.uvt.nl>EDU: U. Tilburg NL</a>
HideSite	*.uvt.nl
GroupSite	*.uwf.edu		<a href=http://www.uwf.edu>EDU: U. West Florida US</a>
HideSite	*.uwf.edu
GroupSite	*.uwgb.edu		<a href=http://www.uwgb.edu>EDU: U. Winsconsin G.B. US</a>
HideSite	*.uwgb.edu
GroupSite	*.velocom.com.ar	<a href=http://www.velocom.com.ar>ISP: VeloCom AR</a>
HideSite	*.velocom.com.ar
GroupSite	*.veloxzone.com.br	<a href=http://www.veloxzone.com.br>ISP: Velox BR</a>
HideSite	*.veloxzone.com.br
GroupSite	*.dsl-verizon.net	<a href=http://www.verizon.net>ISP: Verizon</a>
HideSite	*.dsl-verizon.net
GroupSite	*.verizon.net		<a href=http://www.verizon.net>ISP: Verizon</a>
HideSite	*.verizon.net
GroupSite	*.versanet.de		<a href=http://www.versatel.net>ISP: Versatel DE</a>
HideSite	*.versanet.de
GroupSite	*.versatel.net		<a href=http://www.versatel.net>ISP: Versatel NL</a>
HideSite	*.versatel.net
GroupSite	*.viaginterkom.de	<a href=http://www.viaginterkom.de>ISP: O2 DE</a>
HideSite	*.viaginterkom.de
GroupSite	*.vianetworks.nl	<a href=http://www.vianetworks.nl>ISP: Via Net. NL</a>
HideSite	*.vianetworks.nl
GroupSite	*.videotron.ca		<a href=http://www.videotron.ca>ISP: Videotron CA</a>
HideSite	*.videotron.ca
GroupSite	*.virgin.net		<a href=http://www.virgin.net>ISP: Virgin UK/IE</a>
HideSite	*.virgin.net
GroupSite	*.virginia.edu		<a href=http://www.virginia.edu>EDU: U. Virginia US</a>
HideSite	*.virginia.edu
GroupSite	*.vol.cz		<a href=http://www.vol.cz>ISP Czech On Line CZ</a>
HideSite	*.vol.cz
GroupSite	*.vsi.ru		<a href=http://www.vsi.ru>ISP: vsi? RU</a>
HideSite	*.vsi.ru
GroupSite	*.vu.edu.au		<a href=http://www.vu.edu.au>EDU: Victoria U. AU</a>
HideSite	*.vu.edu.au
GroupSite	*.vu.nl			<a href=http://www.vu.nul>EDU: Vrije U. NL</a>
HideSite	*.vu.nl
GroupSite	*.vuosas.cz		<a href=http://www.vuosas.cz>EDU Vousas Research CZ</a>
HideSite	*.vuosas.cz
GroupSite	*.vutbr.cz		<a href=http://www.vutbr.cz>EDU: Brno U. Techn. CZ</a>
HideSite	*.vutbr.cz
GroupSite	*.w3.org		<a href=http://www.w3.org>W3C US</a>
HideSite	*.w3.org
GroupSite	*.wanadoo.be		<a href=http://www.wanadoo.be>ISP: Wanadoo BE</a>
HideSite	*.wanadoo.be
GroupSite	*.wanadoo.com		<a href=http://www.wanadoo.fr>ISP: Wanadoo **</a>
HideSite	*.wanadoo.com
GroupSite	*.wanadoo.fr		<a href=http://www.wanadoo.fr>ISP: Wanadoo FR</a>
HideSite	*.wanadoo.fr
GroupSite	*.wanadoo.nl		<a href=http://www.wanadoo.nl>ISP: Wanadoo NL</a>
HideSite	*.wanadoo.nl
#GroupSite	*.wanadoo		<a href=http://www.wanadoo.fr>ISP: Wanadoo **</a>
#HideSite	*.wanadoo
GroupSite	*.warpdriveonline.com	<a href=http://www.warpdriveonline.com>ISP: warpdrive</a>
HideSite	*.warpdriveonline.com
GroupSite	*.washington.edu	<a href=http://www.washington.edu>EDU: U. Washington US</a>
HideSite	*.washington.edu
GroupSite	*.bobby.watchfire.com	<a href=http://bobby.watchfire.com>Accessibility checker</a> 
HideSite	*.bobby.watchfire.com
GroupSite	*.watchfire.com		<a href=http://www.watchfire.com>Search: Watchfire *</a>
HideSite	*.watchfire.com
GroupSite	*.wasp.net.au		<a href=http://www.wasp.net.au>ISP: WASP AU</a>
HideSite	*.wasp.net.au
GroupSite	*.way.com.ar		<a href=http://www.way.com.ar>ISP: Internet Way AR</a>
HideSite	*.way.com.ar
GroupSite	*.wcom.com		<a href=http://www.wcom.net>ISP: MCI **</a>
HideSite	*.wcom.com
GroupSite	*.wcom.net		<a href=http://www.wcom.net>ISP: MCI **</a>
HideSite	*.wcom.net
GroupSite	*.webtv.net		<a href=http://www.webtv.net>ISP: WebTV</a>
HideSite	*.webtv.net
GroupSite	*.whois.sc		<a href=http://www.whois.sc>Spider: Whois.sc</a>
HideSite	*.whois.sc
GroupSite	*.wien.at		<a href=http://www.wien.at>Portal: wien.at AT</a>
HideSite	*.wien.at
GroupSite	*.wireless.com.py	<a href=http://www.wireless.com.py>ISP: Telesurf PY</a>
HideSite	*.wireless.com.py
GroupSite	*.wits.ac.za		<a href=http://www.wits.ac.za>EDU: U. Witwatersrand ZA</a>
HideSite	*.wits.ac.za
GroupSite	*.wlcom.com.mx		<a href=http://www.wlcom.com.mx>ISP: WL COM MX</a>
HideSite	*.wlcom.com.mx
GroupSite	*.worldonline.dk	<a href=http://worldonline.dk>ISP: Worldonline Tiscali DK</a>
HideSite	*.worldonline.dk
GroupSite	*.wright.edu		<a href=http://wright.edu>EDU: Wrigt State U. Dayton, Ohio US</a>
HideSite	*.wright.edu
GroupSite	*.wtnet.de		<a href=http://www.wtnet.de>ISP: wilhem.tel (wtnet.de) DE</a>
HideSite	*.wtnet.de
GroupSite	*.xo.net		<a href=http://www.xo.net>ISP: XO US</a>
HideSite	*.xo.net
GroupSite	*.xs4all.nl		<a href=http://www.xs4all.nl>ISP: xs4all NL</a>
HideSite	*.xs4all.nl
GroupSite	*.xtec.es		<a href=http://www.xtec.es>EDU: XTEC Catalonia Edu. Network ES</a>
HideSite	*.xtec.es
GroupSite	*.xtec.net		<a href=http://www.xtec.es>EDU: XTEC Catalonia Edu. Network ES</a>
HideSite	*.xtec.net
GroupSite	*.xunta.es		<a href=http://www.xunta.es>GOV: Xunta de Galicia ES</a>
HideSite	*.xunta.es
GroupSite	*.x-echo.com		<a href=http://www.x-echo.com>ISP: Echo - Wanadoo FR</a>
HideSite	*.x-echo.com
GroupSite	*.jazzfree.ya.com	<a href=http://www.jazzfree.ya.com>ISP: Jazzfree ES</a>
HideSite	*.jazzfree.ya.com
GroupSite	*.ya.com		<a href=http://www.ya.com>ISP: Ya.com ES</a>
HideSite	*.ya.com
GroupSite	mmcrc.search.scd.yahoo.com	Spider: Yahoo! **</a>
HideSite	mmcrc.search.scd.yahoo.com
GroupSite	*.yorku.ca		<a href=http://www.yorku.ca>EDU: York U. CA</a>
HideSite	*.yorku.ca
GroupSite	*.yournet.ne.jp		<a href=http://www.yournet.ne.jp>ISP: yournet JP</a>
HideSite	*.yournet.ne.jp
GroupSite	*.zonepro-network.com	<a href=http://zonepro.com>ISP: Zonepro FR</a>
HideSite	*.zonepro-network.com
GroupSite	*.zonnet.nl		<a href=http://www.zonnet.nl>ISP: ZON NL</a>
HideSite	*.zonnet.nl
GroupSite	*.zoominternet.net	<a href=http://www.zoominternet.net>ISP: Armstrong US</a>
HideSite	*.zoominternet.net
GroupSite	*.fav.zcu.cz		<a href=http://www.fav.zcu.cz>EDU: Fakulta Aplikovanych CZ</a>
HideSite	*.fav.zcu.cz
GroupSite	*.zcu.cz		<a href=http://www.zcu.cz>EDU: Zapadoceska U. Vplzni CZ</a>
HideSite	*.zcu.cz
GroupSite	*.zonaturbo.com		<a href=http://www.zonaturbo.com>ISP: Zonaturbo ??</a>
HideSite	*.zonaturbo.com
GroupSite	*.zps.cz		<a href=http://www.zps.cz>ISP: Tajmac-zps CZ</a>
HideSite	*.zps.cz

EOF

echo "GroupReferrer   yahoo.com/      Yahoo!"
echo "GroupReferrer   excite.com/     Excite"
echo "GroupReferrer   infoseek.com/   InfoSeek"
echo "GroupReferrer   webcrawler.com/ WebCrawler"

#GroupUser      root            Admin users
#GroupUser      admin           Admin users
#GroupUser      wheel           Admin users

# The following is a great way to get an overall total
# for browsers, and not display all the detail records.
# (You should use MangleAgent to refine further...)

# Browser:
echo "GroupAgent      Opera/      Opera"
echo "HideAgent       Opera/"
echo "GroupAgent      \"MSIE 7\"    Microsoft Internet Explorer 7"
echo "HideAgent       MSIE 7"
echo "GroupAgent      \"MSIE 6\"    Microsoft Internet Explorer 6"
echo "HideAgent       MSIE 6"
echo "GroupAgent      \"MSIE \"     Older Microsoft Explorers"
echo "HideAgent       MSIE"
echo "GroupAgent      Firefox/3.  Firefox 3"
echo "HideAgent       Firefox/3."
echo "GroupAgent      Firefox/2.  Firefox 2"
echo "HideAgent       Firefox/2."
echo "GroupAgent      Firefox/1.  Firefox 1.x"
echo "HideAgent       Firefox/1."
echo "GroupAgent      Konqueror   Konqueror"
echo "HideAgent       Konqueror"
echo "GroupAgent      Safari      Safari"
echo "HideAgent       Safari"
echo "GroupAgent      Lynx*       Lynx"
echo "HideAgent       Lynx*"
echo "GroupAgent      Wget/       WGet"
echo "HideAgent       Wget/"
echo "GroupAgent      (compatible;    Other Mozilla Compatibles"
echo "HideAgent       (compatible;"
echo "GroupAgent      Mozilla*    Mozilla/Netscape"
echo "HideAgent       Mozilla*"
echo "GroupAgent      Avant*            Avant Browser"
echo "HideAgent       Avant*"
echo "GroupAgent      Lotus-Notes/6.0   Lotus-Notes 6.0"
echo "HideAgent       Lotus-Notes/6.0"
echo "GroupAgent      Konqueror*        Konquerer"
echo "HideAgent       Konqueror*"
echo "GroupAgent      SlimBrowser       SlimBrowser"
echo "HideAgent       SlimBrowser"
echo "GroupAgent      NCBrowser         NCBrowser"
echo "HideAgent       NCBrowser"

# Player:
echo "GroupAgent      NSPlayer/*        NSPlayer"
echo "HideAgent       NSPlayer/*"
echo "GroupAgent      Windows-Media-Player*  Windows Media Player"
echo "HideAgent       Windows-Media-Player*"
echo "GroupAgent      VLC               VLC Player"
echo "HideAgent       VLC"

# Downloadmanager:
echo "GroupAgent      GetRight/*        Downloadmanager GetRight"
echo "HideAgent       GetRight/*"
echo "GroupAgent      LeechGet*         Downloadmanager LeechGet"
echo "HideAgent       LeechGet*"
echo "GroupAgent      Totalcmd          Total Commander"
echo "HideAgent       Totalcmd"

#Robots und Spider:
echo "GroupAgent      msnbot            Suchmaschine: MSN"
echo "HideAgent       msnbot"
echo "GroupAgent      Googlebot/        Suchmaschine: Google"
echo "HideAgent       Googlebot/"
echo "GroupAgent      Mediapartners-Google Suchmaschine: Google"
echo "HideAgent       Mediapartners-Google"
echo "GroupAgent      Yahoo*            Suchmaschine: Yahoo!"
echo "HideAgent       Yahoo*"
echo "GroupAgent      cooter            Suchmaschine: Altavista"
echo "HideAgent       cooter"
echo "GroupAgent      Ask               Spider: Teoma"
echo "HideAgent       Ask"
echo "GroupAgent      ia_archiver       Spider: Archive.org"
echo "HideAgent       ia_archiver"

echo "GroupAgent      PEAR              Programmierung: PEAR Library (PHP)"
echo "HideAgent       PEAR"
echo "GroupAgent      PHP               Programmierung: PHP-based client"
echo "HideAgent       PHP"
echo "GroupAgent      libwww-perl       Programmierung: LIB-WWW (Perl library)"
echo "HideAgent       libwww-perl"
echo "GroupAgent      RPT               Programmierung: RPT-HTTP (Java)"
echo "HideAgent       RPT"
echo "GroupAgent      ColdFusion        Programmierung: ColdFusion"
echo "HideAgent       ColdFusion"
echo "GroupAgent      Python            Programmierung: Python-urllib"
echo "HideAgent       Python"
echo "GroupAgent      Indy              Programmierung: Indy (Delphi-based client)"
echo "HideAgent       Indy"

#echo "GroupAgent
#echo "HideAgent

echo "GroupAgent      W3C               W3C Validator"
echo "HideAgent       W3C"
echo "GroupAgent      DA                Proxy: DA"
echo "HideAgent       DA"
echo "GroupAgent      1&1\xa0Mailmanager 1&1 Mailmanager"
echo "HideAgent       1&1\xa0Mailmanager"

#Alle Suchmaschinen ausblenden:
echo "GroupAgent      Bot                diverse Suchroboter, Spider, Crawler etc."
echo "GroupAgent      bot                diverse Suchroboter, Spider, Crawler etc."
echo "GroupAgent      Crawler            diverse Suchroboter, Spider, Crawler etc."
echo "GroupAgent      crawler            diverse Suchroboter, Spider, Crawler etc."
echo "GroupAgent      meta               diverse Suchroboter, Spider, Crawler etc."
echo "GroupAgent      Meta               diverse Suchroboter, Spider, Crawler etc."
echo "GroupAgent      Spider             diverse Suchroboter, Spider, Crawler etc."
echo "GroupAgent      spider             diverse Suchroboter, Spider, Crawler etc."
echo "HideAgent       Bot"
echo "HideAgent       bot"
echo "HideAgent       Crawler"
echo "HideAgent       crawler"
echo "HideAgent       meta"
echo "HideAgent       Meta"
echo "HideAgent       Spider"
echo "HideAgent       spider"

# HideAllSites allows forcing individual sites to be hidden in the
# report.  This is particularly useful when used in conjunction
# with the "GroupDomain" feature, but could be useful in other
# situations as well, such as when you only want to display grouped
# sites (with the GroupSite keywords...).  The value for this
# keyword can be either 'yes' or 'no', with 'no' the default,
# allowing individual sites to be displayed.
#HideAllSites   no

# The GroupDomains keyword allows you to group individual hostnames
# into their respective domains.  The value specifies the level of
# grouping to perform, and can be thought of as 'the number of dots'
# that will be displayed.  For example, if a visiting host is named
# cust1.tnt.mia.uu.net, a domain grouping of 1 will result in just
# "uu.net" being displayed, while a 2 will result in "mia.uu.net".
# The default value of zero disable this feature.  Domains will only
# be grouped if they do not match any existing "GroupSite" records,
# which allows overriding this feature with your own if desired.
#GroupDomains   0

# The GroupShading allows grouped rows to be shaded in the report.
# Useful if you have lots of groups and individual records that
# intermingle in the report, and you want to diferentiate the group
# records a little more.  Value can be 'yes' or 'no', with 'yes'
# being the default.
#GroupShading   yes

# GroupHighlight allows the group record to be displayed in BOLD.
# Can be either 'yes' or 'no' with the default 'yes'.
#GroupHighlight yes

# The Ignore* keywords allow you to completely ignore log records based
# on hostname, URL, user agent, referrer or username.  I hesitated in
# adding these, since the Webalizer was designed to generate _accurate_
# statistics about a web servers performance.  By choosing to ignore
# records, the accuracy of reports become skewed, negating why I wrote
# this program in the first place.  However, due to popular demand, here
# they are.  Use the same as the Hide* keywords, where the value can have
# a leading or trailing wildcard '*'.  Use at your own risk ;)  Please
# remember, the use of these will MAKE YOUR STATS INACCURATE and you
# should consider using an equivalent 'Hide*' keyword instead.
#IgnoreSite bad.site.net
#IgnoreURL  /test*
#IgnoreReferrer file:/*
#IgnoreAgent    RealPlayer
#IgnoreUser     root

# The Include* keywords allow you to force the inclusion of log records
# based on hostname, URL, user agent, referrer or username.  They take
# precidence over the Ignore* keywords.  Note: Using Ignore/Include
# combinations to selectivly process parts of a web site is _extremely
# inefficent_!!! Avoid doing so if possible (ie: grep the records to a
# seperate file if you really want that kind of report).
# Example: Only show stats on Joe User's pages...
#IgnoreURL  *
#IncludeURL ~joeuser*
# Or based on an authenticated username
#IgnoreUser     *
#IncludeUser    someuser

# The MangleAgents allows you to specify how much, if any, The Webalizer
# should mangle user agent names.  This allows several levels of detail
# to be produced when reporting user agent statistics.  There are six
# levels that can be specified, which define different levels of detail
# supression.  Level 5 shows only the browser name (MSIE or Mozilla)
# and the major version number.  Level 4 adds the minor version number
# (single decimal place).  Level 3 displays the minor version to two
# decimal places.  Level 2 will add any sub-level designation (such
# as Mozilla/3.01Gold or MSIE 3.0b).  Level 1 will attempt to also add
# the system type if it is specified.  The default Level 0 displays the
# full user agent field without modification and produces the greatest
# amount of detail.  User agent names that can't be mangled will be
# left unmodified.
#MangleAgents    0
echo "MangleAgents    1"

# The SearchEngine keywords allow specification of search engines and
# their query strings on the URL.  These are used to locate and report
# what search strings are used to find your site.  The first word is
# a substring to match in the referrer field that identifies the search
# engine, and the second is the URL variable used by that search engine
# to define it's search terms.
#SearchEngine   .google.    q=
#SearchEngine   yahoo.com   p=
#SearchEngine   altavista.com   q=
#SearchEngine   aolsearch.      query=
#SearchEngine   ask.co          q=
#SearchEngine   eureka.com  q=
#SearchEngine   lycos.com   query=
#SearchEngine   hotbot.com  MT=
#SearchEngine   msn.com     q=
#SearchEngine   infoseek.com    qt=
#SearchEngine   excite      search=
#SearchEngine   netscape.com    query=
#SearchEngine   mamma.com   query=
#SearchEngine   alltheweb.com   q=
#SearchEngine   northernlight.com  qr=

#  IPs from google  (this is really going to hurt performance)
cat << EOF
# Google 64.233.160.0 - 64.233.191.255
SearchEngine	64.233.160.		q=
SearchEngine	64.233.161.		q=
SearchEngine	64.233.162.		q=
SearchEngine	64.233.163.		q=
SearchEngine	64.233.164.		q=
SearchEngine	64.233.165.		q=
SearchEngine	64.233.166.		q=
SearchEngine	64.233.167.		q=
SearchEngine	64.233.168.		q=
SearchEngine	64.233.169.		q=
SearchEngine	64.233.170.		q=
SearchEngine	64.233.171.		q=
SearchEngine	64.233.172.		q=
SearchEngine	64.233.173.		q=
SearchEngine	64.233.174.		q=
SearchEngine	64.233.175.		q=
SearchEngine	64.233.176.		q=
SearchEngine	64.233.177.		q=
SearchEngine	64.233.178.		q=
SearchEngine	64.233.179.		q=
SearchEngine	64.233.180.		q=
SearchEngine	64.233.181.		q=
SearchEngine	64.233.182.		q=
SearchEngine	64.233.183.		q=
SearchEngine	64.233.184.		q=
SearchEngine	64.233.185.		q=
SearchEngine	64.233.186.		q=
SearchEngine	64.233.187.		q=
SearchEngine	64.233.188.		q=
SearchEngine	64.233.189.		q=
SearchEngine	64.233.190.		q=
SearchEngine	64.233.191.		q=
#  Google IPs 66.102.0.0 - 66.102.15.255
SearchEngine	66.102.0.		q=
SearchEngine	66.102.1.		q=
SearchEngine	66.102.2.		q=
SearchEngine	66.102.3.		q=
SearchEngine	66.102.4.		q=
SearchEngine	66.102.5.		q=
SearchEngine	66.102.6.		q=
SearchEngine	66.102.7.		q=
SearchEngine	66.102.8.		q=
SearchEngine	66.102.9.		q=
SearchEngine	66.102.10.		q=
SearchEngine	66.102.11.		q=
SearchEngine	66.102.12.		q=
SearchEngine	66.102.13.		q=
SearchEngine	66.102.14.		q=
SearchEngine	66.102.15.		q=
# IPs from lop.com
SearchEngine	66.220.17.15*		s=
# one IP from fr.abacho.com
SearchEngine	194.231.30.		q=
# more google IPs  66.249.64.0 - 66.249.95.255
SearchEngine	66.249.64.	q=
SearchEngine	66.249.65.	q=
SearchEngine	66.249.66.	q=
SearchEngine	66.249.67.	q=
SearchEngine	66.249.68.	q=
SearchEngine	66.249.69.	q=
SearchEngine	66.249.70.	q=
SearchEngine	66.249.71.	q=
SearchEngine	66.249.72.	q=
SearchEngine	66.249.73.	q=
SearchEngine	66.249.74.	q=
SearchEngine	66.249.75.	q=
SearchEngine	66.249.76.	q=
SearchEngine	66.249.77.	q=
SearchEngine	66.249.78.	q=
SearchEngine	66.249.79.	q=
SearchEngine	66.249.80.	q=
SearchEngine	66.249.81.	q=
SearchEngine	66.249.82.	q=
SearchEngine	66.249.83.	q=
SearchEngine	66.249.84.	q=
SearchEngine	66.249.85.	q=
SearchEngine	66.249.86.	q=
SearchEngine	66.249.87.	q=
SearchEngine	66.249.88.	q=
SearchEngine	66.249.89.	q=
SearchEngine	66.249.90.	q=
SearchEngine	66.249.91.	q=
SearchEngine	66.249.92.	q=
SearchEngine	66.249.93.	q=
SearchEngine	66.249.94.	q=
SearchEngine	66.249.95.	q=
# this is the Google Web Accelerator, 72.14.192.0 - 72.14.239.255
# seems that they also use that IP block for google searches
SearchEngine	72.14.207.104	q=
# one IP from www.metaspinner-media.de
SearchEngine	212.227.33.241		qry=
# more google IPs  216.239.32.0 - 216.239.63.255 
SearchEngine	216.239.32.		q=
SearchEngine	216.239.33.		q=
SearchEngine	216.239.34.		q=
SearchEngine	216.239.35.		q=
SearchEngine	216.239.36.		q=
SearchEngine	216.239.37.		q=
SearchEngine	216.239.38.		q=
SearchEngine	216.239.39.		q=
SearchEngine	216.239.40.		q=
SearchEngine	216.239.41.		q=
SearchEngine	216.239.42.		q=
SearchEngine	216.239.43.		q=
SearchEngine	216.239.44.		q=
SearchEngine	216.239.45.		q=
SearchEngine	216.239.46.		q=
SearchEngine	216.239.47.		q=
SearchEngine	216.239.48.		q=
SearchEngine	216.239.49.		q=
SearchEngine	216.239.50.		q=
SearchEngine	216.239.51.		q=
SearchEngine	216.239.52.		q=
SearchEngine	216.239.53.		q=
SearchEngine	216.239.54.		q=
SearchEngine	216.239.55.		q=
SearchEngine	216.239.56.		q=
SearchEngine	216.239.57.		q=
SearchEngine	216.239.58.		q=
SearchEngine	216.239.59.		q=
SearchEngine	216.239.60.		q=
SearchEngine	216.239.61.		q=
SearchEngine	216.239.62.		q=
SearchEngine	216.239.63.		q=
# search.100.com also uses tsterm=
SearchEngine	search.100.com		sterm=
SearchEngine	news.100.com		tsterm=
SearchEngine	163.com			q=
SearchEngine	1blink.com		q=
SearchEngine	1klik.dk		query=
SearchEngine	2020search.com		Keywords=
SearchEngine    348north.com    	search=
SearchEngine	.3721.com		name=
SearchEngine	.4ei.com		keywords=
SearchEngine	7search.com		qu=
SearchEngine	7metasearch.com		keyword=
SearchEngine	9online.fr		query=
SearchEngine	a9.com			q=
SearchEngine	a-site.at		query=
#SearchEngine	abacho.at		q=
#SearchEngine	abacho.ch		q=
#SearchEngine	abacho.de		q=
SearchEngine	abacho.			q=
SearchEngine    abcsearch.com   	terms=
#SearchEngine	aberja.at		txt=
#SearchEngine	aberja.ch		txt=
#SearchEngine	aberja.de		txt=
SearchEngine	aberja.			txt=
# about uses zu=?
SearchEngine	about.com		terms=
SearchEngine	accoona.com		qt=
SearchEngine	busca.achei.com		query=
#achei.com.br hides its searches
##SearchEngine	busca.achei.com.br	query=
SearchEngine	acoon.de		begriff=
# aeiwi hides searches
SearchEngine	aeiwi.com		ss=
# aesop is dead
SearchEngine	aesop.com		q=
# ah-ha is a paid listing engine
SearchEngine	ah-ha.com		Q=
SearchEngine	ahoj.pl			q=
SearchEngine	aj.com			ask=
SearchEngine	antisearch.net		KEYWORDS=
SearchEngine	alexa.com		q=
SearchEngine	allesklar.at		words=
SearchEngine	allesklar.de		words=
# allisonline won't usually reveal the search string. Uses also mots=, Marque=
#SearchEngine	allisonline.com		searchword=
SearchEngine	allsearchengines.co.uk	query=
SearchEngine	alltheinternet.com	q=
SearchEngine	allthesites.com		query=
# alltheweb used query= before
SearchEngine    alltheweb.com   	q=
SearchEngine	alltheweb.		query=
# altavista used query= before
#SearchEngine	altavista.at		q=
#SearchEngine	altavista.ch		q=
#SearchEngine	altavista.de		q=
#SearchEngine	altavista.com   	q=
#SearchEngine	altavista.digital.com	q=
SearchEngine	altavista.		q=
# buscador.americaonline.com.ar uses also q=
SearchEngine	buscador.americaonline.com.ar	query=
SearchEngine	ameego.com		keywords=
SearchEngine	americaonline.com.mx	q=
SearchEngine	ananzi.co.za		qt=
# answers has this url http://www.answers.com/definition%20of%20intractive%20computing
####SearchEngine	answers.com  definition
SearchEngine    antisearch.net  	KEYWORDS=
SearchEngine	aolbusqueda.		query=
SearchEngine    aolsearch.       	query=
# aol. also used query= ?
SearchEngine    busca.aol.com.		query=
SearchEngine    busqueda.aol.com.mx	query=
SearchEngine	aol.			q=
SearchEngine	busca.aonde.com.br	q=
SearchEngine	busca.aonde.com		keys=
SearchEngine	aport.ru		r=
SearchEngine	arabianlist.com		keywords=
# arama.comis turkish
SearchEngine	arama.com		q=
SearchEngine	arcor.de		Keywords=
# ask.com used ask=
SearchEngine    ask.com 		q=
# ask.com used ask= .Also uses oq= (former query)
SearchEngine    ask.co.uk       	q=
# askgeeves merged into ask.com
SearchEngine	askgeeves.com		ask=
# askjeeves merged into ask.com
SearchEngine	askjeeves.		ask=
SearchEngine	atlas.cz		searchtext=
SearchEngine	atomz.com		sp-q=
SearchEngine	atsearch.at		qs=
SearchEngine	att.net			qry=
SearchEngine	attbi.com		qry=
SearchEngine    augurnet.ch     	q=
SearchEngine	austria.at		query=
SearchEngine	austria-seek.at		Terms=
SearchEngine	austria-www.at		sb1=
SearchEngine	austronaut.at		begriff=
SearchEngine	auyantepui.com		clave=
SearchEngine	av.com			q=
SearchEngine	avantfind.com		Keywords=
SearchEngine	a-site.at		query=
SearchEngine	babieca.com		keywords=
SearchEngine    search.backflip.com	query=
# baidu also used word=
SearchEngine	image.baidu.com		word=
SearchEngine	mp3.baidu.com		word=
SearchEngine	post.baidu.com		kw=
SearchEngine    baidu.com       	wd=
SearchEngine    barrahome.org   	query=
# batoota is arabic
SearchEngine	batoota.com		search_string=
SearchEngine	bbc.co.uk		q=
SearchEngine	bedrane.com		q=
SearchEngine	bellsouth.net/s/s.dll	string=
SearchEngine	search.biglobe.ne.jp	q=
SearchEngine	biglotron.com		q=
SearchEngine	search.bigmir.net	q=
SearchEngine    blogdex.net     	q=
# blogdigger used queryString=
SearchEngine    blogdigger.com  	q=
SearchEngine    blogosphere.us  	s=
SearchEngine    blogmatrix.com  	search=
SearchEngine    blogwise.com    	query=
SearchEngine	bloogz.com		q=
SearchEngine	blowsearch.com		qkw=
SearchEngine	blueyonder.co.uk	q=
SearchEngine	bodenseeinformation.ch	search1=
# boitho is norwegian (also english)
SearchEngine    boitho.com      	query=
SearchEngine	miner.bol.com.br	q=
SearchEngine	boowa.com		kw=
# brisbane has url brisbane.t-online.de/fast-cgi/tsc
SearchEngine	brisbane.t-online.de	q=
SearchEngine	browserwise.		Terms=
###SearchEngine	askbt.bt.com (does not give search info)
SearchEngine	btopenworld.		query=
SearchEngine	bulkfeeds.net		q=
SearchEngine	buscador.com		s_lookfor=
SearchEngine	buscadorespanol.com	keyword=
##### buscafox puts search word in referrer http://buscafox.com/griho
# buscamelo is spanish
SearchEngine    buscamelo.com		keywords=
SearchEngine	business.com		query=
SearchEngine    by.com  		query=
SearchEngine	c4.com			SearchText=
SearchEngine	www.cabtfe.com		busqueda=
SearchEngine	recherche.canoe.com	q=
SearchEngine	ceek.jp			q=
SearchEngine	centrum.cz		q=
# home.fra.chello.be and home.nld.chello.be used keywords=
SearchEngine	home.fra.chello.be	srchText=
SearchEngine	home.nld.chello.be	srchText=
SearchEngine	home.nor.chello.no	srchText=
SearchEngine	clusty.com		query=
SearchEngine	websearch.cnn.com	query=
SearchEngine	comcast.net		query=
# cometsystems used query=
SearchEngine	cometsystems.com	qry=
SearchEngine	cissearch.compuserve.com	Term=
SearchEngine	oolwebsearch.com	qq=
SearchEngine	pesquisa.clix.pt	question=
SearchEngine	club-internet.fr	key=
SearchEngine	crawler.com		qry=
SearchEngine	eureka.creativeweb.com	clau=
SearchEngine	cromas.com		Page=
SearchEngine	websearch.cs.com	query=
SearchEngine	cyberbritain.co.uk	qry=
SearchEngine	www.cyler.com		query=
SearchEngine	danielsen.com		q=
SearchEngine	dataseeq.com		q=
SearchEngine    daypop.com      	q=
SearchEngine	dealtime.com		KW=
SearchEngine	deja.com		AN=
SearchEngine	departeclair.com	q=
SearchEngine	sw.detelefoongids.nl	q=
SearchEngine	devilfinder.com		q=
SearchEngine	dino-online.de		query=
SearchEngine    dir.com 		req=
SearchEngine	directhit.com		qry=
SearchEngine    dmoz.org        	search=
# dogpile now uses http://www.dogpile.com/info.dogpl/search/web/convertir%2Barchivos%2Blit%2Btexto
SearchEngine    dogpile.com     	q=
SearchEngine	dpxml			qkw=
SearchEngine	doutorbusca.com.br	search=
SearchEngine	search.earthlink.	q=
SearchEngine	edderkoppen.dk		query=
SearchEngine	.edintorni.		q=
SearchEngine    egoto.com       	keywords=
# eicus uses POST
#####SearchEngine	eicus.info		
SearchEngine	www.encis.es		query=
SearchEngine    elf8888.at      	query0=
SearchEngine	ariadna.elmundo.es	q=
# spanish
SearchEngine	encuentralo.com		search=
SearchEngine	eniro.se		q=
# eon.si has been bought by matkurja.com
SearchEngine    eon.si/portal/iskanje.asp       KljucneBesede=
# .eoob. uses POST
####SearchEngine	.eoob.		/search
SearchEngine	epilot.com		k=
SearchEngine	eresmas.com		q=
# spotting affiliates use very long URLs, so probably
#   their keywords are not recorded by webalizer
SearchEngine	espotting.com		keyword=
############# exalead hides queries
##########SearchEngine	exalead.com		
SearchEngine    eureka.com      	q=
SearchEngine	everyday.com		query=
SearchEngine	evreka.com		q=
SearchEngine	evreka.passagen.se	q=
# euroseek now uses string=
SearchEngine	euroseek.		query=
SearchEngine	eurkester.com		w=
SearchEngine	evrsoft.com		search=
SearchEngine	excite.at		key=
SearchEngine	excite.ch		key=
SearchEngine	excite.com		qkw=
SearchEngine	excite.co.jp		search=
SearchEngine	excite.de		q=
SearchEngine	excite.es		q=
SearchEngine	excite.fr		qkw=
SearchEngine	msxml.excite.com	qkw=
# not all excite engines use search=, so more entries should be added for each country, maybe has changed to qkw
SearchEngine	excite.			search=
SearchEngine	ez2find.com		qry_str=
#########SearchEngine	.ezanga.com		/search/web/
# feedster is a RSS search engine
SearchEngine    feedster.com    	q=
SearchEngine	find-a-needle.com	Keywords=
SearchEngine	findarticles.com	key=
SearchEngine	findwhat.com		mt=
SearchEngine	fireball.de		q=
SearchEngine	firehunt.com		Keywords=
SearchEngine	find.dk			words=
# francite used KEYWORDS= before
SearchEngine	francite.com		name=
SearchEngine	freeball.de		q=
SearchEngine	freenet.de		query=
SearchEngine	freeserve.		q=
SearchEngine	freeweb.hu		KERESES=
SearchEngine	free.fr			q=
SearchEngine	fresheye.com		kw=
SearchEngine	freshmeat.net		q=
SearchEngine	ftxt.quick.cz		query=
SearchEngine	furttalweb.ch		query=
SearchEngine    gais.cs.ccu.edu.tw      q=
SearchEngine    galaxy.com      	k=
SearchEngine	szukacz.gazeta.pl	tekst=
SearchEngine	geeves.com		ask=
SearchEngine    gigablast.com   	q=
SearchEngine	gigabusca.teta.8x.com.br	query=
SearchEngine	gimenei.com		qry_str=
SearchEngine	zoom.globo.com		q=
SearchEngine	busca.globo.com		q=
SearchEngine	busca.zoom.globo.com	q=
# go.com used qt= before
SearchEngine	go.com			q=
# go2net before it used general=
SearchEngine	go2net			query=
SearchEngine	goeureka.com.au		q=
SearchEngine	gogole			q=
SearchEngine	gojester.com		str=
SearchEngine	goliat.hu		KERESES=
### no need to add engines that have google. in the name and use q=, as they are all catched here
#SearchEngine	google.at		q=
#SearchEngine	google.be		q=
#SearchEngine	google.ch		q=
#SearchEngine	google.co.jp		q=
#SearchEngine	google.co.kr		q=
#SearchEngine	google.co.uk		q=
#SearchEngine	google.com.br		q=
#SearchEngine	google.de		q=
#SearchEngine	google.es		q=
#SearchEngine	google.fr		q=
SearchEngine    google.  		q=
SearchEngine    goo.ne.jp       	MT=
SearchEngine	goto.com		Keywords=
SearchEngine	gotonet.		q=
SearchEngine	gohip.com		sc=
SearchEngine	goshdarn.com		query=
# grippo now uses custom google
SearchEngine	grippo.com.ar		words=
SearchEngine	haku.suomi24.fi		query=
SearchEngine	haku.www.fi		mt=
SearchEngine	hanafos.com		query=
SearchEngine	heureka.hu		heureka=
SearchEngine	buscamundo.com		qu=
SearchEngine	buscar.hispavista.com	cadena=
SearchEngine	highindex.com		q=
SearchEngine	hoga.pl			qt=
SearchEngine	go.hopala.ru		q=
# hotbot before used MT=
SearchEngine	hotbot.			query=
SearchEngine	htsearch.		words=
## google.icq.com is already matched by google.
##SearchEngine	google.icq.com		q=
SearchEngine	icq.com			q=
##### icispicy hides queries SearchEngine	icispicy.com		
SearchEngine	ifind.freeserve		q=
SearchEngine	ilmotore.com		query=
SearchEngine	ilse.nl			search_for=
# index.hu now uses search.php3
SearchEngine	index.hu		KERESES=
SearchEngine	infoclic.fr		iFullTexQuery=
SearchEngine	infoseek.co.jp		qt=
SearchEngine	infoseek.de		qt=
# infoseek used qt=
SearchEngine    infoseek.	    	q=
# infospace uses (qh=|qn=|qhqn=|qname=|qkw=|/search/web/|/search/images/|search/audio/|search/multimedia|search/news)
SearchEngine	infospace.com		qkw=
SearchEngine	infotiger.com		qs=
SearchEngine	cab.infoweb.ne.jp	q=
SearchEngine	ingrid.org		key=
# interbusca also uses s=
SearchEngine	.interbusca.com		s=
# interia.pl also matches szukaj.interia.pl
SearchEngine	interia.pl		q=
SearchEngine	internaz.com		str=
# internetto used also searchstr= (disappeared in 2003)
SearchEngine	internetto.hu		query=
SearchEngine	interneto.fr		Query=
SearchEngine	search.iol.ie		q=
SearchEngine	isize.com		QueryString=
SearchEngine	ithaki.net		query=
SearchEngine    ixquick.com     	query=
SearchEngine	iwon.			searchfor=
SearchEngine	jarbas.com.br		qry=
# jayde is german
SearchEngine	jayde.com		query=
# jet2web is german
SearchEngine	jet2web.net		query=
SearchEngine	jubii.dk		soegeord=
#SearchEngine	juppyn.com		/q/
SearchEngine	jword.jp		qt=
SearchEngine	jyxo.cz			s=
SearchEngine	kanoodle.com		query=
SearchEngine	kartoo.com		q=
# katatudo is brazilian
SearchEngine	katatudo.		q=
SearchEngine	kensaku.org		key=
# km is russian
SearchEngine	search.km.ru		sq=
SearchEngine    kobala.nl       	qr=
SearchEngine	bach.istc.kobe-u.ac.jp	q=
SearchEngine	kvasir.no		q=
SearchEngine	lapkereso.hu		key=
SearchEngine	latinguia.com		query=
SearchEngine	lawcrawler.findlaw.com	entry=
SearchEngine	search.li.ru		search_query=
SearchEngine	arianna.libero.it	query=
# libros-directorio.com may be a spammer
SearchEngine	libros-directorio.com	qry_str
SearchEngine	linkspider.co.uk	query=
SearchEngine	list2000.com		qry=
SearchEngine	livedoor.		q=
SearchEngine	search.liveinternet.ru	search_query=
SearchEngine    look.com        	q=
# looksmart also uses q=
SearchEngine    looksmart.       	key=
# info.lookupanything.net is related to lookupanything.biz but I have no info	
SearchEngine	hit.lookupanything.biz	Keywords=
SearchEngine	lop.com			s=
#SearchEngine	lycos.at		query=
#SearchEngine	lycos.ch		query=
SearchEngine	lycos.co.jp		q=
#SearchEngine	lycos.de		query=
#SearchEngine	lycos.fr		query=
#SearchEngine	lycos.es		query=
SearchEngine    lycos.			query=
SearchEngine    mamma.			query=
SearchEngine	manitoo.ch		Terms=
SearchEngine	maloop.com		keyword=
# matkurja.com matkurja.com/bin/search matkurja.com/slo/search
SearchEngine	matkurja.com		keys=
SearchEngine	meceoo.fr		kw=
SearchEngine	ozu.mhmujer.com		q=
SearchEngine	metabot.ru		st=
SearchEngine	metacherche.com		query=
# metacrawler used general= before
# metacrawler uses now search/web/ search/images/ search/audio/ search/multimedia/ search/news/
SearchEngine    metacrawler.de     	qry=
SearchEngine    metacrawler.     	q=
SearchEngine	metaeureka.com		terms=
SearchEngine	metamoteur.net		mots=
SearchEngine	metaspinner.com		qry_str=
SearchEngine	metaspinner.de		qry=
# mirago used txtSearch= before
SearchEngine	mirago.co.uk		qry=
SearchEngine	mirago.			txtsearch=
SearchEngine	nlsearch.		qr=
SearchEngine	meidor.			mei=   
SearchEngine	monstercrawler.com	qry=
SearchEngine	montrealplus.ca		query=
SearchEngine	moonmist.info		q=
SearchEngine	mosaicfx.com		qw=
SearchEngine	mozdex.com		query=
# msn used MT before. Using only "msn." you would be matching also yupimsn.com and search.xtramsn.co.nz
SearchEngine	/spbasic.htm		MT=
#SearchEngine	search.msn.com/spbasic.htm	MT=
#SearchEngine	ninemsn.com.au/spbasic.htm	MT=
#SearchEngine	search.ninemsn.com.au/results.aspx	q=
#SearchEngine	msn.at			q=
#SearchEngine	msn.ch			q=
#SearchEngine	msn.co.jp		q=
#SearchEngine	msn.co.nz		q=
#SearchEngine	msn.de			q=
#SearchEngine	msn.com 		q=
SearchEngine	msn.	 		q=
SearchEngine    msxml.infospace.com	qkw=
SearchEngine	buscadores.multibuscador.com	keywords=
SearchEngine	munky.com		term=
# mygeek hides the keywords
#SearchEngine	mygeek.com		url=
SearchEngine	myrice.com		s=
SearchEngine    mysearch.com    	searchfor=
SearchEngine	myway.com		searchfor=
SearchEngine	mywebsearch.com		searchfor=
SearchEngine	mx.search.yahoo.com	va=
SearchEngine	mweb.co.za		hpq=
SearchEngine	najdi.com		dotaz=
SearchEngine	najdi.si		q=
SearchEngine	nandosearch.com		query=
SearchEngine    naver.com       	query=
SearchEngine	naver.co.jp		query=
SearchEngine	nbci.com		keyword=
# need2find also uses ask=
SearchEngine	need2find.com		searchfor=
SearchEngine	guru.pi.net.pl		PHRASE=
SearchEngine	kapi.netbul.com		keyword=
SearchEngine	netfind.		query
SearchEngine	netoskop.pl		query=
# netscape used query= before
SearchEngine	suche.netscape.de	q=
SearchEngine	netscape.fr		query=
SearchEngine    netscape.com    	search=
SearchEngine	netsprint.pl		qt=
SearchEngine	netster.com		Keywords=
# nifty.com is japanese
SearchEngine	nifty.com		Text=
SearchEngine	ninemsn.com.au		q=
SearchEngine	newhoo.			search=
SearchEngine	nomade.fr		s=
SearchEngine    northernlight.com       qr=
SearchEngine    ntlworld.com    	q=
#labs.nttrd.com is japanese
SearchEngine	labs.nttrd.com		q=
SearchEngine	nusearch.com		nusearch_terms=
SearchEngine	odn.ne.jp		QueryString=
SearchEngine	ofir.dk			querytext=
# oingo was temporaly a site who faked searchs to get money. searchs from there are misleading. leave commented
#SearchEngine	oingo.com	s=
# onet.pl also matches szukaj.onet.pl
SearchEngine	onet.pl			qt=
# openfind used q= before
SearchEngine    openfind        	query=
SearchEngine	opasia.dk		q=
SearchEngine	orbis.dk		querytext=
#overture hides search strings when searching through partners
SearchEngine    overture.		Keywords=
# ozu.es uses also , as_epq=, as_oq=, &as_eq=, as_filetype=, as_sitesearch=
SearchEngine	buscador.ozu.es		as_q=griho
SearchEngine	passagen.se		q=
# peerbot is a favicon search engine
SearchEngine	peerbot.com		q=
SearchEngine	search.peoplepc.com	q=
# paysefeed is used heavily by spammers
SearchEngine	paysefeed.com		q=
SearchEngine	perfectnav.com		keywords=
SearchEngine	pichunter.com		query=
SearchEngine    picsearch.com   	q=
SearchEngine	pingwin.ru		searchword=
SearchEngine	planetclick.com		T=
SearchEngine    popdex  		query=
SearchEngine	pricegrabber.com	form_keyword=
SearchEngine	profusion.com		queryterm=
SearchEngine	punto.ru		text=
SearchEngine	pursuit.		query=
SearchEngine	quebecplus.ca		query=
SearchEngine    quepasa.com     	q=
SearchEngine	quicklinks.ch		MOT_CLE=
# hp.qsrch uses also query=
SearchEngine	hp.qsrch.com		s=
SearchEngine	newnet.qsrch.com	Keywords=
SearchEngine	qsrch.com		query=
# qualigo is german/nederlands
SearchEngine	qualigo.		q=
SearchEngine	radaruol.uol.com.br	q=
SearchEngine	radarweb.com.br		q=
SearchEngine	radio-canada.ca		chaineRecherche=
SearchEngine	randomtaxi.com		q=
SearchEngine	rambler.ru		words=
SearchEngine	raziskovalec.com	query=
SearchEngine	redbox.cz		srch=
SearchEngine	reference.ch		motclef=
SearchEngine	reference.com		q=
SearchEngine	rdn.ac.uk		query=
SearchEngine	blitzsuche.rp-online.de	q=
SearchEngine	pesquisa.sapo.pt	q=
SearchEngine	scrubtheweb.com		keyword=
SearchEngine	sear.ch			qry=
SearchEngine	search.ch		q=
# search.com used qt= before
SearchEngine	.search.com		q=
SearchEngine	search.devx.com		q=
SearchEngine	search.seeder.net	key=
SearchEngine	search123.com		query=
# searchalot used query=
SearchEngine	searchalot.		q=
SearchEngine	searchco.com		term=
SearchEngine	searchengine.com	search=
SearchEngine	searchengine.net	keyword=
SearchEngine	searchhippo.com		q=
# searchfedd is PPC Pay Per Click
SearchEngine	searchfeed.com		k=
SearchEngine	searchforit.com		q=
# searchking used at=
SearchEngine	searchking.com		keyword=
SearchEngine	search.opera.com	
SearchEngine	searchscout.com		k=
SearchEngine    searchspider.com        q=
SearchEngine	searchy.co.uk		search_term=
SearchEngine	seeksall.com		keywords=
SearchEngine	serverpages.com		query=
SearchEngine	seznam.cz		w=
# sharelook used KEYWD before
SearchEngine	sharelook.		keyword=
# sina.com.cn uses also word=
SearchEngine	pic.sina.com.cn		query=
SearchEngine	search.sina.com.cn	_searchkey=
SearchEngine	search.sina.com.tw	word=
SearchEngine	search.sina.com.hk	word=
SearchEngine	singingfish.com		query=
SearchEngine	sitedebusca.com		busca=
SearchEngine	skynet.be		keywords=
SearchEngine    snap.com		keyword=
#sogou and sohu are chinese
SearchEngine    image.search.sohu.com   keyword=
SearchEngine	mp3.search.sohu.com	word=
SearchEngine	news.sogou.com		word=
SearchEngine	shopping.sogou.com	query=
SearchEngine	search.sohu.com		query=
SearchEngine	wap.sogou.com		word=
SearchEngine	sohu.com		query=
SearchEngine	sok.start.no		q=
SearchEngine	sol.dk			q=
SearchEngine	search.sol.no		query=
SearchEngine	searchmeup.com		q=
SearchEngine	searchwatch.net		q=
SearchEngine	sova.com.ua		q=
SearchEngine	splut.			pattern=
# spotting only shows the keyword when searching from an affiliate
SearchEngine	affiliate.spotting.fr	keyword=
SearchEngine	spotjockey.		Search_Keyword=
SearchEngine	spray.			string=
#SearchEngine	google.startsiden.no	q=
SearchEngine	search.starware.com	qry=
SearchEngine	sevenmetasearch.com	q=
SearchEngine	steadysearch.com	w=
SearchEngine	submitawebsite.com	search=
SearchEngine	suchbiene.de		SID=
SearchEngine	suchknecht.at		q=
SearchEngine    suchmaschine21.de       search=
SearchEngine	.web.de			su=
SearchEngine	surfear.com		surf=
SearchEngine    swissart.net		q=
# swissguide.ch uses also search_canton=
SearchEngine	swissguide.ch		search_who_or_what=
SearchEngine	swissone.com		REQ=
SearchEngine	swobee.com		q=
SearchEngine    syndic8 		ShowMatch=
SearchEngine	sympatico.ca		query=
SearchEngine	szukaj.pl		q=
# t-online.de uses also qu=
SearchEngine	t-online.de		q=
SearchEngine	tay.com.br		por=
SearchEngine	find.tdconline.dk	q=
SearchEngine    technorati      	query=
SearchEngine	search.tele2internet.fr	query=
SearchEngine    teensearch      	query=
SearchEngine	tegnher.com.br		q=
# teoma used t= before
SearchEngine    teoma.com       	q=
SearchEngine    teradex.com     	q=
SearchEngine	click.terespondo.com	kw=
SearchEngine	terra.			query=
SearchEngine	terravista.pt		query=
SearchEngine    texis.			q=
SearchEngine	tf1.fr			query=
SearchEngine    thunderstone.com	q=
SearchEngine	tinusi.			term=
SearchEngine	tiscali.fr		s=
SearchEngine	tocc.co.jp		QRY=
SearchEngine	todalanet.com		query=
# todobr.com.br hides the search string
#####SearchEngine	todobr.com.br		
SearchEngine	toile.com		q=
SearchEngine	toile.qc.ca		query=
SearchEngine	search.tom.com		word=
SearchEngine    topclick.com		Q= 
SearchEngine	tricus.com		q=
SearchEngine	trouvez.com		query=
SearchEngine	trustedsearch.net	w=
SearchEngine	tucherches.ch		query=
SearchEngine    turkfree.com		keywords=
SearchEngine	tutorgig.com		keywords=
SearchEngine	ubbi.			q=
SearchEngine	msn.directhit.com	q=
SearchEngine	ukdirectory.		k=
SearchEngine	ukindex.co.uk		stext=
# ukplus also Search=
SearchEngine	ukplus.			key=
SearchEngine	usseek.			string=
SearchEngine	busca.uol.com		q=
SearchEngine	buscar.uol.com		q=
SearchEngine	verizon.net		qkw=
SearchEngine	zoek.vinden.nl		query=
SearchEngine	vindex			in=
SearchEngine	virgilio.it		qs=
SearchEngine	gps.virgin.net		q=
SearchEngine	vivisimo.com		query=
SearchEngine	vizsla.origo.hu/katalogus	q=
SearchEngine	vizsla.origo.hu		search=
# some instances of voila use kw=
SearchEngine	voila.fr		kw=
SearchEngine    voila.			q=
SearchEngine	vonna.com		k=
SearchEngine	wahoo.hu		q=
SearchEngine    walhello.        	key=
# busca.wanadoo.es/search  doesn't show the searched string
SearchEngine	search.warez.com	q=
SearchEngine    waypath.com     	key=
SearchEngine	web.de			su=
SearchEngine	webcrawl.net		q=
# webcrawler now uses /search/web/
SearchEngine	webcrawler.de		qkw=
SearchEngine    webcrawler.      	searchText=
SearchEngine	webfanatic.lunarpages.com	q=
# websearch also uses search/images/keyword, search/news/keyword and search/web/keyword
SearchEngine	websearch.com		qkw=
SearchEngine	webservicehost.com	s=	
# webtop used also QUERY= (it seems to be dead now)
SearchEngine    webtop.com		P=
SearchEngine	webwatch.be		findindb=
SearchEngine	webwizzard.at		q=
SearchEngine	webwombat.com.au	i=
SearchEngine    whatuseek.com		query=
SearchEngine    whois.sc        	q=
SearchEngine	why-it.se		keywords=
SearchEngine	windseek.com		q=
SearchEngine    wisenut.com     	q=
SearchEngine	wotbox.com		q=
SearchEngine	wow.pl			q=
# wp.pl also matches szukaj.wp.pl
SearchEngine	wp.pl			szukaj=
# www.ru uses also s2f=
SearchEngine	www.ru			Str_Find=
SearchEngine	x-recherche.com		MOTS=
SearchEngine	xasa.com		query=
SearchEngine	xupiter.com		Terms=
# ya.com spanish uses also q_phrase=
SearchEngine	buscar.ya.com		q=
SearchEngine    buscador.ya.com 	q=
# we put yp.yahoo (yellow pages) before the other yahoo entries so it gets matched properly
SearchEngine	crawl.yahoo.com p=
SearchEngine	crawl.yahoo.net q=
SearchEngine	avatw.search.yahoo2.akadns.net	q=
SearchEngine	yandex.ru		text=
SearchEngine	yasearch.net		qq=
SearchEngine	yellowpages.con.au	q=
SearchEngine    yoodle.ch		q=
SearchEngine	your-web-search.	qq=
SearchEngine	search.zahav.ru		q=
SearchEngine    zensearch.com		q=
SearchEngine	.zhongsou.com		word=
# zworks uses (what|what_1|what_2|what_3|what_4|what_5|what_6|what_7|what_8|what_9)
SearchEngine	zworks.com		what=
# catches other engines not in the list
SearchEngine	search			q=
# motor for some search engines
SearchEngine	engine.exe		p1=
EOF

# Normally, search strings are converted to lower case in order to
# increase accuracy.  The SearchCaseI option allows them to maintain
# case sensitivity, useful for some sites.  The value can be 'yes'
# or 'no', with 'yes' (case insensitive) being the default.
#SearchCaseI    yes

# The Dump* keywords allow the dumping of Sites, URLs, Referrers
# User Agents, Usernames and Search strings to seperate tab delimited
# text files, suitable for import into most database or spreadsheet
# programs.

# DumpPath specifies the path to dump the files.  If not specified,
# it will default to the current output directory.  Do not use a
# trailing slash ('/').
#DumpPath   /var/www/logs
#echo "DumpPath   /var/www/logs"

# The DumpHeader keyword specifies if a header record should be
# written to the file.  A header record is the first record of the
# file, and contains the labels for each field written.  Normally,
# files that are intended to be imported into a database system
# will not need a header record, while spreadsheets usually do.
# Value can be either 'yes' or 'no', with 'no' being the default.
#DumpHeader no
echo "DumpHeader      yes"

# DumpExtension allow you to specify the dump filename extension
# to use.  The default is "tab", but some programs are pickey about
# the filenames they use, so you may change it here (for example,
# some people may prefer to use "csv").
#DumpExtension  tab
echo "DumpExtension   html"

# These control the dumping of each individual table.  The value
# can be either 'yes' or 'no'.. the default is 'no'.
#DumpSites  no
#DumpURLs   no
#DumpReferrers  no
#DumpAgents no
#DumpUsers  no
#DumpSearchStr  no

echo "DumpSites       yes"
echo "DumpURLs        yes"
echo "DumpReferrers   yes"
echo "DumpAgents      yes"
echo "DumpUsers       yes"
echo "DumpSearchStr   yes"

# The custom graph colors are defined here. Declare them
# in the standard hexadecimal way (as HTML, without the '#')
# If none are given, you will get the standard default colors.
#ColorHit   00805c
#ColorFile  0040ff
#ColorSite  ff8000
#ColorKbyte ff0000
#ColorPage  00e0ff
#ColorVisit ffff00
#ColorMisc      00e0ff

#PieColor1  800080
#PieColor2  80ffc0
#PieColor3  ff00ff
#PieColor4  ffc080

# End of configuration file...  Have a nice day!
}


# ----------------------------------------------------------------------------
# Main
# ----------------------------------------------------------------------------

mkdir -p /etc/webalizer
rm -f /etc/webalizer/webalizer_*.conf

# index no. for all config files 
conf_idx=1

idx=1
while [ "$idx" -le "$WEBALIZER_HOST_N" ]
do
    eval accesslog='$WEBALIZER_HOST_'$idx'_ACCESS_LOG'
    eval output_dir='$WEBALIZER_HOST_'$idx'_OUTPUT_DIR'
    eval host_name='$WEBALIZER_HOST_'$idx'_HOST_NAME'
    eval title='$WEBALIZER_HOST_'$idx'_TITLE'
    eval bgcolor='$WEBALIZER_HOST_'$idx'_BGCOLOR'
    eval type='$WEBALIZER_HOST_'$idx'_TYPE'

    [ ! -f $accesslog ] && echo "WARNING: The file '$accesslog' does not exist!" 

    mkdir -p ${output_dir}
    chmod 0750 ${output_dir}
    chown -R root:apache ${output_dir}

    create_webalizer_configuration > /etc/webalizer/webalizer_${conf_idx}.conf

    idx=`expr $idx + 1`
    conf_idx=`expr $conf_idx + 1`    
done


if [ "$WEBALIZER_VHOSTS_RUN_ALL" = 'yes' ] ; then 
    idx=1
    while [ "$idx" -le "$APACHE2_VHOST_N" ]
    do
        eval active='$APACHE2_VHOST_'$idx'_ACTIVE'
        if [ "$active" != "yes" ] ; then
            idx=`expr $idx + 1`
            continue
        fi
        eval servername='$APACHE2_VHOST_'$idx'_SERVER_NAME'
        eval docroot='$APACHE2_VHOST_'$idx'_DOCUMENT_ROOT'
        eval accesscontrol='$APACHE2_VHOST_'$idx'_ACCESS_CONTROL'
        accesslog="/var/log/apache2/access-${servername}.log"

        # first remove the trailing slash
        doc_root=`echo $docroot | sed -e 's|/$||'`
        case "$WEBALIZER_VHOSTS_OUTPUT_DIR" in
            *VHOST_DOCROOT*)
                output_dir=`echo $WEBALIZER_VHOSTS_OUTPUT_DIR | sed "s|%VHOST_DOCROOT%|${doc_root}|"`
                ;;
            *SERVER_NAME*)
                output_dir=`echo $WEBALIZER_VHOSTS_OUTPUT_DIR | sed "s|%SERVER_NAME%|${servername}|"`
                ;;
            *)
                output_dir="$WEBALIZER_VHOSTS_OUTPUT_DIR"
                ;;        
        esac
        host_name="$servername"
        title="$WEBALIZER_VHOSTS_TITLE"
        bgcolor="$WEBALIZER_VHOSTS_BGCOLOR"
        type='clf'

        [ ! -f $accesslog ] && echo "WARNING: The file '$accesslog' does not exist!"

        mkdir -p ${output_dir}
        chmod 0750 ${output_dir}
        chown -R root:apache ${output_dir}

        create_webalizer_configuration > /etc/webalizer/webalizer_${conf_idx}.conf

        idx=`expr $idx + 1`
        conf_idx=`expr $conf_idx + 1`
    done
fi


if [ "$START_WEBALIZER" = "yes" ] ; then
    mkdir -p /etc/cron/root
    echo "$WEBALIZER_CRON /usr/bin/webalizer-run cron" >/etc/cron/root/webalizer
    rc-service --quiet fcron reload
else
    rm -f /etc/cron/root/webalizer
    rc-service --quiet fcron reload
fi

exit 0
